/* Main program for Melangeur Version 1.0 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <melangeur.h>
#include <assert.h>

/* global variables */
char labelfilename[NAMELEN]; /* labels file name*/
char resultfilename[NAMELEN]; /* results file name*/
int nbtests;           /* runs number to do */
int seqlen;            /* sequence length */

/*****************************************************************************/
/* MAIN : get a labels file as entry and return a new shuffled labels file   */
/* input:	  infile : labels file                                       */
/* input/output:  outfile : mixed labels file                                */
/*****************************************************************************/

int main(int argc, char **argv)
{
  /* argc is the number of arguments */
  /* argv are the arguments...       */
  
  int i;
  char **seq;
  int seqInt[MAXSEQLEN+1];
    
  /*initialize settings -- default settings*/
  nbtests = 1; /* if you want to display more than one test */
  strcpy(resultfilename,"./tmp/res"); 
  strcpy(labelfilename,"./tmp/labels_reaction_graph.txt");
  seqlen=-1;

  if (!parse(argc,argv)) {
    return 0;
  }
    
  if (seqlen>=0) {
    
    /****** if the labels number is given ******/
    if (seqlen <= MAXSEQLEN) {
      for (i=0; i<seqlen; i++) {
        seqInt[i] = i+1;   
      }
      
      /******** Boucle principale ******/
      if (nbtests==1)
        {       
          generateseqInt(seqlen,seqInt);
          writeLabelFileInt(resultfilename,seqInt,seqlen);
        }
      else {
        for(i=1; i<=nbtests; i++ )
          {
            char resfile[NAMELEN];    
            generateseqInt(seqlen,seqInt);
            sprintf(resfile,"%s%i",resultfilename,i);
            writeLabelFileInt(resfile,seqInt,seqlen);
          }
      }
    } else {
      printf("The length %i is too high (> %i).\n", seqlen, MAXSEQLEN);
      return 0;
    }
  }
  
  else {
    
    /******** Allocations necessaires ******/
    seq = (char **) malloc(LMAX*sizeof(char*));
    for (i=0; i<LMAX; i++)
      {
        seq[i]  = (char *) malloc((MAX_CHAR_PER_LINE)*sizeof(char));  
        assert(seq[i]!=NULL);     
      }
  
    /******* Ouverture du fichier de labels ******/
    seqlen = openLabelFile(labelfilename,seq);
    if (seqlen==0) {
      printf("The file %s doesn't exist\n", labelfilename);
      return 0;
    }
 
    /******** Boucle principale ******/
    if (nbtests==1)
      {       
        generateseq(seqlen,seq);
        writeLabelFile(resultfilename,seq,seqlen);
      }
    else {
      for(i=1; i<=nbtests; i++ )
        {
          char resfile[NAMELEN];    
          generateseq(seqlen,seq);
          sprintf(resfile,"%s%i",resultfilename,i);
          writeLabelFile(resfile,seq,seqlen);
        }
    }
    
    /***** Liberations ******/
    for(i=0; i<LMAX; i++)
      free(seq[i]);
    free(seq);
    
  }

  /*printf("Done\n");*/
  return 1;
    
}


/******************************************************/
/* parse : parses string passed as argument and       */
/*           correspondingly sets the class variables */
/*         If specified arguments aren't complete,    */
/*           default values are set.                  */
/* input:         argc : result value to write        */
/* input:         argv : sequence length              */
/******************************************************/
int parse(int argc, char** argv) 
{
  int i;
  /*if (argc<=1) 
    {
    printf("ERROR: no command-line arguments were given\n");
    printHelp();
    }
    else {*/
  for (i=1; i<argc; i++) {
    if (!strcmp("-h", argv[i])) 
      {
        printHelp();
        return 0;
      }
    else if (!strcmp("-l", argv[i]))
      {
        strcpy(labelfilename, argv[++i]);
      }
    else if (!strcmp("-r", argv[i]))
      {
        strcpy(resultfilename, argv[++i]);
      }
    else if (!strcmp("-t", argv[i]))
      {
        nbtests= atoi(argv[++i]);
      }
    else if (!strcmp("-n", argv[i]))
      {
        seqlen= atoi(argv[++i]);
      }     
    else 
      {
        printf("%s : unrecognized argument: %s\n", argv[0], argv[i]);
        printHelp();
        return 0;
      }
  }
  return 1;
  /*}*/
}


/******************************************************/
/* printHelp : writes the help to use melangeur       */
/******************************************************/
void printHelp() {
  printf("Melangeur is a program that shuffles a label file.\n");
  printf(" USAGE :\n");
  printf("    labels_file -l: name of the file containing the labels of your graph.\n");
  printf("                    Default: tmp/labels_reaction_graph.txt.\n");
  printf("    results_file -r: name of the file in which the results of the query are stored.\n");
  printf("                     Default: tmp/res.\n");
  printf("    tests_number -t: number of runs to do.\n");
  printf("                     Default: 1.\n");
  printf("    labels_number -n: number of labels. Can't be use with a labels file.\n");
  printf("                     Default: 0.\n");
  
}
