/* Melangeur Version 1.0 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <melangeur.h>
#include <assert.h>


/******************************************************/
/* generateseq : create a new random sequence         */
/* input:        seqlen: sequence length              */
/* input/output: seq: random sequence to shuffle      */
/******************************************************/
void generateseq(int seqlen, char **seq)
{
  char seedfilename[NAMELEN]; /* file path for random generating seed */
  
  /* initialize random number generator by reading seed from file*/
  sprintf(seedfilename,"%s/%s",WDIR,SEEDFILENAME);
  readseed(seedfilename); 

  /* produce shuffled sequence */
  monoshuffle(seqlen,seq);

  /* store seed for random number generator */
  writeseed(seedfilename);

}


/******************************************************/
/* monoshuffle: do simple 1-let permutation of seq    */
/* input: seqlen: sequence length                     */
/* input/output: seq: random sequence to shuffle      */
/******************************************************/
void monoshuffle(int seqlen, char **seq)
{
  int i, j;
  char *temp;

  for (i = 0; i < seqlen; i++)
    {
      /* choose a random position j>i to swap */
      j = i + randomint(seqlen-i) - 1;
      temp = seq[i];
      seq[i] = seq[j];
      seq[j] = temp;
    }
}
/******************************************************/
/* generateseqInt : create a new random sequence         */
/* input:        seqlen: sequence length              */
/* input/output: seq: random sequence to shuffle      */
/******************************************************/
void generateseqInt(int seqlen, int *seq)
{
  char seedfilename[NAMELEN]; /* file path for random generating seed */
  
  /* initialize random number generator by reading seed from file*/
  sprintf(seedfilename,"%s/%s",WDIR,SEEDFILENAME);
  readseed(seedfilename); 

  /* produce shuffled sequence */
  monoshuffleInt(seqlen,seq);

  /* store seed for random number generator */
  writeseed(seedfilename);

}

/******************************************************/
/* generateseqInt2 : create a new random sequence         */
/* input:        seqlen: sequence length              */
/* input:        filename: path where you have the right to write   */
/* input/output: seq: random sequence to shuffle      */
/******************************************************/
void generateseqInt2(int seqlen, int *seq, char path[])
{
  char seedfilename[NAMELEN]; /* file path for random generating seed */
  
  /* initialize random number generator by reading seed from file*/
  sprintf(seedfilename,"%s/%s",path,SEEDFILENAME);
  readseed(seedfilename); 

  /* produce shuffled sequence */
  monoshuffleInt(seqlen,seq);

  /* store seed for random number generator */
  writeseed(seedfilename);

}




/******************************************************/
/* monoshuffleInt: do simple 1-let permutation of seq    */
/* input: seqlen: sequence length                     */
/* input/output: seq: random sequence to shuffle      */
/******************************************************/
void monoshuffleInt(int seqlen, int *seq)
{
  int i, j;
  int temp;

  for (i = 0; i < seqlen; i++)
    {
      /* choose a random position j>i to swap */
      j = i + randomint(seqlen-i) - 1;
      temp = seq[i];
      seq[i] = seq[j];
      seq[j] = temp;
    }
}



/********************************************************************/
/* openLabelFile : open the labels file                             */
/* input:   filename[] : labels file                                */
/*          seq : result value to write                             */
/* return value : the sequence length (0 if the file doen't exist)  */
/********************************************************************/
int openLabelFile(char filename[],char **seq)
{
  FILE *infile;
  int i;

  infile = fopen(filename, "r");
 
  if (infile == NULL)
  { 
     return 0;  
  }
  i=0;
  while(!feof(infile) && i < LMAX)
  {  
    /* On recupere la ligne courante du fichier */
    fgets(seq[i],MAX_CHAR_PER_LINE,infile);
    /* On incremente le compteur des abscisses */ 
    i++;
  }
  fclose(infile);
  return i;
   
}


/******************************************************/
/* writeLabelFile : write the new labels file         */
/* input/output:  filename[] : labels file            */
/* input:         seq : result value to write         */
/* input:         seqlen: sequence length             */
/* return value : 1 if OK                             */
/******************************************************/
int writeLabelFile(char filename[],char **seq, int seqlen)
{
  FILE *outfile;
  int i;

  outfile = fopen(filename, "w");
  for (i = 0; i < seqlen; i++)
    {
	fprintf(outfile, "%s", seq[i]);
    }
  fclose(outfile);
  return 1;
}


/******************************************************/
/* writeLabelFile : write the new labels file         */
/* input/output:  filename[] : labels file            */
/* input:         seq : result value to write         */
/* input:         seqlen: sequence length             */
/* return value : 1 if OK                             */
/******************************************************/
int writeLabelFileInt(char filename[],int *seq, int seqlen)
{
  FILE *outfile;
  int i;

  outfile = fopen(filename, "w");
  for (i = 0; i < seqlen; i++)
    {
      fprintf(outfile, "%i\n", seq[i]);
    }
  fclose(outfile);
  return 1;
}

