#include <stdio.h>

#define MULTIPLIER 69069
#define SHIFT          1
#define MODULUS    256*256*256*128
#define INVMOD     ( (double) 1 / ((double) MODULUS)) / ((double) 2)

#define DEFAULTSEED 1000001


static unsigned int seed;

/* changeseed **************************************************************************************************************

DESCRIPTION: initiate the static variable seed, used as seed value in functions generating random numbers

SIDE EFFECTS: the value of the static variable seed is changed

RETURN VALUE: the old value of the static variable seed

***************************************************************************************************************************/
unsigned int changeseed(unsigned int Iseed)
{
  unsigned int oldseed;
  
  oldseed = seed;
  seed = Iseed;
  
  return oldseed;
}


/* unif01 ******************************************************************************************************************

DESCRIPTION: Draw a random number from a uniform distribution on the interval [0,1]

SIDE EFFECTS: the value of the static variable seed is changed

RETURN VALUE: the random number

***************************************************************************************************************************/
double unif01(void)
{
  double random;
  
  seed = MULTIPLIER * seed + SHIFT;
  random = ((double) seed) * INVMOD;

  return random;
}


/* readseed  **************************************************************************************************************

DESCRIPTION: read new seed value from file, use standard value if nonexistant

SIDE EFFECTS: the value of the static variable seed is changed

RETURN VALUE: the seed value

***************************************************************************************************************************/
unsigned int readseed(char filename[])
{
  FILE *fp;

  seed = DEFAULTSEED;
  fp = fopen(filename, "r");
  if (fp != NULL)
  { fscanf(fp, "%u", &seed);
    fclose(fp);
  }
  return seed;
}


/* writeseed  *************************************************************************************************************

DESCRIPTION: write seed value to seed file

SIDE EFFECTS: none

RETURN VALUE: 1 if OK (return value from fprintf)

*****************************************************************************************************************************/
int writeseed(char filename[])
{
  FILE *fp;
  int r;
  
  fp = fopen(filename, "w");
  if (fp != NULL)
    r = fprintf(fp, "%u\n", seed);
  else
    r = -1;
  fclose(fp);
  return r;
}





