/* Bipartite_Graph.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _BIPARTITE_GRAPH_H
#define _BIPARTITE_GRAPH_H

#include "Reaction.h"
#include "Compound.h"

//#include "../../nauty/nauty.h"

/**
 * A bipartite graph corresponds to the bipartite representation of the metabolic network. You have two types of nodes: the reactions and the compounds. A reaction is linked to a compound with the label 1 (resp. 2) if the compound is used as a left compound (resp. right compound) by the reaction. Lines are compounds, columns are reactions.
 */
class Bipartite_Graph
{
 private:
  int **adjacency_matrix;
  int nb_compounds;
  int nb_reactions;
  Reaction **reactions; //node labels
  Compound **compounds; //node labels
  list<int> **reaction_colors;

  /*for nauty*/
  /* static graph *canong; //canonical graph in nauty format */
/*   static size_t canong_sz; */
  

 public:
  Bipartite_Graph(int, int);
  Bipartite_Graph(Reaction **,int,Compound **, int);
  Bipartite_Graph(Bipartite_Graph *, int *,int, int *); //filter constructor
  ~Bipartite_Graph();

  int get_nb_reactions();
  int get_nb_compounds();
  int **get_adjacency_matrix();
  Compound **get_compounds();
  Reaction **get_reactions();
  list<int> **get_reaction_colors();
  bool opposite_side(int, int, int);
  int get_topo();


  /*calls nauty directly and get the canonical labelling of the graph*/
  void set_canonical_labelling();

  void print();
  void print_dreadnaut(ostream &);  //print to file using dreadnaut format
  void print_dreadnaut_ordered(ostream &);  //print to file with labels using dreadnaut format
  void print_nodes_matrix(ostream &); //prints nodes and matrix
  void print_to_file_reverse(); //lines are reactions and columns are compounds
  void draw();
  void draw_structure();
  void print_nodes_edges(ostream &);  //prints to file using simple format (nodes and then edges as pairs of nodes)
  void print_edges(ostream &); //prints to file using simple format (edges as pairs of nodes)
  void print_identity_card();
  void print_adjacency_matrix();
  void print_colour_order();
};
#endif
