/* Compound_Graph.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _COMPOUND_GRAPH_H
#define _COMPOUND_GRAPH_H

#include "Reaction.h"
#include "Compound.h"
#include "Bipartite_Graph.h"
#include "Graph.h"

/**
 * A compound graph corresponds to a simplified version of the
 * bipartite representation of the metabolic network where you keep
 * only the nodes corresponding to the compounds. Edges are drawn
 * between compounds when they are on distinct sides of a reaction*/
class Compound_Graph
{
 private:
  bool **adjacency_matrix;
  int nb_compounds;
  int nb_edges;
  Compound **compounds; //node labels
 public:
  Compound_Graph(int);
  Compound_Graph(Bipartite_Graph *);
  Compound_Graph(Compound **, int);
  ~Compound_Graph();
  void print();
  void print_to_file(); //prints the compound graph to a file called Compound_Graph.txt
  void print_nodes_matrix(std::ostream &); //prints nodes and matrix
  void print_identity_card();
  void draw();
};
#endif
