/* FilteredGraph.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _FILTEREDGRAPH_H
#define _FILTEREDGRAPH_H
#include <iostream>
#include "Graph.h"
#include <list>
#include <map>

using namespace std;

/**
 *FilteredGraph class herits Graph class. It filtrates the graph build by the constructor Graph::Graph(graph_file, labels_file,EC_numbers,nb_EC).
 *
 */
class FilteredGraph : public Graph{

private:
  
  list<pair<int,int> > **distance_list; //adjacency list with distance
  int** distance_matrix;                //distance matrix
  int nb_node;                          //number of vertex of this graph
  //int *nodes_filtered; // for each vertex of this graph, give the index vertex of parent graph
  // Se trouve dans Graph -> heritage 
  list <int> **colors; //give colors for each vertex
  
  int local_gap;
  int global_gap;

  //private method

/**
 *Transform the  solution of the filtered graph to the solution in the original graph.
 * @param finalSolution the pointer to the table to put the solution of the original graph 
 * @param tmpSolution the solution in the filtered graph
 * @param motif_size size of the motif
 */
  void filterOneSolution(int**,
			 list<int>,
			 int );
  /**
   */
  void backtrack(list<int>*,
		 int*,
		 int*,
		 list<int>***,
		 list<int>*,
		 int***,
		 unsigned int*,
		 bool ***
		 );

public:
  /**
   * FilteredGraph constructor.
   *
   * @param graph_file name of the file contening the adjacency matrix of a graph. 
   * @param labels_file the file contening the graph node colors.
   * @param EC_numbers
   * @param nb_EC
   * @param colors_to_filter node colors to keep in Filtered Graph.
   * @param color_size number of different color of the  motif (size of colors_to_filter tab).
   * @param local_gap local gap value.
   * @param global_gap global gap value.
   */
  FilteredGraph(char*,char*,char**,int,int*,int,int,int);
  ~FilteredGraph();

  /**
   *Print the distance list.
   *
   */  
  void print_distance_list();
  
  /**
   *Print color of each node.
   */
  void printNodeColor();

  /**
   * Calculate the frequency of the color of the node.
   *
   *
   * @param tabToFill a non allocate pointer on integer, it will be allocate here with the size of the node which will start the search algorithm
   * @return the number of node wich color is less frequent (size of tabToFill)
   */
  int frequency(int **tabToFill);

  /** Launch the search of the given motif colors for the filtered graph.
   *
   * @param solutions
   * @param motif_size
   * @param motif_colors
   */
  int enumerate(list<int*>*,int,int*);

};



#endif
