/* Graph.h
 *
 * Copyright (C) 2007 Laboratoire de BiomÁtrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#ifndef _GRAPH_H
#define _GRAPH_H

#include "Io.h"
#include <list>
//#include "Reaction.h"
//#include "Metabolic_Motif.h"
//#include "Solution_Group.h"


/**
 *Graph class: enables to perform the search independently from the nature of the nodes.
 */
class Graph
{
 protected:
  /**
   *graph representation as an array of adjacency lists (the first reaction of each list is implicit)
   */
  list<int> **adjacency_list;
  
  bool **adjacency_matrix;   
  int nb_nodes;

  int *parent_nodes;  // for each vertex of this graph, give the index
		      // of the corresponding vertex in the parent
		      // graph in the case when this graph has been
		      // built using a filter constructor, or a
		      // shuffle constructor. The values in this array
		      // will always correspond to indices of
		      // reactions in the general reaction list in
		      // Data.
  

  list<int> **colors;
 
  
 public:
  /**
   *default constructor for a graph
   */
  Graph(int);
  /**
   *constructor for a graph from files (adjacency matrix)
   */
  Graph(char *,char *,char **, int);
  /**
   *constructor for a graph from files (list of edges)
   */
  Graph(char *,char *);

  Graph(Graph *);
  /**
   *constructor for a filtered graph
   */
  Graph(Graph *, int *,int);
 
  /**
   *constructor for a shuffled graph
   */
  Graph(Graph *, int *);

  virtual ~Graph();
  /**
   *gives the number of nodes in this graph
   */
  int get_nb_nodes();
  list<int> **get_adjacency_list();
  list<int> **get_colors();
  int *get_parent_nodes();

  /*for inference*/
  int get_nb_colors();
  void get_all_colors(int *tab);


  void print_edges(std::ostream &); //print edges in mfinder format

  void print_degrees(std::ostream &); //print node degrees for distribution fit

  bool **get_adjacency_matrix();
  /**
   *resizes the colors to te new threshold using the correspondance array
   */
  void reset_colors(int *,int);

  /**
   *resizes the colors to te new threshold using the correspondance array
   * @param correspondance matrix for colours at different threshold (i.e. tree of EC numbers)
   * @param threshold for cutting EC numbers
   */
  void reset_colors(int **,int);


  /**
   *Enumeration: given a list of colors, this function gives the collection of solutions (connected sets of nodes colored by the same set of colors). The solutions are given as arrays of integers.
   */
  virtual int enumerate(list<int *> *solutions, int motif_size, int *motif_colors);
  
  /**
   *Enumeration: given a list of colors, this function gives the number of solutions (connected sets of nodes colored by the same set of colors). The number of solutions is given as return value.
   */
  int enumerate(int motif_size, int *motif_colors);
  //virtual void print_adjacency_list();
  void print_adjacency_list();
  void print_colors();
  void print_colors(std::ostream &);
  //void printMotif();
};
#endif

