/* Metabolic_Motif.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _METABOLIC_MOTIF_H
#define _METABOLIC_MOTIF_H

#include <list>
#include "Enzyme.h"
#include "Data.h"

using namespace std;

class Compound;

/**
 * Contains all information on the motif itself. Also includes threshold parameters that enable to decide if a reaction set is an occurrence of this motif.
 */
class Metabolic_Motif
{
 private:
  Compound *input_compound;
  Compound *output_compound;
  list<Enzyme*> *enzymes;
  int *colours; //tab of colours (the number of colours can be smaller than the number of enzymes when two enzymes are identical)
  // + auxillary input + outputs ... later

  int sigma_E;
  int local_gap;
  int global_gap;
  float quorum;
 public:
  Metabolic_Motif(Data *);
  Metabolic_Motif(Data *, int, int, int, int, char*, char*, char*, char *);
  ~Metabolic_Motif();

  /*matches*/
  bool contains_similar_enzyme(Enzyme *);
  bool *enzyme_ranks(Enzyme *); //this function seems useless
  void enzyme_ranks(Enzyme *,list<int> *);//this function seems useless
  int enzyme_first_rank(Enzyme *);//this function seems useless
  int get_colour(Enzyme *);
  void get_colours(Enzyme *,list<int> *);
  
  /*information on motifs*/
  int get_local_gap();
  int get_global_gap();
  float get_quorum();
  int *get_colours();
  int nb_groups_expected();
  Compound *get_input();
  Compound *get_output();
  int size();
  void print_enzymes(std::ostream &);
  void print_colours();
};

#endif
