#ifndef _METABOLIC_NETWORK_H
#define _METABOLIC_NETWORK_H

#include "Data.h"
#include "Metabolic_Motif.h"
#include "Reaction.h"
#include "Reaction_Graph.h"
#include "Solution_Group.h"
#include <list>

using namespace std;


/**
 * Contains all information on the whole network. Note that it is different from the graph. A network is supposed to be independant from the model used for calculation (search). Actually, the only member of this class is the set of connected components (i.e. graph in one piece). Therefore, the metabolic network can be seen here as a graph with several components.
 */
class Metabolic_Network
{
 private:
  list<Reaction_Graph *> *connected_components;


 public:
  Metabolic_Network(Data *);
  Metabolic_Network(Data *, Metabolic_Motif *);  
  ~Metabolic_Network();


  /*********build functions**************/
  /**
   *builds the whole graph.
   */
  void build(Data *);  
  /**
   *builds a smaller network : no need to build the parts where the motif doesn't occur
   */
  int build(Data *, Metabolic_Motif *);  
  /**
   *This is an internal function that builds a graph from a set of reactions that might be connected. We verify if they are connected by looking at every reaction object.
   */
  void first_fusion(list<Reaction *> *reaction_set);  //auxiliary function
  /**
   *This function gets rid of some of the connected components that we are sure don't contain the motif.
   */
  void filter_connected_components(Metabolic_Motif *);

  /**
   *reads the graph given in a file as adjacency matrix and node labels given in a file as label lists
   */
  void read_graph(char *, char *);



  /*********interconversions of graph formats**************/
  void convert_graph(); //transforms the representation of the graph (from list of lists to array of lists)
  void convert_graph_to_matrix(); //creates a new representation of the graph (easier to interpret ?) : a matrix from an array of lists



  /***********enumeration of occurrences of given motif***************/
  /**
   *enumerates occurrences of the given motif in each of the connected component of the network. As the enumeration goes on, the results are gathered in the solution_group.
   */
  void enumerate_solutions(Metabolic_Motif *, Solution_Group *solutions);



  /***********drawing function***************/
  void draw_connected_components();




  /****************print functions******************/
  void print_connected_components();
  void print_connected_components(int verbose);  //print with several levels of detail
  void print_adjacency_lists();
  void print_adjacency_matrices();
  void print_identity_card();  //prints many information about the graph
  void print_identity_card(std::ostream &);  //prints to file many information about the graph
  void print_identity_card_enzyme_graph();
  void print_labels_enzyme_graph(int threshold);
  void print_reaction_arity(char *);
  void print_components_size(char *);
};

#endif
