/* Occurrence.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _OCCURRENCE_H
#define _OCCURRENCE_H

#include <list>
#include "../include/Graph.h"

/**
 * Contains all information on a solution of the search problem
 * (occurrence of the motif). An occurrence is a list of node ids.  In
 * order to be able to further classify by topology, a graph is
 * stored.
 */
class Occurrence
{
 private:
 
  int *nodes; //integers stand for node IDs in the original graph. 
  int size; //number of reactions in the solution

  /* Graph *bipartite_graph; */
  /* Graph *graph; */
  bool **adj_matrix;
  long int topo1;
  long int topo2;
  list<int> ** colors;


 public:
  Occurrence(int *,int);
  Occurrence(int *,int,int *); //constructor when the occurrence is enumerated from a filtered graph
  ~Occurrence();

  /*outside*/
  int get_size();
  long int get_topo1();
  long int get_topo2();
 


  
  /*group the linear solutions*/
  bool same_order(Occurrence *);

  /*group the solutions according to their topology in the bipartite graph*/
  bool same_topology(Occurrence *);

  
  /*graph structure (topology) of the occurrence*/
 /*  void set_bipartite_graph(Bipartite_Graph *,int *colors_to_filter); */
  void set_adj_matrix(Graph *);
  void compute_canonical_labelling(bool order);
  void print_dreadnaut(ostream &);  //print to file using dreadnaut format
  void print_dreadnaut_ordered(ostream &);  //print to file with labels using dreadnaut format


  /*print functions*/
  void print();
  void print(std::ostream&);
  void print_adj_matrix();
  void print_adj_matrix(std::ostream&);
  /* void print(std::ostream&,int verbose); */
/*   void print(int verbose); */
/*   void print_bipartite_graph(); */

};



#endif
