/* Occurrence_Group.h
 *
 * Copyright (C) 2007 Laboratoire de BiomÁtrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _OCCURRENCE_GROUP_H
#define _OCCURRENCE_GROUP_H


#include "Occurrence.h"
#include "Metabolic_Motif.h"

//using namespace std;

/**
 * This class corresponds to a class of occurrences. Classes can be defined according to several criteria. For instance, all occurrences that share the same input compound can be gathered in the same group. Or all occurrences that share a reaction with each other can be gathered in the same group.
 */


class Occurrence_Group
{
 private:
  list<Occurrence *> *occurrences;
 public:
  Occurrence_Group();
  Occurrence_Group(Occurrence *);
  ~Occurrence_Group(); //only deletes the group
  void delete_occurrences(); //deletes the solutions inside the group
  list<Occurrence *> * get_occurrences();
  void add(Occurrence *);


  /*precalculation of properties on occurrences*/
  void set_adj_matrix(Graph *);
  void compute_canonical_labelling(bool order);


  /*organization of occurrences*/
  void group_by_topology(list<Occurrence_Group *> *groups);   //groups together occurrences with same topology (done only for discriminating linear/non linear occurrences)
  void count_observed_topologies(long int *, long int *, int, int *); //modified version taking into account already established topologies  
  bool contains_similar_occurrence(Occurrence *);

  /*print functions*/
  void print();
  void print_occurrences(); //prints only the occurrences
  void print_bipartite_graphs();
  void print_bipartite_graph(); //only print one representative of the group, the first one
  void print(std::ostream&);
  void print_intra_pathway(std::ostream&);
  void print(int verbose);  //print with several levels of detail
  void print(std::ostream&, int verbose);
};



#endif
