/* Reaction_Graph.h
 *
 * Copyright (C) 2007 Laboratoire de BiomÁtrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _REACTION_GRAPH_H
#define _REACTION_GRAPH_H

#include <list>
#include "Reaction.h"
#include "Metabolic_Motif.h"
#include "Solution_Group.h"
#include "Graph.h"

class Reaction_Graph : public Graph
{
 private:
  //graph representations
  //adjacency lists (the first line corresponds to the neighbors of the first node...)
  //list< list<Reaction *> *> *graph; //naive representation of a graph : list of lists where each Reaction is represented as a pointer, this representation is used for building the graph
  /*  list<int> **adjacency_list;*/ //improved representation of a graph : tab of lists, each Reaction is represented as an integer (the first reaction of each line is implicitely refered to as the index of the line), the correspondance between integers and Reaction* is given by tab_reactions
  Reaction **tab_reactions; //adress_book giving the correspondance between int (index) and Reaction * (pointer on Reaction object), it would be nice to change this structure to an associative map

  //adjacency matrix (matrix where the cell (i,j) is set to 1 when there is an edge between reaction i and reaction j)
  /* bool **adjacency_matrix;*/ //the index of each line codes for a reaction (the correspondance is given in tab_reactions)

  //graph parameters, note that these parameters are set only after the graph has been built completely (they are initialized in convert_graph())
  /* int nb_nodes;*/
  int nb_edges;

 public:
  Reaction_Graph(Reaction **,int);
  Reaction_Graph(Reaction_Graph *);
  Reaction_Graph(Reaction_Graph *, int *); //creates a shuffled reaction graph
  Reaction_Graph(Reaction_Graph *, int *, int); //creates a filtered reaction graph
  ~Reaction_Graph();

  int get_nb_nodes();
  int get_nb_edges();
  int get_nb_nodes_enzyme_graph();
  int get_nb_edges_enzyme_graph();
  Reaction **get_tab_reactions();


  /*graph functions*/
  void convert_list_to_matrix(); //creates a matrix representation of the graph
  void convert_matrix_to_list();
  //one could add some basic functions like adding a node or an edge...


  /*filter function : filter connected components in which we are sure not to find occurrence of the motif*/
  //bool contains_all_enzymes(Metabolic_Motif *);


  /*enumeration functions*/
  int enumerate(Solution_Group *solutions, Metabolic_Motif *motif);

  /*shuffle functions*/

  /* /\** */
/*    *shuffles the graph using the program "melangeur", (the vector of */
/*    *labels is shuffled) */
/*    *\/ */
/*   void shuffle(int *); */
  

  /*print functions*/
  void draw();
  void print_adjacency_matrix(std::ostream &);
  void print_nodes_matrix(std::ostream &); //prints nodes and matrix
  void print_labels_enzyme_graph(std::ostream &,int threshold);
  void print_labels_reaction_graph(std::ostream &,int threshold);
  void print_sif(std::ostream &); //print edges in sif format (Ludo)


  /*stat function*/
  //void print_reaction_arity(FILE *); //this could be easily re-implemented
  void print_identity_card(std::ostream &);
};

#endif
