/* my_library.h
 *
 * Copyright (C) 2007 Laboratoire de BiomÁtrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _MY_LIBRARY_H
#define _MY_LIBRARY_H

#include <list>
#include <iostream>
using namespace std;
/**************************************************************************
			DEFINITION OF CONSTANTS
**************************************************************************/

#define MAX_NAME_LEN 64
#define MAX_NB_EC 2048
#define MAX 4096  //256 should be enough but still...
#define NB_FRAGS 31
#define GOOD 1
#define BAD 0
#define VERSION "1.1.3"
#define COLOUR0 "red"
#define COLOUR1 "darkgreen"
#define COLOUR2 "yellow"
#define COLOUR3 "lightblue"
#define COLOUR4 "green"
#define COLOUR5 "blue"



/**************************************************************************
			DEFINITION OF FUNCTIONS
**************************************************************************/




/* min et max de deux nombres*/

#define Min(i, j)  ((i) < (j) ? (i) : (j))
#define Max(i, j)  ((i) > (j) ? (i) : (j))

/*prototypes for stat functions*/
double mean(int *,int);
double variance(int *,int,double);

/*prototypes of string functions*/
int string_length(char *);
void string_copy(char *, char *);
bool string_compare(char *, char *);


/*prototypes of parsing functions*/
void extract_word(char *str, char *word, char end_of_word, int *i);
void split(char *str, int *tab, int n);
int count_char(char *str, char c);

/*prototypes for sort functions*/
void sort(int **tab, int nb_groups);
void sort(int *tab, int nb_groups);
void sort(double *tab, int nb_groups, int *permutation);
bool tabcmp(int *tab1, int *tab2);

/*prototype for clustering function*/
void group_by_overlap(list<int *>* occurrences,list<list<int *> *> *occurrences_groups,int motif_size);
int get_nb_clumps(list<int *>* occurrences,int motif_size);

/*prototypes for multichoice functions*/
bool next_multichoice(int *,int,int);
bool next_multichoice(int *,int *,int,int*,int);
bool next_multichoice_in_block(int *,int,int *,int *);
int get_nb_motifs_done(int *,int,int);

/*prototypes for functions on sets*/
int fact(int);
int choose(int, int);
int multichoose(int, int);


/*prototypes of print functions*/
void print_prout();
void print_tab(int *,int);
void print_tab(int *,int,std::ostream&);
void print_tab(long int *,int);
void print_tab(char **, int);
void print_tab(float *,int);
void print_tab(double *,int);
void print_tab(bool *tab, int size);
void print_mat(bool **mat, int nl, int nc);
void print_mat(int **mat, int nl, int nc);
void print_list(list<int>::iterator begin, list<int>::iterator end);
void print_list(list<int>::iterator begin, list<int>::iterator end,std::ostream& output);
void print_list_list(list<list<int> >::iterator begin, list<list<int> >::iterator end);
void print_tab_list(list<int> **tab_list, int size);
void print_tab_list(list<int> **tab_list, int size, std::ostream& output);

void print_list_tab(list<int *>::iterator begin, list<int *>::iterator end, int size);


/*prototype for finding function*/
int get_index(char *,char **,int); //get the index of the string in an array of strings of given size
int get_index(int,int *,int); //get the index of an int in an array of ints of given size
int get_index(int *, int **, int); //get the index of an int* in an array of ints* of given size
bool** initialize_matrix(bool **adjacency_matrix, int nb_nodes);

/*prototypes for counting functions*/
void first_choice(int *ones, int m);
bool is_last_choice(int *ones, int m, int n);
int next_choice(int *ones, int m, int n);
void initialize_topology_matrix(int **topology_matrix,int nb_topologies,int motif_size);
void fill_topology_matrix(int **topology_matrix,int motif_size);
void first_topology(int *topologies, int size);//not needed anymore
bool next_topology(int *topologies, int size);//not needed anymore
void first_color_choice(int *color_choice, int n);
bool next_color_choice(int *color_choice, int n, int *color_sizes);

void initialize_color_candidates(list<int *> *color_candidates, list<int> *colors, int motif_size);
void fill_color_candidates(int *candidates, int nb_cand, int pos_choices, list<int *> *color_candidates, int* nb_color_candidates, list<int> **colors, int motif_size, int *color_choice, int *color_sizes);
bool color_test(int *color_choice, int partial_size, int *motif, int motif_size, bool *matched_colors);
bool partial_test(list<int *> *color_candidates, int partial_size, int *nb_color_candidates, int *motif, int motif_size, bool *matched_colors);
bool final_test(list<int *> *color_candidates, int partial_size, int nb_color_candidates, int *motif, int motif_size, bool *matched_colors);
void pick_neighbors(int reaction, list<int> **adjacency_list, bool **picked_eliminated, int **neighbors, int *reactions_picked, int *pos);
void unpick_neighbors(int dist_pos,bool **picked_eliminated, int **neighbors, int *reactions_picked, int *pos);


//ced
/* prototype for algorithm fonction*/
int **floyd(int p, bool **data, int n);
bool **adjacencyListToAdjacencyMatrix(list<int>**,int);
list<int> ** distanceMatrixToAdjacencylist(int ** m, int size);
void printMatrix(int **mat, int size);
void printMatrix(bool **mat, int size);
void printList(list<int>**l, int size);
void deleteMatrix(int ** matrix, int size);
void deleteMatrix(char ** matrix, int size);
bool is_list_in_tab(list<int> *l,int *tab,int n);
bool is_in_tab(int query,int *tab,int n);
int** boolMtrixToIntMatrix(bool** m, int size);

#endif
