/* Compound.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Compound.h"
#include <iostream>
#include "../include/Data.h"
#include <cstdlib>

//extern map<void*,bool> compoundPtrs;

Compound::Compound(char *cpd_id, char *cpd_name)
{
  //memory verification
  //compoundPtrs[this] = false;

  id = (char*)malloc((strlen(cpd_id)+1)*sizeof(char));
  id = strcpy(id,cpd_id);
  //string_copy(id, cpd_id);
  name = (char*)malloc((strlen(cpd_name)+1)*sizeof(char));
  name = strcpy(name,cpd_name);
  //string_copy(name, cpd_name);

  frags = (bool*)malloc(NB_FRAGS*sizeof(bool));
  int_frags = (int*)malloc(NB_FRAGS*sizeof(int));

  left_reactions = new list<Reaction *>;
  right_reactions = new list<Reaction *>;
  left_right_reactions = new list<Reaction *>;
  no_status_reactions =new list<Reaction *>;
  reactions = new list<Reaction *>;

  visited_edges=NULL;
}

Compound::~Compound()
{
  free(id);
  free(name);
  free(frags);
  free(int_frags);

  if (visited_edges!=NULL)
    {
      for (int i=0;i<(int)reactions->size();i++)
	free(visited_edges[i]);
      free(visited_edges);
    }
  delete(reactions);
  delete(left_reactions);
  delete(right_reactions);
  delete(left_right_reactions);
  delete(no_status_reactions);

  
}

void Compound::reset()
{
  int size=reactions->size();
  for (int i=0;i<size;i++)
    for (int j=0;j<size;j++)
      visited_edges[i][j]=0;
}

/*test if the reaction is already in the reaction_list*/
/*this function is already in Metabolic Network, we should do something with templates*/
bool in_list2(Reaction *reaction_to_add, list<Reaction *> *rea_list)
{
  list<Reaction *>::iterator it = rea_list->begin();
  while (it != rea_list->end())
    {
      if (reaction_to_add == *it)
	return true;
      it++;
    }
  return false;
}

void Compound::concatenate_reactions()
{
  list<Reaction *>::iterator rea_iter;
  copy(left_reactions->begin(),left_reactions->end(),back_inserter(*reactions));
  //reactions = left_reactions;
  rea_iter = right_reactions->begin();
  while (rea_iter != right_reactions->end())
    {
      if (!in_list2(*rea_iter, reactions))
	reactions->push_back(*rea_iter);
      rea_iter++;
    }
  rea_iter = no_status_reactions->begin();
  while (rea_iter != no_status_reactions->end())
    {
      if (!in_list2(*rea_iter, reactions))
	reactions->push_back(*rea_iter);
      rea_iter++;
    }
  reactions->sort();
  reactions->unique();  
  //we could add something to verify there's no duplicates
  //cout <<"reactions : tailles de left :"<<left_reactions.size()<<" right "<<right_reactions.size()<<" et all "<<reactions.size()<<endl;
}


void Compound::set_frags(int *data_frags)
{
  /*turn int into bool*/
  for (int i=0;i<NB_FRAGS;i++)
    {
      frags[i]=(data_frags[i]>0);
      int_frags[i]=data_frags[i];
    } 
}

void Compound::set_edges()
{
  int size = reactions->size();
  //cout<<id<<" set_edges : size = "<<size<<endl;
  visited_edges = (bool**)malloc(size*sizeof(bool*));
  for (int i=0;i<size;i++)
    {
      visited_edges[i] = (bool*)malloc(size*sizeof(bool));
      for (int j=0;j<size;j++)   
	visited_edges[i][j]=0;
    }
  nb_edges = size*(size-1)/2;
}

void bool_print2(bool *tab, int size)
{
  for (int i=0;i<size;i++)
    cout<<tab[i];
  cout<<endl;
}

/*we want identity*/
// bool Compound::is_similar_to(Compound *cpd)
// {
//   bool out=1;
//   cout <<"je vais comparer les composs "<<id<<"et ";
//   cpd->print();
//   cout<<endl;
//   // for (int i=0; i<NB_FRAGS; i++)
// //     {
// //       out = out & (frags[i] == cpd->get_frags()[i]); 
// //     }
//   for (int i=0; i<NB_FRAGS; i++)
//     {
//       out = out & (int_frags[i] == cpd->get_int_frags()[i]); 
//     }
//   cout<<"rponse "<<out<<endl;
//   return out;
// }

// /*old score function*/
// bool Compound::is_similar_to(Compound *cpd)
// {
//   // cout <<"je vais comparer les composs "<<id<<"et ";
// //   cpd->print();
// //   cout<<endl;
//   float score=0.0;
//   int nb_non_empty=0;
//   bool out=1;
//   for (int i=0; i<NB_FRAGS; i++)
//     {
//       int a=int_frags[i];
//       int b=cpd->get_int_frags()[i];
//       if (frags[i] == cpd->get_frags()[i])
// 	{
// 	  score++;
// 	  if (a!=0)
// 	    {
// 	      score+=(1.0-((float)max(a,b)-(float)min(a,b))/(float)max(a,b));
// 	      nb_non_empty++;
// 	    }
// 	}
//     }
//   out = (score>=(float)NB_FRAGS+0.98*(float)nb_non_empty); //il faudrait passer a en paramtre d'entre du programme :
//   return out;
// }

/*the scoring function used here is not very good, it should be replaced by something better*/
/*the relation of similarity is rather stringent here, we want the compounds to have identical boolean description*/
bool Compound::is_similar_to(Compound *cpd, int threshold)
{
  float score=0.0;
  int nb_non_empty=0;
  bool out=1;
  for (int i=0; i<NB_FRAGS; i++)
    {
      int a=int_frags[i];
      int b=cpd->get_int_frags()[i];
      if (frags[i] == cpd->get_frags()[i])  //egalite des booleens
	{
	  score++;
	  if (a!=0)
	    {
	      score+=(1.0-((float)max(a,b)-(float)min(a,b))/(float)max(a,b));
	      nb_non_empty++;
	    }
	}
    }
  out = (score>=(float)NB_FRAGS+((float)threshold/100)*(float)nb_non_empty); 
  return out;
}

/*TO BE TESTED*/
/*the score is different here : it goes from 0 (every boolean is different) to 1 (every integer is similar)*/
/*the relation of similarity is less stringent here*/
// bool Compound::is_similar_to2(Compound *cpd, int threshold)
// {
//   float score=0.0;
//   int nb_non_empty=0;
//   bool out=1;
//   for (int i=0; i<NB_FRAGS; i++)
//     {
//       int a=int_frags[i];
//       int b=cpd->get_int_frags()[i];
//       if (frags[i] == cpd->get_frags()[i])  //egalite des booleens
// 	{
// 	  //score++;
// 	  if (a!=0)
// 	    {
// 	      score+=(1.0-((float)max(a,b)-(float)min(a,b))/(float)max(a,b));
// 	      nb_non_empty++;
// 	    }
// 	}
//     }
//   out = (score >= (float)NB_FRAGS * (float)threshold/100);  //ne devrait-on pas faire le pourcentage des fragments prsents ?
//   //out = (score>=(float)NB_FRAGS+((float)threshold/100)*(float)nb_non_empty); 
//   return out;
// }


char *Compound::get_id()
{
  return id;
}

char *Compound::get_name()
{
  return name;
}

int Compound::get_nb_edges()
{
  return nb_edges;
}


list<Reaction *> * Compound::get_left_reactions()
{
  return left_reactions;
}

list<Reaction *> * Compound::get_right_reactions()
{
  return right_reactions;
}

list<Reaction *> * Compound::get_no_status_reactions()
{
  return no_status_reactions;
}

list<Reaction *> * Compound::get_reactions()
{
  return reactions;
}

bool * Compound::get_frags()
{
  return frags;
}

int * Compound::get_int_frags()
{
  return int_frags;
}

/*don't need to verify, could be optimised*/
int Compound::add_left_reaction(Reaction *reaction_to_add)
{
  list<Reaction *>::iterator rea_iter = left_reactions->begin();
  while (rea_iter != left_reactions->end())
    {
      if (reaction_to_add == *rea_iter)
	return 0;
      rea_iter++;
    }
  left_reactions->push_back(reaction_to_add);
  return 1;
}

int Compound::add_right_reaction(Reaction *reaction_to_add)
{
  list<Reaction *>::iterator rea_iter = right_reactions->begin();
  while (rea_iter != right_reactions->end())
    {
      if (reaction_to_add == *rea_iter)
	return 0;
      rea_iter++;
    }
  right_reactions->push_back(reaction_to_add);
  return 1;
}

int Compound::add_left_right_reaction(Reaction *reaction_to_add)
{
  list<Reaction *>::iterator rea_iter = left_right_reactions->begin();
  while (rea_iter != left_right_reactions->end())
    {
      if (reaction_to_add == *rea_iter)
	return 0;
      rea_iter++;
    }
  left_right_reactions->push_back(reaction_to_add);
  return 1;
}

int Compound::add_no_status_reaction(Reaction *reaction_to_add)
{
  list<Reaction *>::iterator rea_iter = no_status_reactions->begin();
  while (rea_iter != no_status_reactions->end())
    {
      if (reaction_to_add == *rea_iter)
	return 0;
      rea_iter++;
    }
  no_status_reactions->push_back(reaction_to_add);
  return 1;
}


//this function needs optimization
int get_rank(Reaction *reac, list<Reaction *> *rea_list)   //there should be something in case the reaction is not in the list
{
  //gets the rank of a reaction in a reaction list
  int i=0;
  list<Reaction *>::iterator iter = rea_list->begin();
  while (iter != rea_list->end())
    {
      if ((*iter) == reac) return i;
      iter++;
      i++;
    }
  cerr <<"houston, we got a problem the reaction is not in the list (compound.cpp)";
  return -1;
}

bool Compound::is_visited(Reaction *reac1, Reaction *reac2)
{
  /*int i,j;
  if (left)  //the first reaction is the left reaction
    {
      i = get_rank(reac1, &left_reactions);
      j = get_rank(reac2, &right_reactions);
    }
  else   //the first reaction is the right reaction
    {
      i = get_rank(reac2, &left_reactions);
      j = get_rank(reac1, &right_reactions);
    }
  int offset = right_reactions.size();
  return visited_edges[offset*i + j];
  */
  int i = get_rank(reac1, reactions);
  int j = get_rank(reac2, reactions);
  return visited_edges[i][j];
}


void Compound::set_visited(Reaction *reac1, Reaction *reac2)
{
  /*
  int i,j;
  if (left)  //the first reaction is the left reaction
    {
      i = get_rank(reac1, &left_reactions);
      j = get_rank(reac2, &right_reactions);
    }
  else   //the first reaction is the right reaction
    {
      i = get_rank(reac2, &left_reactions);
      j = get_rank(reac1, &right_reactions);
    }
  int offset = right_reactions.size();
  visited_edges[offset*i + j] = 1; 
  */
  int i = get_rank(reac1, reactions);
  int j = get_rank(reac2, reactions);
  visited_edges[i][j]=1; 
  visited_edges[j][i]=1; 
}

//we could replace those two functions by a single function : switch_visited ending with  : visited_edges[offset*i + j] = !(visited_edges[offset*i + j])
void Compound::set_unvisited(Reaction *reac1, Reaction *reac2)
{
  /*
  int i,j;
  if (left)  //the first reaction is the left reaction
    {
      i = get_rank(reac1, &left_reactions);
      j = get_rank(reac2, &right_reactions);
    }
  else   //the first reaction is the right reaction
    {
      i = get_rank(reac2, &left_reactions);
      j = get_rank(reac1, &right_reactions);
    }
  int offset = right_reactions.size();
  //visited_edges[offset*i + j] = 0;  
  */
  int i = get_rank(reac1, reactions);
  int j = get_rank(reac2, reactions);
  visited_edges[i][j]=0; 
  visited_edges[j][i]=0; 
}

//arity of the compound in the metabolic network, i.e. number of edges in the reaction graph the compound is responsible for
int Compound::arity()
{
  int size=reactions->size(); //nb of reactions in which the compound is involved
  return size*(size-1)/2;
  //we could return the arity of the compound in the compound graph (I.e. number of compounds on the other side of a reaction)
}

//number of reactions in which the compound is involved
int Compound::get_nb_reactions()
{
  return (int)reactions->size();
}

void Compound::print()
{
  //cout << id;
  cout<< name;  //for E coli data, it's clearer but be careful with the NAs since everytime athe common name of a compound is not known we use NA, therfore NA can designate distinct compounds
}



/*print id in file used to draw the graph*/
void Compound::draw(std::ostream &output)
{
  output <<"\""<<name<<"\"";
}

void Compound::print(std::ostream &output)
{
  output <<name;//for E coli data, it's clearer than id
}


void Compound::print_id(std::ostream &output)
{
  output <<id;
}


void Compound::print_nb_reactions()
{
  cout<<reactions->size()<<" ";
}

void Compound::print_visited_edges()
{
  cout <<"visited_edges de "<<name<<endl;
  int size = reactions->size();
  for (int i=0;i<size;i++)
    {
      for (int j=0;j<size;j++)   
	cout <<visited_edges[i][j];
      cout<<endl;
    }
  cout <<endl;
}
