/* Compound_Graph.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Compound_Graph.h"
#include <iostream>
#include "../include/my_library.h"
#include <cstdlib>

#include <fstream>
//#include <map>
//using namespace std;

Compound_Graph::Compound_Graph(int nb_compounds)
{
  this->nb_compounds=nb_compounds;
  /*matrix creation + initialisation*/
  adjacency_matrix = new bool *[nb_compounds];
  for (int i=0;i<nb_compounds;i++)
    {
      adjacency_matrix[i] = new bool[nb_compounds];
      for (int j=0;j<nb_compounds;j++)
	adjacency_matrix[i][j]=0;
    }
}

Compound_Graph::Compound_Graph(Bipartite_Graph *bipartite_graph)
{
  this->nb_compounds = bipartite_graph->get_nb_compounds();
  /*matrix creation + initialisation*/
  adjacency_matrix = new bool *[nb_compounds];
  for (int i=0;i<nb_compounds;i++)
    {
      adjacency_matrix[i] = new bool[nb_compounds];
      for (int j=0;j<nb_compounds;j++)
	adjacency_matrix[i][j]=0;
    }

  /*fill labels*/
  compounds = bipartite_graph->get_compounds();

  /*fill matrix very slowly*/
  // cout<<"le nombre de composs est "<<nb_compounds<<endl;
  for (int k=0;k<bipartite_graph->get_nb_reactions();k++)
    {
      //cout<<"k= "<<k<<endl;
      for (int i=0;i<nb_compounds;i++)
	{
	  //cout<<"i= "<<i<<endl;
	  for (int j=i+1;j<nb_compounds;j++)
	    {
	      //cout<<"j= "<<j<<endl;
	      if (bipartite_graph->opposite_side(i,j,k))
		{
		  adjacency_matrix[i][j] = true;
		  adjacency_matrix[j][i] = true;
		  nb_edges++;
		  //cout<<"cpd1 =";
		  //compounds[i]->print();
		  //cout<<"cpd2 =";
		  //compounds[j]->print();
		  //cout<<"nb_edges= "<<nb_edges<<endl;
		}
	      //cout<<"nb_edges= "<<nb_edges<<endl;
	      // cout<<"i="<<i<<" j="<<j<<endl;
// 	      cout<<"result "<<adjacency_matrix[i][j]<<endl;
	    }
	}
    }
}


Compound_Graph::~Compound_Graph()
{
  for (int i=0;i<nb_compounds;i++)
    delete[](adjacency_matrix[i]);
  delete[](adjacency_matrix);
}

void Compound_Graph::print_identity_card()
{
  int count=0;
  cout<<"Graphe des composs"<<endl;
  cout<<endl;
  cout<<"nombre de noeuds: "<<nb_compounds<<endl;
  cout<<"nombre d'artes: "<<nb_edges<<endl;
  cout<<endl;
  cout<<"Composs sans artes"<<endl;
  bool ok=0;
  for (int i=0;i<nb_compounds;i++)
    {
      ok=0;
      for (int j=0;j<nb_compounds;j++)
	{
	  ok = ok || adjacency_matrix[i][j];
	}
      if (!ok)
	{
	  compounds[i]->print();
	  count++;
	  // for (int k=0;k<nb_compounds;k++)
// 	    cout<<adjacency_matrix[i][k]<<" ";
// 	  cout<<endl;
	  cout<<" ";
	}
    }
  cout<<"count= "<<count;
  cout<<endl;
}

/*prints the adjacency matrix*/
void Compound_Graph::print()
{
  //cout<<"avant print"<<endl;
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_compounds;j++)
	{
	  cout<<adjacency_matrix[i][j];
	}
      cout<<endl;
    }
  //cerr<<"aprs print"<<endl;
}

/*prints the adjacency matrix and the labels*/
void Compound_Graph::print_to_file()
{
  std::ofstream output;
  output.open("compound_graph.txt", ios_base::app);
  // for (int i=0;i<nb_compounds;i++)
//     {
//       compounds[i]->print(output);
//       output<<" ";
//     }
//   output<<endl;
  //output<<endl;
  
  for (int i=0;i<nb_compounds;i++)
    {
      compounds[i]->print(output);
      output<<" ";
      for (int j=0;j<nb_compounds;j++)
	{
	  output<<adjacency_matrix[i][j]<<" ";
	}
      output<<endl;
    }
  output.close();
}

/*print node ids and matrix*/
void Compound_Graph::print_nodes_matrix(std::ostream& output)
{
  /*print nodes ids */  
  for (int i=0;i<nb_compounds;i++)
    {
      compounds[i]->print(output);
      output<<"\t";
    }
  output<<endl;

  /*print matrix*/
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_compounds;j++)
	{
	  output<<adjacency_matrix[i][j];
	}
      output<<endl;
    }
}


void Compound_Graph::draw()
{
  std::ofstream output;
  output.open("bidon2.res", ios_base::app);
  
  output <<"graph G {"<<endl;
  output << "node[shape=ellipse, style=filled, height=.3, width=.3];"<<endl;
  // output << "{node[shape=box, style=filled]";
//   for (int j=0;j<nb_compounds;j++)
// 	{
// 	  compounds[j]->draw(output);
// 	  output<<" ";
// 	}
//   output<<"}"<<endl;

  // output << "{node[shape=ellipse, style=filled, fontsize=10,height=.3, width=.3 ]";
//   for (int j=0;j<nb_compounds;j++)
// 	{
// 	  compounds[j]->draw(output);
// 	  output<<" ";
// 	}
//   output<<"}"<<endl;


  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_compounds;j++)
	{
	  if (adjacency_matrix[i][j])
	    {
	      compounds[i]->draw(output);
	      output <<" -- ";
	      compounds[j]->draw(output);
	      output <<";"<<endl;
	    }
	}
    }
  output<<"}"<<endl;
  output.close();

  /*compile with neato*/
  system("neato bidon2.res -Tps -o draw2.ps -Goverlap=false");
  //system("rm bidon2.res");
  
  /*draw with gv*/
  //system("gv draw2.ps &");
}
