/* Data.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Data.h"
#include <iostream>
#include <sstream>
#include <cstring>
#include <cstdlib>
#include <cstdio>


#include "../include/my_library.h"

#define __DEBUG__

//map<void*,bool> enzymePtrs;
//map<void*,bool> reactionPtrs;
//map<void*,bool> compoundPtrs;
//map<void*,bool> pathwaysPtrs;
//map<void*,bool> organismPtrs;

Data::Data(char *reaction_compound_file, char *enzyme_reaction_file, char *frag_decomposition)
{
  
  enzymes   = new list<Enzyme *>;
  reactions = new list<Reaction *>;
  compounds = new list<Compound *>;
  pathways  = new list<Pathway *>;
  organisms = new list<Organism *>;
  ;

  /*reading reaction_compound file and then creates the instanciations of compounds and reactions (and links between them)*/
  //cout << "je vais lire le fichier kegg.compound_reaction3" <<endl;
  reaction_compound_parser(reaction_compound_file);
  /*reading reaction_enzyme file and then creates the instanciation of enzymes (and links from reactions to enzymes)*/
  //cout << "je vais lire le fichier kegg.enzyme_reaction3" <<endl;
  enzyme_reaction_parser(enzyme_reaction_file);
  /*reading the decomposition of compounds in the basis of fragments*/
  //cout<< "je vais lire la dcomposition en fragments"<<endl;
  frag_decomposition_parser(frag_decomposition);
}

Data::Data(char *reaction_compound_file, char *enzyme_reaction_file)
{
 
  enzymes=new list<Enzyme *>;
  reactions=new list<Reaction *>;
  compounds=new list<Compound *>;
  pathways=new list<Pathway *>;
  organisms=new list<Organism *>;
  /*reading reaction_compound file and then creates the instanciations of compounds and reactions (and links between them)*/
  //cout << "je vais lire le fichier kegg.compound_reaction3 bis" <<endl;
  reaction_compound_parser(reaction_compound_file);
  /*reading reaction_enzyme file and then creates the instanciation of enzymes (and links from reactions to enzymes)*/
  //cout << "je vais lire le fichier kegg.enzyme_reaction3 bis" <<endl;
  enzyme_reaction_parser(enzyme_reaction_file);
  //cout <<"j(ai fini"<<endl;
  // char enzyme_organism_file[MAX]="ECORG2";
//   enzyme_organism_parser(enzyme_organism_file);
//   exit(0);
}

/*reads data file with all the information in the same input file*/
Data::Data(char *reaction_compound_enzyme_file)
{



  enzymes=new list<Enzyme *>;
  reactions=new list<Reaction *>;
  compounds=new list<Compound *>;
  pathways=new list<Pathway *>;
  organisms=new list<Organism *>;
 

  /*parsing of datafile: all objects are created as well as
    associations between objects Example: each reaction knows its
    compounds, its enzymes and its pathways
  */
  reaction_compound_enzyme_gene_parser(reaction_compound_enzyme_file);

  /*sorting of lists*/
  enzymes->sort(Enzyme_Compar());
  reactions->sort(Reaction_Compar());
  compounds->sort(Compound_Compar());

  /*transformation of lists to arrays: */
  nb_reactions=reactions->size();
  tab_reactions = new Reaction *[nb_reactions];
  list<Reaction *>::iterator rea_it = reactions->begin();
  for (int i=0;i<nb_reactions;i++)
    {
      tab_reactions[i]=(*rea_it);
      rea_it++;
    }


  nb_compounds=compounds->size();
  tab_compounds = new Compound *[nb_compounds];
  list<Compound *>::iterator cpd_it = compounds->begin();
  for (int i=0;i<nb_compounds;i++)
    {
      tab_compounds[i]=(*cpd_it);
      cpd_it++;
    }

  // nb_enzymes=enzymes->size();
//   tab_enzymes = new Enzyme *[nb_enzymes];
//   list<Enzyme *>::iterator enz_it = enzymes->begin();
//   for (int i=0;i<nb_enzymes;i++)
//     {
//       tab_enzymes[i]=(*enz_it);
//       enz_it++;
//     }


  /*fill array of all possible EC_numbers, useful for the website and for giving the correspondance between colours and EC numbers*/
  /*memory allocation*/
  colours_EC= new int*[MAX_NB_EC];
  colours_str= new char*[MAX_NB_EC];
  correspondance_colours = new int*[MAX_NB_EC];
  for (int i=0;i<MAX_NB_EC;i++)
    {
      colours_EC[i]= new int[4];
      colours_str[i]= new char[MAX_NAME_LEN];
      correspondance_colours[i]=new int[4];
    }
  /*fill arrays and gives a value to the number of EC*/
  fill_colours();

  // print_mat(correspondance_colours,nb_colours,4);
//   exit(0);

  /*assign colours to reactions*/
  assign_colours_to_reactions(); 


}


Data::~Data()
{
  
  for (list<Enzyme *>::iterator it=enzymes->begin();it!=enzymes->end();it++){
    //  if((enzymePtrs.find(*it)) !=  (enzymePtrs.end()))
    // enzymePtrs[(*it)]=true;
     delete(*it);
    
  }  

  //parcours de la map d'enzyme
  //for(map<void*,bool>::iterator it = enzymePtrs.begin();it!=enzymePtrs.end();it++){
  //cout <<" enzyme dans mapp = " << (*it).first << " "<< (*it).second<< endl;
   
  //}
  

 for (list<Compound *>::iterator it=compounds->begin();it!=compounds->end();it++){
   //if((compoundPtrs.find(*it)) !=  (compoundPtrs.end()))
   // compoundPtrs[(*it)]=true;
  delete(*it);
  
 }
 /*
  //parcours de la map de compound
 for(map<void*,bool>::iterator it = compoundPtrs.begin();it!=compoundPtrs.end();it++){
   cout <<" compound dans mapp = " << (*it).first << " "<< (*it).second<< endl; 
}
*/
 for (list<Reaction *>::iterator it=reactions->begin();it!=reactions->end();it++){
   //if((reactionPtrs.find(*it)) !=  (reactionPtrs.end()))
   // reactionPtrs[(*it)]=true;
   delete(*it);
 }
 /*
  //parcours de la map de reaction
 for(map<void*,bool>::iterator it = reactionPtrs.begin();it!=reactionPtrs.end();it++){
   cout <<" reaction dans mapp = " << (*it).first << " "<< (*it).second<< endl;
  }
*/

  for (list<Pathway *>::iterator it=pathways->begin();it!=pathways->end();it++)
    {
      // if((pathwaysPtrs.find(*it)) !=  (pathwaysPtrs.end()))
      //pathwaysPtrs[(*it)]=true;
      delete(*it);
    }  
  /*
  //parcours de la map de pathways
 for(map<void*,bool>::iterator it = pathwaysPtrs.begin();it!=pathwaysPtrs.end();it++){
   cout <<" pathways dans mapp = " << (*it).first << " "<< (*it).second<< endl;
  }
  */
 for (list<Organism *>::iterator it=organisms->begin();it!=organisms->end();it++){

   //   if((organismPtrs.find(*it)) !=  (organismPtrs.end()))
   //organismPtrs[(*it)]=true;
   delete(*it);
 }  
 /*
 //parcours de la map d'organisms
 cout << "map organism" << endl;
 for(map<void*,bool>::iterator it = organismPtrs.begin();it!=organismPtrs.end();it++){
   cout <<" organisms dans mapp = " << (*it).first << " "<< (*it).second<< endl;
}
 */
 //enzymePtrs.clear();
 // compoundPtrs.clear();
 
 //reactionPtrs.clear();

 // pathwaysPtrs.clear();

 //organismPtrs.clear();
 
 /*free EC_numbers*/
 for (int i=0; i<MAX_NB_EC; i++)
   {
     delete[](colours_EC[i]);
     delete[](colours_str[i]);
     delete[] correspondance_colours[i];
   }
 delete[](colours_EC);
 delete[](colours_str);
 delete[] correspondance_colours;

 /*free lists*/
 delete(enzymes);
 delete(compounds);
 delete(reactions);
 delete(pathways);
 delete(organisms);

 /*free arrays*/
 delete[] tab_compounds;
 delete[] tab_reactions;
 //delete[] tab_enzymes;
}

Reaction *Data::reaction_adress_book(char *rea_id)
{
  list<Reaction *>::iterator reac_iter = reactions->begin();
  while (reac_iter != reactions->end())
    {
      char *tmp_id = (*reac_iter)->get_id();
      if (strcmp(tmp_id, rea_id)==0) return *reac_iter;  //returns the concerned reaction if it exists already
      reac_iter++;
    }
  return NULL;
}

Enzyme *Data::enzyme_adress_book(char *ec)
{
  list<Enzyme *>::iterator enz_iter = enzymes->begin();
  while (enz_iter != enzymes->end())
    {
      char *tmp_ec = (*enz_iter)->get_ec();
      if (strcmp(tmp_ec, ec)==0) return *enz_iter;  //returns the concerned enzyme if it exists already
      enz_iter++;
    }
  //cout <<"sorry pal, your enzyme "<<ec<<"ain't in the list"<<endl;
  return NULL;
}

Compound * Data::compound_adress_book(char *cpd_id)
{
  list<Compound *>::iterator comp_iter = compounds->begin();
  while (comp_iter != compounds->end())
    {
      char *tmp_id = (*comp_iter)->get_id();
      if (strcmp(tmp_id, cpd_id)==0) return *comp_iter;  //returns the concerned compound if it exists already
      comp_iter++;
    }
  return NULL;  
}

Pathway * Data::pathway_adress_book(char *pat_id)
{
  list<Pathway *>::iterator pat_iter = pathways->begin();
  while (pat_iter != pathways->end())
    {
      char *tmp_id = (*pat_iter)->get_id();
      if (strcmp(tmp_id, pat_id)==0) return *pat_iter;  //returns the concerned compound if it exists already
      pat_iter++;
    }
  return NULL;  
}

Organism *Data::organism_adress_book(char *org_id)
{
  list<Organism *>::iterator org_iter = organisms->begin();
  while (org_iter != organisms->end())
    {
      char *tmp_id = (*org_iter)->get_id();
      if (strcmp(tmp_id, org_id)==0) return *org_iter;  //returns the concerned compound if it exists already
      org_iter++;
    }
  return NULL;  
}

//returns 0 for substrate, 1 for product, 2 for undefined or NOT FOUND
int test_compound_status(char *cpd_status)
{
  /*product*/
  if (cpd_status[0]=='p')
    return 1;
  /*substrate*/
  else if (cpd_status[0]=='s')
    return 0;
  /*NOT_FOUND*/
  else 
    return 2;
}

void extract_word_keeping_blanks(char *str, char *word, char end_of_word, int *i)
{
  //extracts a word from a line, withdrawing the blanks
  int j=0;
  while (str[*i] != '\0' && str[*i] != '\n' && str[*i] != end_of_word)
    {
      word[j]=str[*i];
      (*i)++;
      j++;
    }
  word[j]='\0';
  if (str[*i] != '\0' && str[*i] != '\n')
    (*i)++;
}

// void extract_word(char *str, char *word, char end_of_word, int *i)
// {
//   //extracts a word from a line, withdrawing the blanks
//   int j=0;
//   while (str[*i] != '\0' && str[*i] != '\n' && str[*i] != end_of_word )
//     {
//       if (str[*i]!=' ')
// 	{
// 	  word[j]=str[*i];
// 	  (*i)++;
// 	  j++;
// 	}
//       else  (*i)++;
//     }
//   word[j]='\0';
//   //cout<<word<<endl;
//   //  cout<<*i<<endl;
//   if (str[*i] != '\0' && str[*i] != '\n')
//     (*i)++;
// }


void extract_tab(char *str, int *tab, char end_of_word, int *i)
{
  //extracts a tab from a line
  int j=0;
  char *tmp_str = (char*)malloc(MAX*sizeof(char));  //this memmory allocation should be done somewhere else (maybe this tab should be seen as a string and segmented later in the read function
  while (str[*i] != '\0' && str[*i] != end_of_word)
    {
      extract_word(str, tmp_str, ' ', i);
      tab[j]=atoi(tmp_str);
      j++;
    }
  if(str[*i] != '\0')
    (*i)++;
  free(tmp_str);
}


int line_parsing(char *str,char *rea_id,char *pat_id,char *pat_name,char *cpd_id,char *cpd_status,char *cpd_name)
{
  int i=0;
  extract_word(str,rea_id,'|',&i);
  //extract_word(str,rea_name,'|',&i);
  extract_word(str,pat_id,'|',&i);
  extract_word(str,pat_name,'|',&i);
  extract_word(str,cpd_id,'|',&i);
  extract_word(str,cpd_status,'|',&i);
  extract_word(str,cpd_name,'\n',&i);
  cout <<"\non passe  la ligne suivante"<<endl;
  return 1;
}

int line_parsing(char *str, char *rea_id, char *ec)
{
  int i=0;
  extract_word(str,rea_id,'|',&i);
  extract_word(str,ec,'\n',&i);
  return 1;
}

/*used for decomposition of compounds into fragments*/
int line_parsing(char *str, char *cpd_id, int *frags)
{
  int i=0;
  extract_word(str,cpd_id,' ',&i);
  extract_tab(str,frags,'\0',&i);
  return 1;
}


int line_parsing(char *str,char *rea_id,char *pat_id,char *pat_name,char *cpd_id,char *cpd_status,char *cpd_name, char *ec, char *ec_source)
{
  int i=0;
  extract_word(str,rea_id,'\t',&i);
  //extract_word(str,rea_name,'|',&i);
  extract_word(str,pat_id,'\t',&i);
  extract_word_keeping_blanks(str,pat_name,'\t',&i);
  extract_word(str,cpd_id,'\t',&i);
  extract_word(str,cpd_status,'\t',&i);
  extract_word_keeping_blanks(str,cpd_name,'\t',&i);
  extract_word(str,ec,'\t',&i);
  extract_word(str,ec_source,'\n',&i);
  //cout <<"\non passe  la ligne suivante"<<endl;
  return 1;
}

int line_parsing(char *str,char *rea_id,char *pat_id,char *pat_name,char *cpd_id,char *cpd_status,char *cpd_name, char *ec, char *enzyme_name, char *subunit, char *enzyme_id, char *spontaneous, char *gene_id)
{
  int i=0;
  extract_word(str,rea_id,'\t',&i);
  extract_word(str,pat_id,'\t',&i);
  extract_word_keeping_blanks(str,pat_name,'\t',&i);
  extract_word_keeping_blanks(str,cpd_id,'\t',&i);
  extract_word(str,cpd_status,'\t',&i);
  extract_word_keeping_blanks(str,cpd_name,'\t',&i);
  extract_word(str,ec,'\t',&i);
  extract_word(str,enzyme_name,'\t',&i);
  extract_word(str,subunit,'\t',&i);
  extract_word(str,enzyme_id,'\t',&i);
  extract_word(str,spontaneous,'\t',&i);
  extract_word(str,gene_id,'\t',&i);
  return 1;
}


//same reactions are supposed to follow each other
int Data::reaction_compound_enzyme_parser (char *reaction_compound_enzyme_file)
{
 
  int nblines=0;

  FILE *input;
  char *input_name=reaction_compound_enzyme_file;

  char *str=(char*)malloc((7*MAX)*sizeof(char));
  char *rea_id = (char*)malloc((MAX)*sizeof(char));
  //char *rea_name = (char*)malloc((MAX)*sizeof(char));
  char *pat_id = (char*)malloc((MAX)*sizeof(char));
  char *pat_name = (char*)malloc((MAX)*sizeof(char));
  char *cpd_id = (char*)malloc((MAX)*sizeof(char));
  char *cpd_status = (char*)malloc((MAX)*sizeof(char));
  char *cpd_name = (char*)malloc((MAX)*sizeof(char));
  char *ec = (char*)malloc((MAX)*sizeof(char));
  char *ec_source = (char*)malloc((MAX)*sizeof(char));
  //needed for optimization : so that we don't have to traverse the reactions list
  char *last_rea_id = (char*)malloc((MAX)*sizeof(char));


  Reaction *last_reaction = NULL;
  Reaction *tmp_reaction = NULL;
  Compound *current_compound = NULL;
  Pathway *current_pathway = NULL;
  Enzyme *current_enzyme = NULL;

 
  if((input=fopen(input_name,"r"))==NULL)
    {
      cout<<reaction_compound_enzyme_file<<endl;
      cout <<"Network file not found, check that it is in the right directory"<<endl;;
    }

  while ((str=fgets(str,MAX,input)) != NULL)
    {
          cerr<<"la ligne : "<<str<<endl;
      nblines++;
      if (str[0] == '\n') break;  //the last line of the file is empty, there's no treatment for this one
      line_parsing(str,rea_id,pat_id,pat_name,cpd_id,cpd_status,cpd_name,ec,ec_source);
      //traitement de la ligne
      
      //creation of reaction
      if (strcmp(rea_id,last_rea_id) != 0)
	{
	  tmp_reaction = new Reaction(rea_id);
	  reactions->push_back(tmp_reaction);
	  last_rea_id = strcpy (last_rea_id, rea_id);
	  last_reaction = tmp_reaction;
	}

      //returns a reference to the pathway if it has been created
      current_pathway = pathway_adress_book(pat_id);
      
      //creates a new pathway otherwise
      if (current_pathway == NULL)
	{
	  current_pathway = new Pathway (pat_id, pat_name);
	  pathways->push_back(current_pathway);
	}

      //returns a reference to the compound if it has been created
      current_compound = compound_adress_book(cpd_id);

      //creates a new compound otherwise
      if (current_compound==NULL)
	{
	  current_compound = new Compound(cpd_id,cpd_name);
	  compounds->push_back(current_compound);
	}

      //returns a reference to the enzyme if it has been created
      current_enzyme = enzyme_adress_book(ec);
      
      //creates a new enzyme otherwise (NA is in the enzyme list !)
      if (current_enzyme==NULL)
	{
	  current_enzyme = new Enzyme(ec);
	  enzymes->push_back(current_enzyme);
	}


      /****creation of links*****/

      last_reaction->add_pathway(current_pathway);  //the pathway is added only if it has not been added before which is verified in Reaction.add_pathway()  //this could be optimized by checking for duplicates at the end of creation of pathways

      current_pathway->add_reaction(last_reaction); //the reaction is added only if it has not been added before which is verified in Pathway.add_reaction()  //this could be optimized by checking for duplicates at the end of creation of pathways
      
      last_reaction->add_enzyme(current_enzyme); //the enzyme is added only if it has not been added before which is verified in Reaction.add_enzyme()  //this could be optimized by checking for duplicates at the end of creation of enzymes

      current_enzyme->add_reaction(last_reaction); //the reaction is added only if it has not been added before which is verified in Enzyme.add_reaction()  //this could be optimized somehow

      /***tricky stuff: adding links between compounds and reactions (links are labeled 'left' and 'right' ***/
      int which_side = last_reaction->which_side(current_compound); //if the compound has already been seen with this reaction, this int indicates on which side: left 1, right 2, undefined 3, not seen 0
      //cout<<"aprs which side"<<which_side<<endl;
      /*test of compound status*/
      /*three cases : substrate 0, product 1, NOT_FOUND or undefined 2*/

      if (test_compound_status(cpd_status) == 0) //substrate
	{
	  switch(which_side)
	    {
	    case 1 : //seen as substrate before, do nothing
	      break;
	    case 2 : //seen as a product before, keep it as product but add it to the left_right list
	      last_reaction->add_left_right_compound(current_compound);
	      current_compound->add_left_right_reaction(last_reaction);
	      break;
	    case 3 : //new substrate but seen as undefined before, add it to the left but leave it in the undefined list anyway
	      last_reaction->add_left_compound(current_compound); 
	      current_compound->add_right_reaction(last_reaction);	      
	      break;
	    case 0 : //new substrate, add it to the left
	      last_reaction->add_left_compound(current_compound);
	      current_compound->add_right_reaction(last_reaction);
	      break;
	    }
	}

      else if (test_compound_status(cpd_status) == 1)  //product
	{ 
	  switch(which_side)
	    {
	    case 1 : //seen as a substrate before, keep it as substrate but add it to the left_right list
	      //cout<<"avant pb"<<endl;
	      last_reaction->add_left_right_compound(current_compound);
	      current_compound->add_left_right_reaction(last_reaction);
	      //cout<<"aprs pb"<<endl;
	      break;
	    case 2 : //seen as product before, do nothing
	      break;
	    case 3 : //new product but seen as undefined before, add it to the right but leave it in the undefined list anyway
	      last_reaction->add_right_compound(current_compound); 
	      current_compound->add_left_reaction(last_reaction);	      
	      break;
	    case 0 : //new product, add it to the right
	      last_reaction->add_right_compound(current_compound);
	      current_compound->add_left_reaction(last_reaction);
	      break;
	    }
	}

      else //undefined or NOT FOUND
	{
	  switch(which_side)
	    {
	    case 3 : //already in undefined list
	      break;
	    default :
	      last_reaction->add_no_status_compound(current_compound);
	      current_compound->add_no_status_reaction(last_reaction);
	      break;
	    }
	}
    }

  cout<<"avant concatenate_compounds"<<endl;

  /*in case we don't use the substrate/product information, we need to merge the lists which is known here as concatenation*/
  /*by the way, we terminate creation of compounds i.e. number of edges they are responsible for*/
  /*this can only be done when all the reactions have been added*/
  list<Compound *>::iterator comp_iter = compounds->begin();
  while (comp_iter != compounds->end())
    {
      (*comp_iter)->concatenate_reactions();
      (*comp_iter)->set_edges(); //useful for building the graph
      comp_iter++;
    }

  list<Reaction *>::iterator reac_iter = reactions->begin();
  while (reac_iter != reactions->end())
    {
      (*reac_iter)->concatenate_compounds();
      reac_iter++;
    }

  //cout<<"aprs concatenate_compounds"<<endl;

  //cout <<"j'ai cr "<<compounds->size()<<" compounds, "<< pathways->size()<<" pathways, "<<reactions->size()<<" ractions et "<<enzymes->size()<<" enzymes, "<<endl;
  //cout <<"j'ai lu "<<nblines<<"lignes"<<endl;
  
  /*printage des listes pour les vrifier*/
  // cout<<"enzymes"<<endl;
//   for (list<Enzyme *>::iterator it=enzymes->begin();it!=enzymes->end();it++)
//     (*it)->print();
//   cout<<"compounds"<<endl;
//   for (list<Compound *>::iterator it=compounds->begin();it!=compounds->end();it++)
//     (*it)->print();
//   cout<<"reactions"<<endl;
//   for (list<Reaction *>::iterator it=reactions->begin();it!=reactions->end();it++)
//     (*it)->print();
  // cout<<"pathways"<<endl;
//   for (list<Pathway *>::iterator it=pathways->begin();it!=pathways->end();it++)
//     {(*it)->print();cout<<endl;}


  free(str);
  free(rea_id);
  free(pat_id);
  free(pat_name);
  free(cpd_id);
  free(cpd_status);
  free(cpd_name);
  free(ec);
  free(ec_source);
  free(last_rea_id);

  return 1;
}



//same reactions are supposed to follow each other
int Data::reaction_compound_enzyme_gene_parser(char *reaction_compound_enzyme_file)
{
  int nblines=0;

  FILE *input;
  char *input_name=reaction_compound_enzyme_file;

  /*data structures to store information we read*/
  char *str        = new char[12*MAX];
  char *rea_id     = new char[MAX];
  char *pat_id     = new char[MAX]; 
  char *pat_name   = new char[MAX];
  char *cpd_id     = new char[MAX];
  char *cpd_status = new char[MAX];
  char *cpd_name   = new char[MAX];
  char *ec         = new char[MAX];
  char *enzyme_name= new char[MAX];  
  char *subunit    = new char[MAX]; 
  char *enzyme_id  = new char[MAX];  
  char *spontaneous= new char[MAX];  
  char *gene_id    = new char[MAX];  

  /*data structures needed for optimization : so that we don't have to traverse the reactions list*/
  char *last_rea_id = (char*)malloc((MAX)*sizeof(char));
  last_rea_id[0]='\0';
  Reaction *last_reaction = NULL;
  Reaction *tmp_reaction = NULL;
  Compound *current_compound = NULL;
  Pathway *current_pathway = NULL;
  Enzyme *current_enzyme = NULL;


  if((input=fopen(input_name,"r"))==NULL)
    {
      cout<<reaction_compound_enzyme_file<<endl;
      cout <<"reaction_compound_enzyme_file not found, check that it is in the right directory"<<endl;
      delete []str;
      delete []rea_id;
      delete []pat_id;
      delete []pat_name;
      delete []cpd_id;
      delete []cpd_status;
      delete []cpd_name;
      delete []ec;
      delete []enzyme_name;
      delete []subunit;
      delete []enzyme_id;
      delete []spontaneous;
      delete []gene_id;
      exit(0);
    }
  
  while (fgets(str,7*MAX,input) != NULL)
    {
      nblines++;
      if (str[0] == '\n') break;  //the last line of the file is empty, there's no treatment for this one
      line_parsing(str,rea_id,pat_id,pat_name,cpd_id,cpd_status,cpd_name,ec,enzyme_name,subunit,enzyme_id,spontaneous,gene_id);
      
      /*traitement de la ligne*/
      //creation of reaction
      if (strcmp(rea_id,last_rea_id) != 0)
	{
	  tmp_reaction = new Reaction(rea_id);
	  reactions->push_back(tmp_reaction);
	  last_rea_id = strcpy (last_rea_id, rea_id);

	  last_reaction = tmp_reaction;
	}

      //returns a reference to the pathway if it has been created
      current_pathway = pathway_adress_book(pat_id);
      
      //creates a new pathway otherwise
      if (current_pathway == NULL)
	{
	  current_pathway = new Pathway (pat_id, pat_name);
	  pathways->push_back(current_pathway);
	}

      //returns a reference to the compound if it has been created
      current_compound = compound_adress_book(cpd_id);

      //creates a new compound otherwise
      if (current_compound==NULL)
	{
	  current_compound = new Compound(cpd_id,cpd_name);
	  compounds->push_back(current_compound);
	}

      //returns a reference to the enzyme if it has been created
      current_enzyme = enzyme_adress_book(ec);
      
      //creates a new enzyme otherwise (NA is in the enzyme list !)
      if (current_enzyme==NULL)
	{
	  current_enzyme = new Enzyme(ec);
	  enzymes->push_back(current_enzyme);
	}


      /****creation of links*****/

      last_reaction->add_pathway(current_pathway);  //the pathway is added only if it has not been added before which is verified in Reaction.add_pathway()  //this could be optimized by checking for duplicates at the end of creation of pathways

      current_pathway->add_reaction(last_reaction); //the reaction is added only if it has not been added before which is verified in Pathway.add_reaction()  //this could be optimized by checking for duplicates at the end of creation of pathways
      
      last_reaction->add_enzyme(current_enzyme); //the enzyme is added only if it has not been added before which is verified in Reaction.add_enzyme()  //this could be optimized by checking for duplicates at the end of creation of enzymes

      current_enzyme->add_reaction(last_reaction); //the reaction is added only if it has not been added before which is verified in Enzyme.add_reaction()  //this could be optimized somehow

      /***tricky stuff: adding links between compounds and reactions (links are labeled 'left' and 'right' ***/
      int which_side = last_reaction->which_side(current_compound); //if the compound has already been seen with this reaction, this int indicates on which side: left 1, right 2, undefined 3, not seen 0
      /*test of compound status*/
      /*three cases : substrate 0, product 1, NOT_FOUND or undefined 2*/
      
      if (test_compound_status(cpd_status) == 0) //substrate
	{
	  switch(which_side)
	    {
	    case 1 : //seen as substrate before, do nothing
	      break;
	    case 2 : //seen as a product before, keep it as product but add it to the left_right list
	      last_reaction->add_left_right_compound(current_compound);
	      current_compound->add_left_right_reaction(last_reaction);
	      break;
	    case 3 : //new substrate but seen as undefined before, add it to the left but leave it in the undefined list anyway
	      last_reaction->add_left_compound(current_compound); 
	      current_compound->add_right_reaction(last_reaction);	      
	      break;
	    case 0 : //new substrate, add it to the left
	      last_reaction->add_left_compound(current_compound);
	      current_compound->add_right_reaction(last_reaction);
	      break;
	    }
	}

      else if (test_compound_status(cpd_status) == 1)  //product
	{ 
	  switch(which_side)
	    {
	    case 1 : //seen as a substrate before, keep it as substrate but add it to the left_right list
	      last_reaction->add_left_right_compound(current_compound);
	      current_compound->add_left_right_reaction(last_reaction);
	      break;
	    case 2 : //seen as product before, do nothing
	      break;
	    case 3 : //new product but seen as undefined before, add it to the right but leave it in the undefined list anyway
	      last_reaction->add_right_compound(current_compound); 
	      current_compound->add_left_reaction(last_reaction);	      
	      break;
	    case 0 : //new product, add it to the right
	      last_reaction->add_right_compound(current_compound);
	      current_compound->add_left_reaction(last_reaction);
	      break;
	    }
	}

      else //undefined or NOT FOUND
	{
	  switch(which_side)
	    {
	    case 3 : //already in undefined list
	      break;
	    default :
	      last_reaction->add_no_status_compound(current_compound);
	      current_compound->add_no_status_reaction(last_reaction);
	      break;
	    }
	}
    
    }

  /*in case we don't use the substrate/product information, we need to merge the lists which is known here as concatenation*/
  /*by the way, we terminate creation of compounds i.e. number of edges they are responsible for*/
  /*this can only be done when all the reactions have been added*/
  list<Compound *>::iterator comp_iter = compounds->begin();
  while (comp_iter != compounds->end())
    {
      (*comp_iter)->concatenate_reactions();
      (*comp_iter)->set_edges(); //useful for building the graph
      comp_iter++;
    }

  
  list<Reaction *>::iterator reac_iter = reactions->begin();
  while (reac_iter != reactions->end())
    {
      (*reac_iter)->concatenate_compounds();
      reac_iter++;
    }

  //cout <<"j'ai cr "<<compounds->size()<<" compounds, "<< pathways->size()<<" pathways, "<<reactions->size()<<" ractions et "<<enzymes->size()<<" enzymes, "<<endl;
  //cout <<"j'ai lu "<<nblines<<"lignes"<<endl;
  
  /*printage des listes pour les vrifier*/
  // cout<<"enzymes"<<endl;
//   for (list<Enzyme *>::iterator it=enzymes->begin();it!=enzymes->end();it++)
//     (*it)->print();
//   cout<<"compounds"<<endl;
//   for (list<Compound *>::iterator it=compounds->begin();it!=compounds->end();it++)
//     (*it)->print();
//   cout<<"reactions"<<endl;
//   for (list<Reaction *>::iterator it=reactions->begin();it!=reactions->end();it++)
//     (*it)->print();
  // cout<<"pathways"<<endl;
//   for (list<Pathway *>::iterator it=pathways->begin();it!=pathways->end();it++)
//     {(*it)->print();cout<<endl;}

 
//note : on ne peut pas liberer 
  delete []str;

  delete []rea_id;
  delete []pat_id;
  delete []pat_name;
  delete []cpd_id;
  delete []cpd_status;
  delete []cpd_name;
  delete []ec;
  delete []enzyme_name;
  delete []subunit;
  delete []enzyme_id;
  delete []spontaneous;
  delete []gene_id;

  fclose(input);

  free(last_rea_id);

  return 1;
}


/**
 *Withdraws from the data the most frequent compounds. We suppose the compounds are ordered by frequency.
 *
 * @param ubik number of compounds to withdraw
 * @return number of node wich color is less frequent
 */
int Data::remove_frequent_compounds(int nb_frequent_compounds)
{
  list<Compound *>::iterator cpd_it=compounds->begin();
  for (int i=0;i<nb_frequent_compounds;i++)
    {
      /*we have to withdraw the compound everywhere it has been referenced*/
      /*withdraw from the reactions lists*/
      for (list<Reaction *>::iterator rea_it=(*cpd_it)->get_reactions()->begin(); rea_it!=(*cpd_it)->get_reactions()->end(); rea_it++)
	{
	  (*rea_it)->remove_compound(*cpd_it);
	}
      cpd_it++;
    }

  /*remove from the list of compounds*/
  for (int i=0;i<nb_frequent_compounds;i++)
    {
      delete(*compounds->begin());
      compounds->pop_front();
    }

  /*Remove from the array of compounds (i.e. copy and delete the old array)*/
  Compound **old_tab_compounds=tab_compounds;
  nb_compounds=compounds->size();
  tab_compounds = new Compound *[nb_compounds];
  cpd_it = compounds->begin();
  for (int i=0;i<nb_compounds;i++)
    {
      tab_compounds[i]=(*cpd_it);
      cpd_it++;
    }
  delete []old_tab_compounds;


  return 1;
}


//same reactions are supposed to follow each other
int Data::reaction_compound_parser (char *reaction_compound_file)
{
  int nblines=0;

  FILE *input;
  char *input_name=reaction_compound_file;

  char *str=(char*)malloc((7*MAX)*sizeof(char));
  char *rea_id = (char*)malloc((MAX)*sizeof(char));
  //char *rea_name = (char*)malloc((MAX)*sizeof(char));
  char *pat_id = (char*)malloc((MAX)*sizeof(char));
  char *pat_name = (char*)malloc((MAX)*sizeof(char));
  char *cpd_id = (char*)malloc((MAX)*sizeof(char));
  char *cpd_status = (char*)malloc((MAX)*sizeof(char));
  char *cpd_name = (char*)malloc((MAX)*sizeof(char));
  //needed for optimization : so that we don't have to traverse the reactions list
  char *last_rea_id = (char*)malloc((MAX)*sizeof(char));
  Reaction *last_reaction = NULL;
  Reaction *tmp_reaction = NULL;
  Compound *current_compound = NULL;
  Pathway *current_pathway = NULL;


  if((input=fopen(input_name,"r"))==NULL)
    {
      cout<<reaction_compound_file<<endl;
      cout <<"reaction_compound_file not found"<<endl;;
    }

  while ((str=fgets(str,MAX,input)) != NULL)
    {
      // cerr<<"la ligne : "<<str<<endl;
      nblines++;
      if (str[0] == '\n') break;  //the last line of the file is empty, there's no treatment for this one
      line_parsing(str,rea_id,pat_id,pat_name,cpd_id,cpd_status,cpd_name);
      //traitement de la ligne
      
      //creation of reaction
      if (strcmp(rea_id,last_rea_id) != 0)
	// if (!string_compare(rea_id,last_rea_id))
	{
	  tmp_reaction = new Reaction (rea_id);
	  reactions->push_back(tmp_reaction);
	  last_rea_id = strcpy (last_rea_id, rea_id);
	  //string_copy(last_rea_id, rea_id);
	  last_reaction = tmp_reaction;
	}

      //returns a reference to the pathway if it has been created
      current_pathway = pathway_adress_book(pat_id);
      
      //creates a new pathway otherwise
      if (current_pathway == NULL)
	{
	  current_pathway = new Pathway (pat_id, pat_name);
	  pathways->push_back(current_pathway);
	}

      //returns a reference to the compound if it has been created

      current_compound = compound_adress_book(cpd_id);

      //creates a new compound otherwise
      if (current_compound==NULL)
	{
	  current_compound = new Compound(cpd_id,cpd_name);
	  compounds->push_back(current_compound);
	}

      //creation of links
      //cout <<"je vais ajouter mon pathway  la raction"<<endl;
      last_reaction->add_pathway(current_pathway);  //the pathway is added only if it has not been added before which is verified in Reaction.add_pathway()  //this could be optimized by checking for duplicates at the end of creation of pathways

      /*test of compound status*/
      /*three cases : product 1, substrate 0, NOT_FOUND 2*/
      if (test_compound_status(cpd_status) == 1)
	{
	  last_reaction->add_right_compound(current_compound);  //we should verify here if the compound has not been added before (because of another pathway), but the redondance can be withdrawn afterwards during the concatenation phasis
	  current_compound->add_left_reaction(last_reaction);
	}
      else if (test_compound_status(cpd_status) == 0)
	{
	  last_reaction->add_left_compound(current_compound);
	  current_compound->add_right_reaction(last_reaction);
	}
      else 
	{
	  last_reaction->add_no_status_compound(current_compound);
	  current_compound->add_no_status_reaction(last_reaction);
	}
    }

  /*in case we don't use the substrate/product information, we need to merge the lists which is known here as concatenation*/
  /*by the way, we terminate creation of compounds i.e. number of edges they are responsible for*/
  /*this can only be done when all the reactions have been added*/
  list<Compound *>::iterator comp_iter = compounds->begin();
  while (comp_iter != compounds->end())
    {
      (*comp_iter)->concatenate_reactions();
      (*comp_iter)->set_edges();
      comp_iter++;
    }

  list<Reaction *>::iterator reac_iter = reactions->begin();
  while (reac_iter != reactions->end())
    {
      (*reac_iter)->concatenate_compounds();
      reac_iter++;
    }

  //cout <<"j'ai cr"<<compounds.size()<<" compounds, "<< pathways.size()<<" pathways et "<<reactions.size()<<" ractions "<<endl;
  //cout <<"j'ai lu "<<nblines<<"lignes"<<endl;
  
  free(str);
  free(rea_id);
  free(pat_id);
  free(pat_name);
  free(cpd_id);
  free(cpd_status);
  free(cpd_name);
  free(last_rea_id);

  return 1;
}

//same enzymes are supposed to follow each other (order by enzyme)
int Data::enzyme_reaction_parser (char *enzyme_reaction_file)
{
  FILE *input;
  char *input_name=enzyme_reaction_file;

  char *str=(char*)malloc((5*MAX)*sizeof(char));
  char *ec = (char*)malloc((MAX)*sizeof(char));
  char *rea_id = (char*)malloc((MAX)*sizeof(char));
    
  //needed for optimization
  char *last_ec = (char*)malloc((MAX)*sizeof(char));
  Enzyme *last_enzyme = NULL;

  if((input=fopen(input_name,"r"))==NULL)
    {
      cout << "no enzyme_reaction_file \n" <<endl;
    }
  
  while ((str=fgets(str,MAX,input)) != NULL)
    {
      if (str[0] == '\n') break;  //the last line of the file is empty, there's no treatment for this one
      line_parsing(str,ec,rea_id);

      //creation of enzyme if not created already
      if (strcmp(ec,last_ec) != 0)
	{
#ifdef __DEBUG__
	  //cout <<"je cre une enzyme"<<endl;
#endif
	  last_enzyme = new Enzyme(ec);
	  enzymes->push_back(last_enzyme);
	  last_ec = strcpy (last_ec, ec);
	}
      Reaction *tmp_reaction = reaction_adress_book(rea_id);
      if (tmp_reaction == NULL)
	cout <<"houston, we got a problem : the reaction is not in the list"<<endl;
      //creation of links
      tmp_reaction->add_enzyme(last_enzyme);
      last_enzyme->add_reaction(tmp_reaction);
    }
  free(str);
  free(ec);
  free(last_ec);
  free(rea_id);

  //cout <<"j'ai cr "<<enzymes.size()<<" enzymes"<<endl;
  //peut-tre faire des free mais attention,  tester !
  return 1;
}

int Data::frag_decomposition_parser(char *frag_decomposition)
{
  FILE *input;
  Compound *cpd=NULL;
  char *input_name=frag_decomposition;
  
  char *str=(char*)malloc((2*MAX)*sizeof(char));
  char *cpd_id = (char*)malloc((MAX)*sizeof(char));
  int *frags = (int*)malloc((NB_FRAGS)*sizeof(int));
  
  if((input=fopen(input_name,"r"))==NULL)
    {
      cout << "no frag_decomposition_file \n" <<endl;
    }
 
  while ((str=fgets(str,MAX,input)) != NULL)
    {
      if (str[0] == '\n') break;  //the last line of the file is empty, there's no treatment for this one
      line_parsing(str, cpd_id, frags);
      cpd=compound_adress_book(cpd_id);  //we could speed that up by ordering the compounds by id
      if (cpd != NULL)
	cpd->set_frags(frags);
    }

  return 1;
}

int Data::enzyme_organism_parser(char *enzyme_organism_file)
{
  FILE *input;
  char *input_name=enzyme_organism_file;
  
  char *str=(char*)malloc((2*MAX)*sizeof(char));
  char *ec = (char*)malloc((MAX)*sizeof(char));
  char *org_id = (char*)malloc((MAX)*sizeof(char));
    
//needed for optimization
  char *last_ec = (char*)malloc((MAX)*sizeof(char));
  Enzyme *last_enzyme = NULL;
  Organism *tmp_org=NULL;

  if((input=fopen(input_name,"r"))==NULL)
    {
      cout << "no frag_decomposition_file \n" <<endl;
    }
 
  while ((str=fgets(str,MAX,input)) != NULL)
    {
      //cout <<"je rentre dans le while"<<endl;
      if (str[0] == '\n') break;  //the last line of the file is empty, there's no treatment for this one
      line_parsing(str,ec,org_id);
      //cout <<"enzyme : "<<ec<<endl;
      // cout <<"org : "<<org_id<<endl;

      /*if the organism is new, we create it and get its adress ; else, we just get its adress*/
      tmp_org=organism_adress_book(org_id);
      if (tmp_org==NULL)
	{
	  tmp_org=new Organism(org_id);
	  organisms->push_back(tmp_org);
	}
      /*we get the adress of the enzyme*/
      if (strcmp(ec,last_ec) != 0)  //the enzyme read is different from the last enzyme read, then we use the adress book (this is optimized if the file is sorted by enzymes)
	{
	  last_enzyme = enzyme_adress_book(last_ec);
	  last_ec = strcpy (last_ec, ec);
	}
      /*we add the links*/
      if (last_enzyme !=NULL)
	{
	  //cout <<"je cre les liens enzyme-organismes"<<endl;
	  tmp_org->add_enzyme(last_enzyme);
	  last_enzyme->add_organism(tmp_org);
	}
    }
  free(str);
  free(ec);
  free(last_ec);
  free(org_id);

  cout <<"j'ai cr "<<organisms->size()<<" organismes"<<endl;
  //peut-tre faire des free mais attention,  tester !
  return 1;
}

/*reset function needed when the program is lauched several times with different motifs*/
void Data::reset_compounds()
{
  for (list<Compound *>::iterator it=compounds->begin();it!=compounds->end();it++)
    (*it)->reset();
}

void Data::reset_reactions()
{
  for (list<Reaction *>::iterator it=reactions->begin();it!=reactions->end();it++)
    (*it)->reset();
}

/**
 *fills tab with EC numbers cut at given threshold and stores them in
 *tab of ints. This is needed for systematic analysis (test all motifs
 *of given size) and for printing all EC numbers for building motifs.
*/
int Data::fill_tab_enzymes(int threshold, int **tab)
{
  
  /*local copy of the enzyme list*/
  list<Enzyme *> *local_list = new list<Enzyme *>;
  for (list<Enzyme *>::iterator it=enzymes->begin(); it!=enzymes->end(); it++)
    {
      local_list->push_back(*it);
    }

  /*fill tab: start with an enzyme, put it in the tab, withdraw all its friends (i.e. similar enzymes) from the list, go to the next remaining enzyme in the list and do the same until you reach the end of the list.*/
  int i=0;
  for (list<Enzyme *>::iterator it=local_list->begin(); it!=local_list->end(); it++)
    {
      if ((*it)->get_ec_size() >= threshold)
	{
	  /*copy partial EC number to tab*/
	  for (int j=0;j<threshold;j++)
	    tab[i][j]=(*it)->get_EC_number()[j];
	  for (int j=threshold;j<4;j++)
	    tab[i][j]=-1;

	  for (list<Enzyme *>::iterator it2=local_list->begin(); it2!=local_list->end(); it2++)
	    {
	      if ((*it2)->is_similar_to(*it, threshold) && it2!=it)
		{
		  it2=local_list->erase(it2);
		  it2--;
		}
	    }
	  i++;
	}
    }

  local_list->clear();
  delete(local_list);
  return i;
}



/**
 *fills tab with EC numbers cut at given threshold. This is needed for systematic analysis (test all motifs of given size)
*/
int Data::fill_tab_enzymes(int threshold, char **tab)
{
  /*local copy of the enzyme list*/
  list<Enzyme *> *local_list = new list<Enzyme *>;

  for (list<Enzyme *>::iterator it=enzymes->begin(); it!=enzymes->end(); it++)
    {
      local_list->push_back(*it);
    }

  /*fill tab: start with an enzyme, put it in the tab, withdraw all its friends (i.e. similar enzymes) from the list, go to the next remaining enzyme in the list and do the same until you reach the end of the list.*/
  int i=0;
  for (list<Enzyme *>::iterator it=local_list->begin(); it!=local_list->end(); it++)
    {
      if ((*it)->get_ec_size() >= threshold)
	{
	  (*it)->get_partial_ec(threshold,tab[i]);
	  //cout<<"tab[i] "<<tab[i]<<endl;
	  for (list<Enzyme *>::iterator it2=local_list->begin(); it2!=local_list->end(); it2++)
	    {
	      if ((*it2)->is_similar_to(*it, threshold) && it2!=it)
		{
		  it2=local_list->erase(it2);
		  it2--;
		}
	    }
	  i++;
	}
    }

  local_list->clear();
  delete local_list;
  return i;
}





/**This function fills the correspondance tab between two tabs giving
   all EC numbers cut at given threshold. The first cell of
   nb_children will give the number of cells in the lower tab (low
   threshold) correponding to the first cell in the upper tab (high
   threshold). In this method, we suppose that the list of EC numbers
   is sorted.*/
void Data::fill_nb_children(int threshold, int n, int *nb_children)
{
  /*initialization of nb_children*/
  for (int i=0;i<n;i++)
    nb_children[i]=0;

  /*local copy of the enzyme list*/
  list<Enzyme *> *local_list = new list<Enzyme *>;
  for (list<Enzyme *>::iterator it=enzymes->begin(); it!=enzymes->end(); it++)
    {
      local_list->push_back(*it);
    }

  /*fill tab: start with an enzyme, start counting, withdraw all its friends (i.e. similar enzymes) from the list, go to the next remaining enzyme in the list and do the same until you reach the end of the list.*/
  int i=0;
  for (list<Enzyme *>::iterator it=local_list->begin(); it!=local_list->end(); it++)
    {
      // cerr<<"it ";
//       (*it)->print();
//       cerr<<endl;
      if ((*it)->get_ec_size() >= threshold)
	{
	  if ((*it)->get_ec_size() >= threshold+1)
	    nb_children[i]=1;
	  //(*it)->get_partial_ec(threshold,tab[i]);
	  //  cout<<tab[i]<<endl;
	  //tab[i] = strcpy(tab[i],(*it)->get_partial_ec(threshold));
	  list<Enzyme *>::iterator current_it=it;
	  for (list<Enzyme *>::iterator it2=local_list->begin(); it2!=local_list->end(); it2++)
	    {
	     //  cerr<<"it2 ";
// 	      (*it2)->print();
// 	      cerr<<endl;

// 	      cerr<<"current it ";
// 	      (*current_it)->print();
// 	      cerr<<endl;

// 	      // if ((*it2)->is_
//	      similar_to(*it, threshold) && it2!=it)
// 		{
// 		  /*count the number of children in the tab giving enzymes with lower threshold : to be counted, the EC number has to be of length at least threshold +1 and not to be counted twice*/
// 		  if ((*it2)->get_ec_size() >= threshold+1)
// 		    nb_children[i]++;//pb : donne le nb d'EC  3 chiffres sans supprimer la redondance
// 		  if ((*it2)->is_similar_to(*current_it, threshold+1)) //here, we suppose that the list of EC numbers is sorted
// 		    nb_children[i]--;
// 		  current_it=it2;
// 		  it2=local_list->erase(it2);
// 		  it2--;
// 		}

	      if ((*it2)->is_similar_to(*it, threshold) && it2!=it)
		{
		  /*count the number of children in the tab giving enzymes with lower threshold : to be counted, the EC number has to be of length at least threshold +1 and not to be counted twice*/
		  if ((*it2)->get_ec_size() >= threshold+1)
		    nb_children[i]++;//pb : donne le nb d'EC  3 chiffres sans supprimer la redondance
		  if ((*it2)->is_similar_to(*current_it, threshold+1)) //here, we suppose that the list of EC numbers is sorted
		    nb_children[i]--;
		  current_it=it2;
		  // it2=local_list->erase(it2);
// 		  it2--;
		}


	    }
	  it=local_list->erase(it,current_it);
	  //it=local_list->erase(it);
	  //it--;
	  i++;
	}
    }

  local_list->clear();
  delete(local_list);
  //return i;
}



/*fills array containing all possible ECs one can use to build a
  motif. This array is also useful to give the correspondance between
  colors and EC numbers*/
int **Data::fill_colours()
{
  /*memory allocation*/
  //EC_numbers= new int*[MAX_NB_EC];

  int **EC_numbers1= new int*[MAX_NB_EC];
  int **EC_numbers2= new int*[MAX_NB_EC];
  int **EC_numbers3= new int*[MAX_NB_EC];
  int **EC_numbers4= new int*[MAX_NB_EC];
  for (int i=0; i<MAX_NB_EC; i++)
    {
      //EC_numbers[i]=new int[4];
      EC_numbers1[i]=new int[4];
      EC_numbers2[i]=new int[4];
      EC_numbers3[i]=new int[4];
      EC_numbers4[i]=new int[4];
    }

  
  /*first get all partial EC numbers contained in the data*/
  int nb_EC1=fill_tab_enzymes(1,EC_numbers1);
  int nb_EC2=fill_tab_enzymes(2,EC_numbers2);
  int nb_EC3=fill_tab_enzymes(3,EC_numbers3);
  int nb_EC4=fill_tab_enzymes(4,EC_numbers4);

  /*merge all tabs into one and sort it.*/
  for (int i=0;i<nb_EC1;i++)
    for(int j=0;j<4;j++)
      colours_EC[i][j]=EC_numbers1[i][j];
  for (int i=0;i<nb_EC2;i++)
    for(int j=0;j<4;j++)
      colours_EC[nb_EC1+i][j]=EC_numbers2[i][j];
  for (int i=0;i<nb_EC3;i++)
    for(int j=0;j<4;j++)
      colours_EC[nb_EC1+nb_EC2+i][j]=EC_numbers3[i][j];
  for (int i=0;i<nb_EC4;i++)
    for(int j=0;j<4;j++)
      colours_EC[nb_EC1+nb_EC2+nb_EC3+i][j]=EC_numbers4[i][j];
 
  nb_colours=nb_EC1+nb_EC2+nb_EC3+nb_EC4;

  if (nb_colours > MAX_NB_EC)
    {
      cerr<<"The number of EC_numbers is too big, revise NB_MAX_EC"<<endl;
      exit(0);
    }       


  /*sorts EC numbers in ascending order. 1 is before 1.1 and 2*/
  //sort(EC_numbers,nb_EC);
  sort(colours_EC,nb_colours);



  //cerr<<"avant init"<<endl;
  /*fill correspondance_colours 1,2,3,4. This program uses the fact that the array colours_EC is sorted (all EC numbers starting with 1 follow each other*/
  /*watch out, this method might not work if all partial EC numbers were not represented*/
  /*ex: 1.2.3 followed by 1.3.3 (there would be an error at the last digit which would be detected as similar while it is not*/
  /*initialisation*/
  correspondance_colours[0][0]=0;
  correspondance_colours[0][1]=-1;
  correspondance_colours[0][2]=-1;
  correspondance_colours[0][3]=-1;

  //cerr<<"avant fill matrix"<<endl;
  /*fill matrix*/
  for (int i=1;i<nb_colours;i++)
    {
      for (int j=0;j<4;j++)
	{
	  /*if the EC number is not long enough, then correspondance_colours is filled with -1's*/
	  if (colours_EC[i][j]==-1)
	    correspondance_colours[i][j]=-1;

	  /*if no difference, then we just copy, the father is common (EX: 1.1.1.2 follows 1.1.1.1, the three first digits will have the same father*/
	  else if (colours_EC[i-1][j]==colours_EC[i][j])
	    correspondance_colours[i][j]=correspondance_colours[i-1][j];
	  
	  /*if there is a difference, then it becomes its own father*/
	  else 
	    correspondance_colours[i][j]=i;
	}
    }
  //cerr<<"aprs fill matrix"<<endl;
  // print_mat(correspondance_colours,nb_colours,4);
//   print_mat(colours_EC,nb_colours,4);
//   exit(0);



  /*fill colours_str: correspondance between colours and EC numbers as strings*/
  /*the hard part is to turn an int to a string*/
  for (int i=0;i<nb_colours;i++)
    {
      ostringstream oss; //useful to turn int to string
      bool out=0;
      int j=0;
      while(!out && j<3)
	{
	  if (colours_EC[i][j]!=-1)
	    {
	      oss<<colours_EC[i][j];
	      if (colours_EC[i][j+1]!=-1)
		oss<<".";
	    }
	  
	  else
	    out=1;
	  
	  j++;
	}
      
      if (colours_EC[i][3]!=-1)
	oss<<colours_EC[i][3];

      /*copy oss to colours_str[i]*/
      int k=0;
      while (oss.str()[k]!='\0')
	{
	  colours_str[i][k]=oss.str()[k];
	  k++;
	}
      colours_str[i][k]='\0';
      //cout<<colours_str[i]<<endl;
      //colours_str[i]=oss.str();
      //strcpy(colours_str[i],oss.str());
      // colours_str=strcpy(
    }


  /*free memory*/
  for (int i=0; i<MAX_NB_EC; i++)
    {
      //delete[](EC_numbers[i]);
      delete[](EC_numbers1[i]);
      delete[](EC_numbers2[i]);
      delete[](EC_numbers3[i]);
      delete[](EC_numbers4[i]);
    }
 
  delete[](EC_numbers1);
  delete[](EC_numbers2);
  delete[](EC_numbers3);
  delete[](EC_numbers4);


  return colours_EC;
}



/**
 *Assign colors (integers) to all the reactions, depending on the EC numbers that catalyse them. In the end, every reactions is given between 1 and 4 colours (depending on the threshold of its EC number in the dataset. The sketch of this method is the following:
 1- traverse the true list of EC numbers, 2- get their EC_number() 3- get the color (integer) of this EC number, 4- store this value in the field colours of all the reactions catalysed by this EC number. Finally, use the correspondance_colours matrix to assign colors with lower thresholds to the reactions.
 *
 */
void Data::assign_colours_to_reactions()
{
  for (list<Enzyme *>::iterator enz_iter=enzymes->begin();enz_iter!=enzymes->end();enz_iter++)
    {
      /*color of the EC number being considered*/
      //print_tab((*enz_iter)->get_EC_number(),4);
      int col = get_index((*enz_iter)->get_EC_number(),colours_EC,nb_colours);

      for (list<Reaction *>::iterator rea_iter=(*enz_iter)->get_catalysed_reactions()->begin(); rea_iter != (*enz_iter)->get_catalysed_reactions()->end(); rea_iter++)
	{
	  /*add the colours to the reaction as well as the colours cut at different thresholds. Note that duplicates are removed in the add_colour method.*/

	  if (col==-1)  //in case the colour is NA
	    {
	      // cout<<"the color is -1"<<endl;
// 	      (*enz_iter)->print();
// 	      cout<<"the list of enzymes of this reaction is"<<endl;
// 	      for (list<Enzyme *>::iterator enz_iter2=(*rea_iter)->get_enzymes()->begin();enz_iter2!=(*rea_iter)->get_enzymes()->end();enz_iter2++)
// 		(*enz_iter)->print();
// 	      cout<<endl;

	      (*rea_iter)->add_colour(-1);
	    }

	  /*add all the colours corresponding to the EC cut at all possible thresholds*/
	  else
	    for (int j=0;j<4;j++)
	      if (correspondance_colours[col][j] != -1)
		(*rea_iter)->add_colour(correspondance_colours[col][j]);
	}

    }

}

/*prints all possible ECs you can use to build your motif. The list is sorted.*/
void Data::print_possible_ecs(std::ostream& output)
{
  /*sort EC numbers, 1 before 1.1 and before 2*/
  sort(colours_EC,nb_colours);


  /*prints EC numbers without printing -1's*/
  for (int i=0;i<nb_colours;i++)
    {
      bool out=0;
      int j=0;
      while(!out && j<3)
	{
	  if (colours_EC[i][j]!=-1)
	    {
	      output<<colours_EC[i][j];
		  if (colours_EC[i][j+1]!=-1)
		    output<<".";
	    }
	  
	  else
	    out=1;
	  
	  j++;
	}
      
      if (colours_EC[i][3]!=-1)
	output<<colours_EC[i][3]<<endl;
      else
	output<<endl;

    }
}


/*print functions: prints the EC numbers of the dataset cut at a given threshold*/
/*print functions: this function looks very strange cos the iterator is incremented inside the loop*/
void Data::print_enzymes(int threshold, char *file_name)
{
  FILE *output;
  output=fopen(file_name, "w");
  for (list<Enzyme *>::iterator it=enzymes->begin(); it!=enzymes->end(); it++)
    {
      if ((*it)->print(threshold, output))
	fprintf(output, "\n");
      it++;
    }
}

/*test if the reaction is already in the reaction_list*/
/*this function is already in Metabolic Network, we should do something with templates*/
bool in_list3(Reaction *reaction_to_add, list<Reaction *> *rea_list)
{
  list<Reaction *>::iterator it = rea_list->begin();
  while (it != rea_list->end())
    {
      if (reaction_to_add == *it)
	return true;
      it++;
    }
  return false;
}


void Data::print_enzyme_arity(int threshold, char *file_name)
{
  /*local copy of the enzyme list*/
  list<Enzyme *> *local_list = new list<Enzyme *>;
  list<Reaction *> *rea_list = new list<Reaction *>;
  for (list<Enzyme *>::iterator it=enzymes->begin(); it!=enzymes->end(); it++)
    local_list->push_back(*it);
    
  /*ID : you take the first enzyme from the list, you get all the enzymes that are similar to this one and you add the arity => you get the final arity and you withdraw all the tested enzymes from the list*/ 
  FILE *output;
  output=fopen(file_name, "w");
  int arity=0;
  for (list<Enzyme *>::iterator it=local_list->begin(); it!=local_list->end(); it++)
    {
      if ((*it)->print(threshold, output))
	{
	  arity=0;
	  rea_list->clear();
	  for (list<Enzyme *>::iterator it2=local_list->begin(); it2!=local_list->end(); it2++)
	    {
	      if ((*it2)->is_similar_to(*it, threshold) && it2!=it)
		{
		  for (list<Reaction *>::iterator it3=(*it2)->get_catalysed_reactions()->begin(); it3!=(*it2)->get_catalysed_reactions()->end(); it3++)
		    {
		      if (!in_list3(*it3,rea_list))
		      {
			rea_list->push_back(*it3);
			arity++;
		      }
		    }
		  local_list->erase(it2);
		  it2--;
		}
	    }
	  /*calculate the arity of the first loop reaction*/
	  for (list<Reaction *>::iterator it3=(*it)->get_catalysed_reactions()->begin(); it3!=(*it)->get_catalysed_reactions()->end(); it3++)
	    {
	      if (!in_list3(*it3,rea_list))
		{
		  rea_list->push_back(*it3);
		  arity++;
		}
	    }
	  local_list->erase(it);
	  it--;
	  fprintf(output," %i\n",arity);
	}
    }
  fclose(output);
  local_list->clear();
  delete(local_list);
  delete(rea_list);
}

void Data::print_compound_arity(char *compound_arity)
{
  FILE *output;
  char *output_name=compound_arity;
  output=fopen(output_name, "w");
  cout <<"let's print the compound arity"<<endl;
  list<Compound *>::iterator comp_iter = compounds->begin();
  while(comp_iter != compounds->end())
    {
      fprintf(output, "%s\t",(*comp_iter)->get_id());
      fprintf(output, "%d\t", (*comp_iter)->get_nb_reactions());
      fprintf(output, "%d\t", (*comp_iter)->arity());
      fprintf(output, "%s",(*comp_iter)->get_name());
      fprintf(output, "\n");
      comp_iter++;
    }
  fclose(output);
}

void Data::print_reaction_colours()
{
  for (list<Reaction *>::iterator it=reactions->begin();it!=reactions->end();it++)
    {
      (*it)->print();
      (*it)->print_colours();
    }
}

/**
 *Prints general information on the data. Prints number of reactions, EC numbers, compounds, pathways, number of reactions with several enzymes... also prints the number of multiple labels when the threshold for comparing enzymes is set to 3.
 */
void Data::print_identity_card()
{
  cout<<"number of reactions : "<<reactions->size()<<endl;
  cout<<"number of compounds : "<<compounds->size()<<endl;
  cout<<"number of EC numbers : "<<enzymes->size()<<endl;
  cout<<"number of pathways : "<<pathways->size()<<endl;

  int rea0=0;
  int rea1=0;
  int rea2=0;
  int rea2bis=0;
  int rea2ter=0;
  int rea3=0;
  int rea3bis=0;
  int rea3ter=0;
  int rea4=0;
  int rea4bis=0;
  int rea4ter=0;
  for (list<Reaction *>::iterator rea_iter=reactions->begin();rea_iter!=reactions->end();rea_iter++)
    {
      list<Enzyme *>::iterator first_enzyme=(*rea_iter)->get_enzymes()->begin();
      list<Enzyme *>::iterator enz_iter=first_enzyme++;
      bool ok=0;
      switch((*rea_iter)->get_enzymes()->size())
	{
	case 0 : rea0++;
	  break;
	case 1 : rea1++;
	  /*in case the enzyme is fictitious (NA)*/
	  if (!strcmp("NA",(*(*rea_iter)->get_enzymes()->begin())->get_ec()))
	    {
	      rea1--;
	      rea0++;
	    }
	  break;
	case 2 : rea2++;
	  /*computation of the number of bicolor reactions that become unicolor when threshold is 3*/
	  ok=1;
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 3));
	      enz_iter++;
	    }
	  rea2bis+=ok;

	  /*computation of the number of bicolor reaction that become unicolor when threshold is 2*/
	  ok=1;
	  enz_iter=(*rea_iter)->get_enzymes()->begin();
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 2));
	      enz_iter++;
	    }
	  rea2ter+=ok;

	  break;
	case 3 : rea3++;
	  ok=1;
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 3));
	      enz_iter++;
	    }
	  rea3bis+=ok;	

	  ok=1;
	  enz_iter=(*rea_iter)->get_enzymes()->begin();
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 2));
	      enz_iter++;
	    }
	  rea3ter+=ok;
	  break; 
	default : rea4++;
	  ok=1;
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 3));
	      enz_iter++;
	    }
	  rea4bis+=ok;

	  ok=1;
	  enz_iter=(*rea_iter)->get_enzymes()->begin();
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 2));
	      enz_iter++;
	    }
	  rea4ter+=ok;
	  // break; 
	  break; 
	}
    }
  cout<<"number of reactions with no EC number :"<<rea0  <<endl;
  cout<<"number of reactions with 1 EC number :"<<rea1  <<endl;
  cout<<"number of reactions with 2 EC numbers :"<<  rea2<<endl;
  cout<<"number of reactions with 3 EC numbers :"<<  rea3<<endl;
  cout<<"number of reactions with more than 3 EC numbers :"<<  rea4<<endl;
  cout<<endl;
  cout<<"To sum up :"<<endl;
  cout<<"With a threshold of 4"<<endl;
  cout<<"|"<<rea0<<"|"<<rea1<<"|"<<rea2<<"|"<<rea3<<"|"<<rea4<<"|"<<endl;
  //cout<<endl;
  cout<<"With a threshold of 3"<<endl;
  cout<<"|"<<rea0<<"|"<<rea1+rea2bis+rea3bis+rea4bis<<"|"<<rea2-rea2bis<<"|"<<rea3-rea3bis<<"|"<<rea4-rea4bis<<"|"<<endl;

  cout<<"With a threshold of 2"<<endl;
  cout<<"|"<<rea0<<"|"<<rea1+rea2ter+rea3ter+rea4ter<<"|"<<rea2-rea2ter<<"|"<<rea3-rea3ter<<"|"<<rea4-rea4ter<<"|"<<endl;



  int ec0=0;
  int ec1=0;
  int ec2=0;
  int ec3=0;
  int ec4=0;

  /*print size of EC numbers*/
  for (list<Enzyme *>::iterator it=enzymes->begin();it!=enzymes->end();it++)
    {
      // (*it)->print();
//       cout<<endl;
      int size=(*it)->get_ec_size();
      switch(size)
	{
	case 0 : ec0++;
	  break;
	case 1 : ec1++;
	  break;
	case 2 : ec2++;
	  break;
	case 3 : ec3++;
	  break; 
	case 4 : ec4++;
	  break; 
	}
    }
  cout<<"Voici les tailles des EC numbers, attention, on ne donne pas les multiplicits associes (il peut y avoir plusieurs 1.1 alors que cela ne compte que pour 1 dans ce tableau"<<endl;
  cout<<"|"<<ec0<<"|"<<ec1<<"|"<<ec2<<"|"<<ec3<<"|"<<ec4<<"|"<<endl;
  
  int nb_edges=0;
  for (list<Compound *>::iterator it = compounds->begin(); it!=compounds->end(); it++)
    nb_edges+=(*it)->get_nb_edges();
  cout<<"le nombre d'artes du graphe calcul  partir des composs est :"<<nb_edges<<endl;
}



/**
 *Prints general information on the data. Prints number of reactions, EC numbers, compounds, pathways, number of reactions with several enzymes... also prints the number of multiple labels when the threshold for comparing enzymes is set to 3.
 */
void Data::print_identity_card(std::ostream& output)
{
  output<<"number of reactions : "<<reactions->size()<<endl;
  output<<"number of compounds : "<<compounds->size()<<endl;
  output<<"number of enzymes (EC numbers): "<<enzymes->size()<<endl;
  output<<"number of pathways : "<<pathways->size()<<endl;
  output<<endl;
  int rea0=0;
  int rea1=0;
  int rea2=0;
  int rea2bis=0;
  int rea2ter=0;
  int rea3=0;
  int rea3bis=0;
  int rea3ter=0;
  int rea4=0;
  int rea4bis=0;
  int rea4ter=0;
  for (list<Reaction *>::iterator rea_iter=reactions->begin();rea_iter!=reactions->end();rea_iter++)
    {
      list<Enzyme *>::iterator first_enzyme=(*rea_iter)->get_enzymes()->begin();
      list<Enzyme *>::iterator enz_iter=first_enzyme++;
      bool ok=0;
      switch((*rea_iter)->get_enzymes()->size())
	{
	case 0 : rea0++;
	  break;
	case 1 : rea1++;
	  /*in case the enzyme is fictitious (NA)*/
	  if (!strcmp("NA",(*(*rea_iter)->get_enzymes()->begin())->get_ec()))
	    {
	      rea1--;
	      rea0++;
	    }
	  break;
	case 2 : rea2++;
	  /*computation of the number of bicolor reactions that become unicolor when threshold is 3*/
	  ok=1;
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 3));
	      enz_iter++;
	    }
	  rea2bis+=ok;

	  /*computation of the number of bicolor reaction that become unicolor when threshold is 2*/
	  ok=1;
	  enz_iter=(*rea_iter)->get_enzymes()->begin();
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 2));
	      enz_iter++;
	    }
	  rea2ter+=ok;

	  break;
	case 3 : rea3++;
	  ok=1;
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 3));
	      enz_iter++;
	    }
	  rea3bis+=ok;	

	  ok=1;
	  enz_iter=(*rea_iter)->get_enzymes()->begin();
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 2));
	      enz_iter++;
	    }
	  rea3ter+=ok;
	  break; 
	default : rea4++;
	  ok=1;
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 3));
	      enz_iter++;
	    }
	  rea4bis+=ok;

	  ok=1;
	  enz_iter=(*rea_iter)->get_enzymes()->begin();
	  while (ok && enz_iter!=(*rea_iter)->get_enzymes()->end())
	    {
	      ok = ((*enz_iter)->is_similar_to(*first_enzyme, 2));
	      enz_iter++;
	    }
	  rea4ter+=ok;
	  break; 
	}
    }
  output<<"number of reactions with no EC number :"<<rea0  <<endl;
  output<<"number of reactions with 1 EC number :"<<rea1  <<endl;
  output<<"number of reactions with 2 EC numbers :"<<  rea2<<endl;
  output<<"number of reactions with 3 EC numbers :"<<  rea3<<endl;
  output<<"number of reactions with more than 3 EC numbers :"<<  rea4<<endl;
  output<<endl;

  int ec0=0;
  int ec1=0;
  int ec2=0;
  int ec3=0;
  int ec4=0;

  /*print size of EC numbers*/
  for (list<Enzyme *>::iterator it=enzymes->begin();it!=enzymes->end();it++)
    {
      int size=(*it)->get_ec_size();
      switch(size)
	{
	case 0 : ec0++;
	  break;
	case 1 : ec1++;
	  break;
	case 2 : ec2++;
	  break;
	case 3 : ec3++;
	  break; 
	case 4 : ec4++;
	  break; 
	}
    }
  // output<<"Voici les tailles des EC numbers, attention, on ne donne pas les multiplicits associes (il peut y avoir plusieurs 1.1 alors que cela ne compte que pour 1 dans ce tableau"<<endl;
//   output<<"|"<<ec0<<"|"<<ec1<<"|"<<ec2<<"|"<<ec3<<"|"<<ec4<<"|"<<endl;
  
}


void Data::print_nb_compounds_per_reaction()
{
  for (list<Reaction *>::iterator it=reactions->begin(); it!=reactions->end(); it++)
    {
      (*it)->print_nb_compounds();
      cout<<endl;
    }
}

void Data::print_nb_reactions_per_compound()
{
  for (list<Compound *>::iterator it=compounds->begin(); it!=compounds->end(); it++)
    {
      (*it)->print_nb_reactions();
      cout<<endl;
    }
}

void Data::print_pathway_reaction(std::ostream& output)
{
  for (list<Pathway *>::iterator pat_it=pathways->begin(); pat_it!=pathways->end(); pat_it++)
    {
      for (list<Reaction *>::iterator rea_it=(*pat_it)->get_reactions()->begin(); rea_it!=(*pat_it)->get_reactions()->end(); rea_it++)
	{
	  (*pat_it)->print(output);
	  output<<"\t";  
	  (*rea_it)->print(output);  
	  output<<endl;
	}
      output<<endl;
    }
}

/*prints the list of reactions with for each reaction the pathway it is involved in*/
/*each line of the file is a tuple reaction pathway. A reaction can be in several pathways*/
void Data::print_reaction_pathway(std::ostream& output)
{
  for (list<Reaction *>::iterator it=reactions->begin(); it!=reactions->end(); it++)
    {
      (*it)->print_reaction_pathways(output);  //print reaction and pathways as couples. A reaction is repeated several times if it is involved in several pathways. (easier to parse)
      output<<endl;
    }
}

void Data::print_reaction_enzyme(std::ostream& output)
{
  for (list<Reaction *>::iterator it=reactions->begin(); it!=reactions->end(); it++)
    {
      (*it)->print_reaction_enzymes(output); //print reaction and enzymes that catalyse the reaction
      output<<endl;
    }
}

// void Data::print_enzyme_reaction(std::ostream& output)
// {
//   for (list<Enzyme *>::iterator it=enzymes->begin(); it!=enzymes->end(); it++)
//     {
//       (*it)->print_enzyme_reactions(output); //print enzyme and reactions. The enzyme is repeated several times if it catalyses several reactions.
//       output<<endl;
//     }
// }

void Data::print_enzyme_reaction(std::ostream& output)
{
  for (list<Enzyme *>::iterator enz_it=enzymes->begin(); enz_it!=enzymes->end(); enz_it++)
    {
      for (list<Reaction *>::iterator rea_it=(*enz_it)->get_catalysed_reactions()->begin(); rea_it!=(*enz_it)->get_catalysed_reactions()->end(); rea_it++)
	{
	  (*enz_it)->print(output);
	  output<<"\t";  
	  (*rea_it)->print(output);  
	  output<<endl;
	}
      output<<endl;
    }
}
