/* Enzyme.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <iostream>
#include "../include/Enzyme.h"
#include <cstdio>
#include <map>
#include <cstring>
#include <cstdlib>

//extern map<void*,bool> enzymePtrs;

//this function is identical to the extract_word function in Data.cpp, it would be a good idea to put them in a common file (?constantes.h renamed) or to do something like linking
//on note qu'il y a une condition supplmentaire dans la sortie de la boucle while
void extract_word2(char *str, char *word, char end_of_word, int *i)
{
  //extracts a word from a line, whithdrawing the blanks
  int j=0;
  while (str[*i] != '\0' && str[*i] != end_of_word)
    {
      if (str[*i]!=' ') 
	{
	  word[j]=str[*i];
	  (*i)++;
	  j++;
	}
      else  (*i)++;
    }
  word[j]='\0';
  if(str[*i] != '\0')
    (*i)++;
}


/*returns true if a number has been extracted and false otherwise (meaning the EC number is partial*/
bool extract_word3(char *str, char *word, char end_of_word, int *i)
{
  int j=0;
  //extracts a word from a line
  if ((*i)>0)
    if (str[(*i)-1] == '\0')
      return 0;

  if (str[*i] == '\0' )
    return 0;
  else 
    {
      while (str[*i] != '\0' && str[*i] != end_of_word)
	{
     	  word[j]=str[*i];
	  (*i)++;
	  j++;
	}
      word[j]='\0';
      if(str[*i] != '\0')
	(*i)++;
    }
  return 1;
}



/*Enzymes with NA are created but they are given a whole -1 EC number*/
Enzyme::Enzyme(char *ec_str)
{
  //memory verification
  //  enzymePtrs[this] = false;

  //we keep the EC number as a string too
  ec = (char*)malloc((strlen(ec_str)+1)*sizeof(char));
  ec = strcpy(ec,ec_str);

  catalysed_reactions = new list <Reaction *>;
  organisms = new list <Organism *>;

  for (int i=0;i<4;i++)
    EC_number[i]=0;
  
  //and we cast it to an array of int
  //int offset = 48;  //needed when casting char into int
  int i=0;
  char *tmp = (char*)malloc(4*sizeof(char)); //one number of the ec_number
  
  for (int j=0;j<4;j++)
    {
      if (!extract_word3(ec_str,tmp,'.',&i) || !strcmp(ec_str,"NA"))
	//case when the enzyme has fewer numbers, we add this '-' symbol which is turned into -1 later on
     	{
	  tmp[0]='-';
	  tmp[1]='\0';
	}
      if (tmp[0]=='-')
	EC_number[j] =-1;
      else	
	EC_number[j] = atoi(tmp);
    }
  free(tmp);
}


Enzyme::~Enzyme()
{ 
  //catalysed_reactions->clear();
  delete(catalysed_reactions);
  
  // organisms->clear();
  delete(organisms);
  //delete[](EC_number);
  //free(EC_number);
  free(ec);
}

list <Reaction *> * Enzyme::get_catalysed_reactions()
{
  return catalysed_reactions;
}

/**adds a reaction to the list of reactions checking that it is not already in the list.
 */
int Enzyme::add_reaction(Reaction *reaction_to_add)
{
  list<Reaction *>::iterator rea_iter;
  rea_iter = catalysed_reactions->begin();
  while (rea_iter != catalysed_reactions->end())
    {
      if (reaction_to_add == *rea_iter)
	return 0;
      rea_iter++;
    }
  catalysed_reactions->push_back(reaction_to_add);
  return 1;
}

void Enzyme::add_organism(Organism *org_to_add)
{
  organisms->push_back(org_to_add);
}

char *Enzyme::get_ec()
{
  return ec;
}

int *Enzyme::get_EC_number()
{
  return EC_number;
}

/**
 *returns the number of digits known for this EC number
 */
int Enzyme::get_ec_size()
{
  int i=0;
  while (EC_number[i]!=-1 && i<4)
    i++;
  return i;
}

/**
 *returns a string corresponding to the first threshold numbers of the
 *EC number, without the final dot. This method is called only when we
 *know that the size of the EC number in the data is larger than the
 *threshold. (Ex: it won't be called to extract a partial EC number of
 *size 2 on an EC number for which we only know the first number. This
 *method seems to have a memory bug detected by Valgrind. A possible
 *reason is that partial_ec is not initialised. The real problem is
 *when we extract a number that has exactly the length of the
 *threshold.

 * @param threshold number of digits to consider in the EC number
 * @param partial_ec return value for this method
 */
void Enzyme::get_partial_ec(int threshold, char *partial_ec)
{
  int nb_dots=0;
  int i=0;
  //partial_ec=strcpy(partial_ec,ec);
  //cout<<"ec : "<<ec<<endl;

  // while (nb_dots<threshold)
//     {
//       cout<<"ec[i] "<<ec[i]<<endl; 
//       cout<<"i "<<i<<endl;
//       if (ec[i]=='.')
// 	nb_dots++;
//       i++;
//     }
//   partial_ec[i-1]='\0'; //termination signal of a string in C (we add this instead of the last dot)
//   cout<<"partial_ec :"<<partial_ec<<endl;

  while (nb_dots<threshold && ec[i]!='\0')
    {
      // if (ec[i]=='\0')
// 	{i++;break;}
      partial_ec[i]=ec[i];
      if (ec[i]=='.')
	nb_dots++;
      i++;
    }

  if (nb_dots==threshold)
    partial_ec[i-1]='\0'; //termination signal of a string in C (we add this instead of the last dot)


  if (ec[i]=='\0')
    partial_ec[i]='\0';
}

// /*if the number is not specified (-1), it is interpreted as a joker*/
// /*the joker matches all numbers*/
// bool Enzyme::is_similar_to(Enzyme *enzyme_to_compare, int threshold)
// {
//   for (int i=0;i<threshold;i++)
//     {
//       if (EC_number[i] != -1 && enzyme_to_compare->get_EC_number()[i] != -1)
// 	if (EC_number[i] != enzyme_to_compare->get_EC_number()[i])
// 	  return 0;
//     }
//   return 1;
// }

/*if the number is not specified (-1), it is interpreted as a joker*/
/*the joker matches no other number*/
bool Enzyme::is_similar_to(Enzyme *enzyme_to_compare, int threshold)
{
  for (int i=0;i<threshold;i++)
    {
      //if (EC_number[i] != -1 && enzyme_to_compare->get_EC_number()[i] != -1)
      if (EC_number[i] != enzyme_to_compare->get_EC_number()[i] || EC_number[i] == -1 || enzyme_to_compare->get_EC_number()[i] == -1)
	return 0;
    }
  return 1;
}

/*in this version of the comparison, the threshold is implicitely given by the incompleteness of the enzyme from the motif*/
/*the joker matches no other number*/
bool Enzyme::is_similar_to(Enzyme *motif_enzyme)
{
  int i=0;
  while (motif_enzyme->get_EC_number()[i] != -1 && i != 4)
    {
      if (EC_number[i] != motif_enzyme->get_EC_number()[i] || EC_number[i] == -1)
	return 0;
      i++;
    }
  return 1;
}

int Enzyme::arity()
{
  return (int)catalysed_reactions->size();
}

bool Enzyme::print(int threshold, FILE *output)
{
  int i=0;
  if (EC_number[threshold-1]!=-1)  //we only print enzymes that have more than threshold numbers (ex : we don't print 1.1 if threshold is 3)
    {
      for(i=0;i<threshold-1;i++)
	{
	  fprintf(output, "%i.", EC_number[i]);
	}
      fprintf(output, "%i", EC_number[i]);
      return true;
    }
  else
    return false;
}

void Enzyme::print(int threshold,std::ostream& output)
{
  int i=0;
  if (EC_number[threshold-1]!=-1)  //we only print enzymes that have more than threshold numbers (ex : we don't print 1.1 if threshold is 3)
    {
      for(i=0;i<threshold-1;i++)
	{
	  output<<EC_number[i]<<".";
	  //fprintf(output, "%i.", EC_number[i]);
	}
      output<<EC_number[i];
      //fprintf(output, "%i", EC_number[i]);
      //return true;
    }
  else
    output<<"NA";
}

/*print the enzyme in a file*/
void Enzyme::print(std::ostream& output)
{
  output<<ec;  
//fprintf(output, "%s", ec);
}

void Enzyme::print()
{
  cout <<ec;
  //for(int i=0;i<4;i++) cout<<EC_number[i]<<" ";
}


// /*prints to file the reaction and pathways in which this reaction is included*/
// void Enzyme::print_enzyme_reactions(ostream& output)
// {
//   for (list<Reaction *>::iterator iter=catalysed_reactions->begin(); iter!=catalysed_reactions->end(); iter++)
//     {
//       output<<ec;
//       output<<"\t";
//       (*iter)->print(output);
//       output<<endl;
//     }
// }
