/* Graph.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include "../include/Graph.h"
#include <iostream>
#include <math.h>
#include "../include/my_library.h"
#include <cstring>
#include <cstdlib>
#include <cstdio>

/*counts the number of lines in the graph files, skipping empty lines, which gives finally the number of nodes*/
int read_nb_nodes(char *graph_file)
{
  int n = 0; /* nombre de lignes lues */
  char ligne[256];
  FILE *f = fopen( graph_file, "rt" ); /* ouvre en lecture */
  if (f==NULL) {
    cout<<"erreur ouverture de "<<graph_file<<endl;
    //printf("erreur ouverture de\n");
    return 1;        /* sort du programme (code erreur 1) */
  }
  while (fgets( ligne, 256, f ) != NULL) { /* essai lecture ligne */
    if (ligne[0] != '\n')
      n++;                                 
  }
  //printf("%d lignes lues\n", n);    /* affiche */
  fclose(f);  /* ferme le fichier */
  return n;   
}


int read_graph(char *graph_file,bool **adjacency_matrix,int nb_nodes)
{
  //char *str=(char*)malloc((nb_nodes+2)*sizeof(char));
  char *str= new char[nb_nodes+2];
  FILE *f = fopen(graph_file, "rt" ); /* ouvre en lecture */
  if (f==NULL) 
    {
      cout<<"erreur ouverture de "<<graph_file<<endl;
      //printf("erreur ouverture\n");
      delete []str;
      return 0;        /* sort du programme (code erreur 1) */
    }
  int nb_line=0;
  while (fgets( str, nb_nodes+2, f) != NULL) 
    { /* essai lecture ligne */
      if (str[0] == '\n') break;  //the last line of the file is empty, there's no treatment for this one

      for (int i=0;i<nb_nodes;i++)     
	adjacency_matrix[nb_line][i]=!(str[i]=='0');  
      //print_mat(adjacency_matrix,nb_nodes,nb_nodes);
      nb_line++;  
    }
  delete []str;
  fclose(f);  /* ferme le fichier */
  return 1;
}

/*new version when graph is a list of edges*/
/*watch out, this does not work when graph is in mfinder format*/
int read_graph2(char *graph_file,bool **adjacency_matrix,int nb_nodes)
{
  int rc=0;
  int source, target;
  FILE *f = fopen(graph_file, "rt" ); /* ouvre en lecture */
  if (f==NULL) 
    {
      cout<<"could not open file : "<<graph_file<<endl;
      return 0;        /* sort du programme (code erreur 1) */
    }

  while ( (rc = fscanf( f,"%d %d \n", &source, &target) != EOF) )

    { /* essai lecture ligne */
      if (source >= nb_nodes || target >= nb_nodes)
	{
	  cerr<<"ERROR: Nodes should be numbered from 1 to n or labels file must be incomplete, number of lines should be equal to number of nodes"<<endl;
	  exit(0);
	}
      adjacency_matrix[source][target]=1;
      adjacency_matrix[target][source]=1;

    }
  fclose(f);  /* ferme le fichier */
  return 1;
}




/**
 *For each successive tabulation or blank in the string given in argument, move by one the position i of the string.
 * @param str the string to parse
 * @param i index to start the parsing
 */
void parseBlank(char* str, int *i){
  while(str[*i]=='\t' || str[*i]==' '){
    (*i)++;
  }
  if(str[*i]=='\0' || str[*i] == '\n' || str[*i] == '\r')
    *i=-1;
}


/**
 *
 *
 */
void parseEnzymeLabel(char* str, int* i, char ** value){
  int k=0;
  int start = *i;
  while(isalnum(str[*i]) || str[*i]=='.' || str[*i]=='-'){
    (*i)++;
    k++;
  }
 
  strncpy(*value,str+start,k+1);
  (*value)[k] = '\0';
}

/**
 *Read labels and interpret them as EC numbers
 *
 */
int read_labels(char *labels_file,list<int> **colors, char **EC_numbers,int nb_EC)
{
  char *str =  new char[MAX];
  //char *str_short = new char[MAX];
  char *label = new char[MAX];

  if(labels_file==NULL)
    {
      cerr<<"Fichier invalide : "<<labels_file <<endl;
      return 0;
    }
  FILE *f = fopen(labels_file, "rt" ); /* ouvre en lecture */
  if (f==NULL){
      cout<<"erreur ouverture de "<<labels_file<<endl;
      delete []str;
      //delete []str_short;
      //printf("erreur ouverture\n");
      return 0;        /* sort du programme (code erreur 1) */
    }
  
  int nb_line=0;
  while (fgets( str, MAX, f) != NULL){
    int i=0;
    while(i!=-1){
      parseBlank(str,&i);
      if(i == -1){
	break;
      }
      parseEnzymeLabel(str,&i,&label);
      
      if(strcmp(label,"NA")!=0)
	colors[nb_line]->push_back(get_index(label,EC_numbers,nb_EC));
      else 
	colors[nb_line]->push_back(-1);   
      
    }
    nb_line++;
  }

  fclose(f);  
  delete[]label;
  delete []str;
  //delete []str_short;
  return 1;
}


/**
 *Read labels and interpret them as integers
 *
 */
int read_labels2(char *labels_file,list<int> **colors)
{
  char *str =  new char[MAX];
  // char *str_short = new char[MAX];
  char *label = new char[MAX];

  if(labels_file==NULL)
    {
      cerr<<"Fichier invalide : "<<labels_file <<endl;
      return 0;
    }
  FILE *f = fopen(labels_file, "rt" ); /* ouvre en lecture */
  if (f==NULL){
      cout<<"error when opening "<<labels_file<<endl;
      delete []str;
      //delete []str_short;
      //printf("erreur ouverture\n");
      return 0;        /* sort du programme (code erreur 1) */
    }
  
  int nb_line=0;
  while (fgets( str, MAX, f) != NULL){
    int i=0;
    while(i!=-1){
      parseBlank(str,&i);
      //cout<<"i= "<<i<<endl;
      if(i == -1){
	break;
      }
      
      
      parseEnzymeLabel(str,&i,&label);
      
      int col=atoi(label);

     //  cout<<"col is "<<col<<endl;
//       cout<<"nb line is "<<nb_line<<endl;

      colors[nb_line]->push_back(col);
      
      // if(strcmp(label,"NA")!=0)
// 	colors[nb_line]->push_back(get_index(label,EC_numbers,nb_EC));
//       else 
// 	colors[nb_line]->push_back(-1);   
    }
    nb_line++;
    
  }

  fclose(f);  
  delete[]label;
  delete []str;
  //delete []str_short;
  return 1;
}


/*converts matrix to list*/
void convert(bool **adjacency_matrix, list<int> **adjacency_list, int nb_nodes)
{
  for (int i=0;i<nb_nodes;i++)
    for (int j=i;j<nb_nodes;j++)
      {
	if (adjacency_matrix[i][j])
	  {
	    adjacency_list[i]->push_back(j);
	    adjacency_list[j]->push_back(i);
	  }

      }
}




void Graph::reset_colors(int *correspondance,int n)
{
  for (int i=0;i<nb_nodes;i++){
      for (list<int>::iterator it=colors[i]->begin();it!=colors[i]->end();it++){
	  *it=correspondance[*it];
	}
    }
}

void Graph::reset_colors(int **correspondance,int threshold)
{
  for (int i=0;i<nb_nodes;i++){
      for (list<int>::iterator it=colors[i]->begin();it!=colors[i]->end();it++){
	  *it=correspondance[*it][threshold-1];
	}
    }
}


/* Constructor for Reaction_Graph */
Graph::Graph(int nb_vertex) {

  nb_nodes = nb_vertex;

  /*allocate memory*/
  //adjacency_matrix = initialize_matrix(adjacency_matrix, nb_nodes);
  adjacency_matrix = new bool *[nb_nodes];
  adjacency_list = new list<int> *[nb_nodes];
  colors = new list<int> *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    {
      adjacency_list[i]=new list<int>;
      colors[i]=new list<int>;
      adjacency_matrix[i] = new bool[nb_nodes];
      for (int j=0;j<nb_nodes;j++)
	adjacency_matrix[i][j]=0;
    }
  parent_nodes=NULL;
}

/**
 *Constructor needed when the topology is specified as a file.
 *
 * @param graph_file file giving the graph as an adjacency matrix
 * @param labels_file file giving the labels of the graph as a list of EC numbers
 * @param EC_numbers array of authorized EC numbers, the index of the array is taken as the colour of the node
 * @param nb_EC size of the array = number of authorized EC numbers
 */
Graph::Graph(char *graph_file, char *labels_file, char **EC_numbers, int nb_EC)
{
  /*get size of the graph*/
  nb_nodes = read_nb_nodes(graph_file);

  /*allocate memory*/
  adjacency_matrix = initialize_matrix(adjacency_matrix, nb_nodes);
  adjacency_list = new list<int> *[nb_nodes];
  colors = new list<int> *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    {
      adjacency_list[i]=new list<int>;
      colors[i]=new list<int>;
    }
  parent_nodes=NULL;

  /*fill data*/
  read_graph(graph_file,adjacency_matrix,nb_nodes);
  read_labels(labels_file, colors, EC_numbers, nb_EC);
  convert(adjacency_matrix,adjacency_list,nb_nodes);
 
  //print_adjacency_list();
}


/*Constructor for a graph with a different format : list of edges and
  list of vertex labels*/
/*labels_file contains one line per node and possibly many colors per
  line*/
/*graph_file contains a list of edges, nodes are numbered from 0 to
  n*/
Graph::Graph(char *graph_file, char *labels_file)
{
  /*get the number of nodes and read colours*/
  nb_nodes = read_nb_nodes(labels_file);

  /*since the first node is counted as 1 and not 0, the number of
    nodes should be n+1 so that the data structures are adapted*/
  //nb_nodes++;
  //cout<<"the number of nodes is "<<nb_nodes-1<<endl;

  /*allocate memory*/
  adjacency_matrix = initialize_matrix(adjacency_matrix, nb_nodes);
  adjacency_list = new list<int> *[nb_nodes];
  colors = new list<int> *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    {
      adjacency_list[i]=new list<int>;
      colors[i]=new list<int>;
    }
  parent_nodes=NULL;

  /*fill data*/
  // cout<<"before read graph"<<endl;
  read_graph2(graph_file,adjacency_matrix,nb_nodes);
  //cout<<"read graph complete"<<endl;
  read_labels2(labels_file, colors);
  //print_colors();
  convert(adjacency_matrix,adjacency_list,nb_nodes);
  //print_adjacency_list();
  //cout<<"nb of nodes is "<<nb_nodes<<endl;
}


/*copy constructor: copy only the adjacency list cos it's useful for the search*/
Graph::Graph(Graph *graph_to_copy)
{
  nb_nodes = graph_to_copy->get_nb_nodes();

  /*allocate memory*/
  adjacency_list = new list<int> *[nb_nodes];
  colors = new list<int> *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    {
      adjacency_list[i]=new list<int>;
      colors[i]=new list<int>;
    }

  /*copy variables*/
  for (int i=0;i<nb_nodes;i++)
    {
      *colors[i]=*graph_to_copy->get_colors()[i];
      *adjacency_list[i]=*graph_to_copy->get_adjacency_list()[i];
    }

  /*set remaining variables to nil*/
  adjacency_matrix=NULL;
  parent_nodes=NULL;
}

/*copy + shuffle constructor: copy only the adjacency list cos it's useful for the search*/
Graph::Graph(Graph *graph_to_copy, int *shuffle)
{
  nb_nodes = graph_to_copy->get_nb_nodes();

  /*allocate memory*/
  adjacency_list = new list<int> *[nb_nodes];
  colors = new list<int> *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    {
      adjacency_list[i]=new list<int>;
      colors[i]=new list<int>;
    }
  parent_nodes = new int[nb_nodes];

  //cerr<<"avant shuffle"<<endl;

  /*copy  + shuffle variables*/
  /*and keep track of how they got shuffled*/
  for (int i=0;i<nb_nodes;i++)
    {
      *colors[i]=*graph_to_copy->get_colors()[shuffle[i]]; //colors are shuffled
      *adjacency_list[i]=*graph_to_copy->get_adjacency_list()[i]; //topology is fixed
      parent_nodes[i]=shuffle[i];
    }


  /*set remaining variables to nil*/
  adjacency_matrix=NULL;
  //parent_nodes=NULL;
}


/**
 *Copy and Filter constructor. The graph is first copied and then filtered using only colours from the motif.
 * @param graph_to_copy graph to copy and filter
 * @param colors_to_filter which colours we should keep in the filtered graph
 * @param motif_size number of colours to keep
 */
Graph::Graph(Graph *graph_to_copy, int *colors_to_filter, int motif_size)
{

  list<int>::iterator it;

  int initial_nb_nodes = graph_to_copy->get_nb_nodes();
  parent_nodes=new int[initial_nb_nodes]; //gives indices of nodes you gotta keep since they have the right color
  int *correspondance=new int[initial_nb_nodes];//gives the correspondance between indices in the old graph and the new graph
  
  /*get size of the filtered graph*/
  nb_nodes=0;
  for (int i=0;i<initial_nb_nodes;i++)
    if(is_list_in_tab(graph_to_copy->get_colors()[i],colors_to_filter,motif_size))
      {      
	parent_nodes[nb_nodes]=i;
	correspondance[i]=nb_nodes;
	nb_nodes++;
      }
    else 
      correspondance[i]=-1;

  //cout<<"the size of the filetered graph: "<<nb_nodes<<endl;

  /*allocate memory (with the right size)*/
  adjacency_list = new list<int> *[nb_nodes];
  colors = new list<int> *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    {
      adjacency_list[i]=new list<int>;
      colors[i]=new list<int>;
    }



  /*copy and filter adjacency_list and colors*/
  /*instead of withdrawing nodes and edges, we simply transfer the
    nodes to keep to the new adjacency list checking that the edges go
    to existing nodes (ID: don't withdraw but keep)*/
  for (int i=0;i<nb_nodes;i++)
    {
      for (it = graph_to_copy->get_adjacency_list()[parent_nodes[i]]->begin();
	   it !=graph_to_copy->get_adjacency_list()[parent_nodes[i]]->end();
	   it++)
	{
	  if (correspondance[*it]!=-1){
	    adjacency_list[i]->push_back(correspondance[*it]);
	  }
	}
      //cout << endl;

      /*copy colors: only colors present in the motif are kept*/
      for (it = graph_to_copy->get_colors()[parent_nodes[i]]->begin(); it != graph_to_copy->get_colors()[parent_nodes[i]]->end(); it++)
	if (is_in_tab(*it,colors_to_filter,motif_size))
	  colors[i]->push_back(*it);

	//*colors[i]=*graph_to_copy->get_colors()[parent_nodes[i]];
    }
 
 //  cout<<"here are the colors of the filtered graph"<<endl;
//   print_tab_list(colors, nb_nodes);


  /*set remaining variables to nil*/
  adjacency_matrix=NULL;


  delete[]correspondance;
}


Graph::~Graph()
{

  if(adjacency_list!=NULL)
    {
      for (int i=0;i<nb_nodes;i++)
	{
	  delete adjacency_list[i];
	}
      delete[] adjacency_list;
    }

  if (adjacency_matrix!=NULL)
    {
      for (int i=0;i<nb_nodes;i++)
	delete[] adjacency_matrix[i];
      delete[] adjacency_matrix;
    }

  if (colors!=NULL){
    for (int i=0;i<nb_nodes;i++)
      delete (colors[i]);
    delete[] colors;
  }

  if (parent_nodes!=NULL)
    delete[] parent_nodes;
}


int Graph::get_nb_nodes()
{
  return nb_nodes;
}

list<int> **Graph::get_adjacency_list()
{
  return adjacency_list;
}

list<int> **Graph::get_colors()
{
  return colors;
}

void Graph::print_colors(std::ostream &output)
{
  //print_tab_list(colors,nb_nodes);
  print_tab_list(colors,nb_nodes,output);
}


int Graph::get_nb_colors()
{
  list<int> all_colors;
  for (int i=0;i<nb_nodes;i++)
    {
      for (list<int>::iterator it=colors[i]->begin();it!=colors[i]->end();it++)
	
	all_colors.push_back(*it);
    }
  all_colors.sort();
  all_colors.unique();
  return (int)all_colors.size();
}

void Graph::get_all_colors(int *tab)
{
  list<int> all_colors;
  for (int i=0;i<nb_nodes;i++)
    {
      for (list<int>::iterator it=colors[i]->begin();it!=colors[i]->end();it++)
	
	all_colors.push_back(*it);
    }
  all_colors.sort();
  all_colors.unique();

  int i=0;
  for (list<int>::iterator it=all_colors.begin();it!=all_colors.end();it++)
    {    
      tab[i]=*it;
      i++;	
    }


}



int *Graph::get_parent_nodes()
{
  return parent_nodes;
}

void Graph::print_edges(std::ostream & output)
{
  for (int i=0;i<nb_nodes;i++)
    for (int j=i;j<nb_nodes;j++)
      if (adjacency_matrix[i][j])
	output<<i<<" "<<j<<" "<<"1"<<endl;
}

/*print degrees to file*/
void Graph::print_degrees(std::ostream & output)
{
  for (int i=0;i<nb_nodes;i++)
    {
      int degree=0;
      for (int j=0;j<nb_nodes;j++)
	degree+=adjacency_matrix[i][j];
	  //if (adjacency_matrix[i][j])
      output<<degree<<endl;
    }
}

/*print the adjacency list representation of the graph*/
void Graph::print_adjacency_list()
{
  print_tab_list(adjacency_list,nb_nodes);
}

void Graph::print_colors()
{
  print_tab_list(colors, nb_nodes);
  cout << "fin list colors" << endl;

}

// /*FROM NOW ON, THESE FUNCTIONS PERFORM A FAST SEARCH OF ALL OCCURRENCES OF THE MOTIF IN THE GRAPH
// (what is different from the first search is that it gives also non-linear motifs and it performs the search on a more compact representation of the graph*/


/*new version of the enumeration algorithm : 02/05*/
/*faster, able to account for non-linear occurrences, and using a more compact representation of graph (adjacency list as a tab of lists)*/
int Graph::enumerate(list<int *> *solutions, int motif_size, int *motif_colors)
{

  /*structures needed for computation*/
  /*first tab that references the reactions I picked*/
  int *reactions_picked;
  reactions_picked=new int[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    reactions_picked[i]=-1;

  /*second tab giving for each reaction of tab 1, the direct neighbors (first row gives the index in tab 1 of the first neighbor and second row gives the index of last neighbor)*/
  int **neighbors;
  neighbors=new int *[2];
  for (int i=0;i<2;i++)
    neighbors[i]=new int[nb_nodes];
  for (int i=0;i<2;i++)
    for(int j=0;j<nb_nodes;j++)
      neighbors[i][j]=-1;

  /*third tab giving for each reaction, if it has been picked (first column) or eliminated (second column)*/
  bool **picked_eliminated;
  picked_eliminated=(bool**)malloc(nb_nodes*sizeof(bool*));
  for (int i=0;i<nb_nodes;i++)
    picked_eliminated[i]=(bool*)malloc(2*sizeof(bool));

  for (int i=0;i<nb_nodes;i++)
    for(int j=0;j<2;j++)
      picked_eliminated[i][j]=0;

  /*tab of size |motif| to store choices for a given topology*/
  int *choices;
  choices=new int[motif_size];   //all digits are used (except for the first one) : then the first digits are for distance 1, then distance 2... (according to the topology being tested)
  bool *last_choice;  // last_choice[i]=1 if all choices have been explored for distance i
  last_choice=new bool[motif_size];
  last_choice[0]=0; //useful cos tested when backtracking
  
  int end=0; //initialisation ? position of first digit equal to 0 in topology
  

  /**************ADDED 20/02/06***************/
  
  /*matrix giving all topologies to test when starting from a node. Every line of the matrix is a different topology*/
  /*Example: for a motif of size 4, the topology 0 1 2 1 means you pick 1 reaction at distance 1, 2 reactions at distance 2 and 1 reaction at distance 3 (the first cell is never used)*/
  /*The number of toplogies for a motif of size k is equal to the number of ways to decompose k as a sum of integers: 2^k -1*/
  int **topology_matrix;
  int nb_topologies= (int)(pow((double)2,(double)motif_size-2));
  topology_matrix = new int*[nb_topologies];
  for (int i=0;i<nb_topologies;i++)
    topology_matrix[i]=new int[motif_size+1];//the number of columns should be motif_size - 1 but I think that at some point we are testing one step too far.
  initialize_topology_matrix(topology_matrix,nb_topologies,motif_size+1);
  fill_topology_matrix(topology_matrix,motif_size);


/**************END OF ADDED 20/02/06***************/

  int pos=0; //postion of pointer in reactions_picked
  int *dist_pos;  //dist_pos[i] gives the position of the first reaction at distance i in reactions_picked (offset)
  dist_pos=new int[motif_size+1];
  
  /*for optimizing test colors : these tabs are never used at a global scope, nevertheless, they are declared here so that the memory allocation is done only once (even if the whole space won't be needed at each call)*/
  bool *matched_colors;
  matched_colors=new bool[motif_size];
  int *color_choice; //useful to fill color_candidates
  color_choice=new int[motif_size];
  int *color_sizes;  //useful to fill color_candidates
  color_sizes=new int[motif_size];

  bool test=0; //test color result, if test not ok, then move to the next choice

  int *solution;
  solution = new int[motif_size];

  /*useful to handle multiple matches*/
  list<int *> *color_candidates; //if all nodes are unicolor, there will be only one tab, otherwise there si a tab for all possible sets of colors related to a group of candidate nodes
  color_candidates = new list<int *>;
  int *nb_color_candidates;  // needed to optimize the space : nb_color_candidates[j] gives the nb of tabs to consider in color_candidates at level j
  nb_color_candidates = new int[motif_size];
  
    
  /*start computing*/
	
  /*special case : there's only one color in the motif*/
  if (motif_size==1)
    {
      //cout<<"je suis dans le cas motif_size=1"<<endl;
      for (int i=0;i<nb_nodes;i++)
	{
	  solution[0]=i;
	  int *temp_sol=new int[motif_size];
	  for (int i=0;i<motif_size;i++)
	    temp_sol[i]=solution[i];
	  solutions->push_back(temp_sol);
	}
      delete[](reactions_picked);
      for (int i=0;i<2;i++)
	delete[](neighbors[i]);
      delete[](neighbors);
      for (int i=0;i<nb_nodes;i++)
	free(picked_eliminated[i]);
      free(picked_eliminated);
      delete[](choices);
      delete[](last_choice);
      delete[](dist_pos);
      for (int i=0;i<nb_topologies;i++)
	delete[] (topology_matrix[i]);
      delete[] topology_matrix;
      delete[](matched_colors);
      delete[](color_choice);
      delete[](color_sizes);
      delete[](solution);
      delete[](nb_color_candidates);
      for (list<int *>::iterator it=color_candidates->begin();it!=color_candidates->end();it++)
	delete[](*it);
      delete(color_candidates);
      return 1;
    }



  for (int i=0;i<nb_nodes;i++)
    {
      /*start bfs from reaction i*/
      pos=0;
      
      /*reset reactions_picked and picked_eliminated*/
      for (int k=0;k<nb_nodes;k++)
	{
	  reactions_picked[k]=-1;
	  picked_eliminated[k][0]=0;
	  neighbors[0][k]=-1;
	  neighbors[1][k]=-1;
	}
      
      /*pick reaction i*/
      reactions_picked[pos]=i;
      pos++;
      dist_pos[0]=0;
      picked_eliminated[i][0]=1;
      solution[0]=i;
      
      /*initialize color_candidates*/
      initialize_color_candidates(color_candidates, colors[i],motif_size);
      nb_color_candidates[0]=colors[i]->size();
      	

      /*get neighbors at distance 1*/
      pick_neighbors(i,adjacency_list, picked_eliminated, neighbors, reactions_picked, &pos);
      dist_pos[1]=1;
      dist_pos[2]=pos;  //reactions at distance 2 will be added from position pos

      /*for every topology*/      
      for (int t=0;t<nb_topologies;t++)  //for every topology
	{
	  /*initialize choices*/
	  int j=0; //distance being considered 
	  int pos_choices=1;  //current position of cursor in tab choices (it is also the position of cursor in tab solution)
	  end=1;
	  while (topology_matrix[t][end]!=0)  //initialisation of end
	    end++;
	  
	  /*for every distance to the source node*/
	  for (j=1; j<end;j++)
	    {
	      first_choice(choices+pos_choices, topology_matrix[t][j]);
	      pos_choices+=topology_matrix[t][j];
	      last_choice[j]=0;
	    }
	  pos_choices=1;
	  j=1;
	  bool out=0;
	  while (!out)   //for all choices
	    {
	      /*generation of candidates + test*/
	      //choose nodes among neighbors dist 1 according to choices_j : generate candidates  
	      //nb neighbors dist j = dist_pos[j+1] - dist_pos[j]  (n)
	      //nb nodes we have to choose : topologies[j-1]  (m)
	      if (topology_matrix[t][j] > dist_pos[j+1] - dist_pos[j])  //we don't have enough candidates
		{
		  /*situation where we do not have enough candidates*/
		  test=0;
		  last_choice[j]=1;
		}
	      else  //we have enough candidates
		{
		  //cout <<"je vais choisir parmi mes candidats"<<endl;
// 		  cout<<"pos_choices = "<<pos_choices<<" et topologies[j] = "<<topologies[j]<<endl;
		  for (int k=pos_choices;k<topology_matrix[t][j]+pos_choices;k++)
		    {
		      /*adding candidates to the solution*/
		      solution[k]=reactions_picked[dist_pos[j]+choices[k]];
		    }
		  // cout<<"j'ai "<<topologies[j]<<" candidats"<<endl; 
		  

		  /*Since one node may have several colors, in order
		    to test a set of candidate nodes, you have to test
		    all combinations of colors these nodes can give
		    (multicolor nodes)*/
		  /*therefore, we build a list of tabs
		    (candidate_colors) summarizing all color_choices
		    possible for a set of nodes*/
		  fill_color_candidates(solution+pos_choices, topology_matrix[t][j],pos_choices, color_candidates, nb_color_candidates+j-1, colors, motif_size,color_choice, color_sizes);
		  
		  /*test colors*/
		  if (j==end-1)  //if one of the color_choice is ok, the test is ok and you don't have to test the others
		    test=final_test(color_candidates,pos_choices+topology_matrix[t][j], nb_color_candidates[j], motif_colors, motif_size, matched_colors);
		  
		  else //all color choices must be tested, if one of them is ok, the test is ok but you still have to test them all to get rid of the 'not ok' ones
		    //test=1;  //just to test the execution time without this pruning
		    test=partial_test(color_candidates, pos_choices+topology_matrix[t][j],nb_color_candidates+j,motif_colors, motif_size, matched_colors);

		  
		}
	      
	      
	      last_choice[j]=is_last_choice(choices+pos_choices,topology_matrix[t][j],dist_pos[j+1]-dist_pos[j]);
	     

	      /*treatment after test*/
	      				
	      if (test  && j<end-1)  //so far, the colors are ok, let's extend the solution to the next level (distance)
		{
		  //add candidates to partial_solution
		  
		  //extend based on choices_j;
		  for (int i=0;i<topology_matrix[t][j];i++)
		      pick_neighbors(*(solution+pos_choices+i),adjacency_list, picked_eliminated, neighbors, reactions_picked, &pos);
		  
		  pos_choices += topology_matrix[t][j];
		  j++;
		  dist_pos[j+1]=pos;
		}
	      else
		{
		  if (test && j==end-1)
		    {
		      /*We found a solution*/
		      int *temp_sol=new int[motif_size];
		      for (int i=0;i<motif_size;i++)
			temp_sol[i]=solution[i];
		      solutions->push_back(temp_sol);
		    }
		  
		  /*either the test was good but we reached the end or the test was bad, anyway, we need to move to next choice*/
		  //cout<<"je retire mes candidats de ma solution car le test tait mauvais ou bon et solution"<<endl;
		 
		  /*we backtrack as long as all choices have been explored for level j*/
		  while (last_choice[j])
		    {
		      /*initialize choices : you have to reinitialize choices while computing because when you come back to level j, you will need to start from the first choice*/
		      first_choice(choices+pos_choices, topology_matrix[t][j]);
		      last_choice[j]=0;

		      j--;
		      pos_choices-=topology_matrix[t][j];
		      if (j>0)
			{
			  unpick_neighbors(dist_pos[j+1],picked_eliminated, neighbors, reactions_picked, &pos);
			}
		    }

		  /*after backtracking there is still a choice not yet explored*/
		  if (j>=1)
		    {
		      next_choice(choices+pos_choices,topology_matrix[t][j],dist_pos[j+1]-dist_pos[j]) ;//returns 0 if the last choice has been tested
		    }

		  /*after backtracking, all choices have been explored, we move to the next topology*/
		  else 
		    out=1;
		  
		}
	    }
	  /*move to next topology*/
	}

      /*eliminate reaction i*/
      picked_eliminated[i][1]=1;
    }
  
  
  /*free memory*/
  delete[] (reactions_picked);
  for (int i=0;i<2;i++)
    delete[] (neighbors[i]);
  delete[] (neighbors);
  for (int i=0;i<nb_nodes;i++)
    free(picked_eliminated[i]);
  free(picked_eliminated);
  delete[] (choices);
  delete[] (last_choice);
  
  //delete(topologies);
  delete[] (dist_pos);
  
  for (int i=0;i<nb_topologies;i++)
    delete[] (topology_matrix[i]);
  delete[] topology_matrix;

  delete[] (matched_colors);
  delete[] (color_choice);
  delete[] (color_sizes);
  delete[] (solution);
  
  delete[] (nb_color_candidates);
  for (list<int *>::iterator it=color_candidates->begin();it!=color_candidates->end();it++)
    delete[] (*it);
  delete (color_candidates);

  // for (int i=0;i<nb_nodes;i++)
//     delete(colors[i]);
//   delete(colors);
  return 1;
}



/*same as before but we only report the number of occurrences which is given as return value*/
/*new version of the enumeration algorithm : 02/2005*/
/*faster, able to account for non-linear occurrences, and using a more compact representation of graph (adjacency list as a tab of lists)*/
int Graph::enumerate(int motif_size, int *motif_colors)
{
 
  int nb_solutions=0;
    
  /*structures needed for computation*/
  /*first tab that references the reactions I picked*/
  int *reactions_picked;
  reactions_picked=new int[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    reactions_picked[i]=-1;

  /*second tab giving for each reaction of tab 1, the direct neighbors (first row gives the index in tab 1 of the first neighbor and second row gives the index of last neighbor)*/
  int **neighbors;
  neighbors=new int *[2];
  for (int i=0;i<2;i++)
    neighbors[i]=new int[nb_nodes];
  for (int i=0;i<2;i++)
    for(int j=0;j<nb_nodes;j++)
      neighbors[i][j]=-1;

  /*third tab giving for each reaction, if it has been picked (first column) or eliminated (second column)*/
  bool **picked_eliminated;
  picked_eliminated=(bool**)malloc(nb_nodes*sizeof(bool*));
  for (int i=0;i<nb_nodes;i++)
    picked_eliminated[i]=(bool*)malloc(2*sizeof(bool));

  for (int i=0;i<nb_nodes;i++)
    for(int j=0;j<2;j++)
      picked_eliminated[i][j]=0;

  /*tab of size |motif| to store choices for a given topology*/
  int *choices;
  choices=new int[motif_size];   //all digits are used (except for the first one) : then the first digits are for distance 1, then distance 2... (according to the topology being tested)
  bool *last_choice;  // last_choice[i]=1 if all choices have been explored for distance i
  last_choice=new bool[motif_size];
  last_choice[0]=0; //useful cos tested when backtracking
  
  int end=0; //initialisation ? position of first digit equal to 0 in topology
  

  /**************ADDED 20/02/06***************/
  
  /*matrix giving all topologies to test when starting from a node. Every line of the matrix is a different topology*/
  /*Example: for a motif of size 4, the topology 0 1 2 1 means you pick 1 reaction at distance 1, 2 reactions at distance 2 and 1 reaction at distance 3 (the first cell is never used)*/
  /*The number of toplogies for a motif of size k is equal to the number of ways to decompose k as a sum of integers: 2^k -1*/
  int **topology_matrix;
  int nb_topologies= (int)(pow((double)2,(double)motif_size-2));
  topology_matrix = new int*[nb_topologies];
  for (int i=0;i<nb_topologies;i++)
    topology_matrix[i]=new int[motif_size+1];//the number of columns should be motif_size - 1 but I think that at some point we are testing one step too far.
  initialize_topology_matrix(topology_matrix,nb_topologies,motif_size+1);
  fill_topology_matrix(topology_matrix,motif_size);


  /**************END OF ADDED 20/02/06***************/


  int pos=0; //postion of pointer in reactions_picked
  int *dist_pos;  //dist_pos[i] gives the position of the first reaction at distance i in reactions_picked (offset)
  dist_pos=new int[motif_size+1];
  
  /*for optimizing test colors : these tabs are never used at a global scope, nevertheless, they are declared here so that the memory allocation is done only once (even if the whole space won't be needed at each call)*/
  bool *matched_colors;
  matched_colors=new bool[motif_size];
  int *color_choice; //useful to fill color_candidates
  color_choice=new int[motif_size];
  int *color_sizes;  //useful to fill color_candidates
  color_sizes=new int[motif_size];

  bool test=0; //test color result, if test not ok, then move to the next choice

  int *solution;
  solution = new int[motif_size];

  /*useful to handle multiple matches*/
  list<int *> *color_candidates; //if all nodes are unicolor, there will be only one tab, otherwise there si a tab for all possible sets of colors related to a group of candidate nodes
  color_candidates = new list<int *>;
  int *nb_color_candidates;  // needed to optimize the space : nb_color_candidates[j] gives the nb of tabs to consider in color_candidates at level j
  nb_color_candidates = new int[motif_size];
  
    
  /*special case : there's only one enzyme in the motif*/
  if (motif_size==1)
    {
      for (int i=0;i<nb_nodes;i++)
	{
	  /*version when you only want to count the number of occurrences*/
	  nb_solutions++;
	}
	 
      delete[](reactions_picked);
      for (int i=0;i<2;i++)
	delete[](neighbors[i]);
      delete[](neighbors);
      for (int i=0;i<nb_nodes;i++)
	free(picked_eliminated[i]);
      free(picked_eliminated);
      delete[](choices);
      delete[](last_choice);
      delete[](dist_pos);
      for (int i=0;i<nb_topologies;i++)
	delete[] (topology_matrix[i]);
      delete[] topology_matrix;
      delete[](matched_colors);
      delete[](color_choice);
      delete[](color_sizes);
      delete[](solution);
      delete[](nb_color_candidates);
      for (list<int *>::iterator it=color_candidates->begin();it!=color_candidates->end();it++)
	delete[](*it);
      delete(color_candidates);
      
      return nb_solutions;
    }


  for (int i=0;i<nb_nodes;i++)
    {
      pos=0;
      /*reset reactions_picked and picked_eliminated*/
      for (int k=0;k<nb_nodes;k++)
	{
	  reactions_picked[k]=-1;
	  picked_eliminated[k][0]=0;
	  neighbors[0][k]=-1;
	  neighbors[1][k]=-1;
	}
      
      /*pick reaction i*/
      reactions_picked[pos]=i;
      pos++;
      dist_pos[0]=0;
      picked_eliminated[i][0]=1;
      solution[0]=i;
      
      /*initialize color_candidates*/
      initialize_color_candidates(color_candidates, colors[i],motif_size);
      nb_color_candidates[0]=colors[i]->size();
      	

      /*get neighbors at distance 1*/
      pick_neighbors(i,adjacency_list, picked_eliminated, neighbors, reactions_picked, &pos);
      dist_pos[1]=1;
      dist_pos[2]=pos;  //reactions at distance 2 will be added from position pos
           
      /*for every topology*/      
      for (int t=0;t<nb_topologies;t++)  
	{
	  /*initialize choices*/
	  int j=0; //distance being considered 
	  int pos_choices=1;  //current position of cursor in tab choices (it is also the position of cursor in tab solution)
	  end=1;

	  /*initialisation of end*/
	  while (topology_matrix[t][end]!=0)  
	    end++;
	  for (j=1; j<end;j++)
	    {
	      first_choice(choices+pos_choices, topology_matrix[t][j]);
	      pos_choices+=topology_matrix[t][j];
	      last_choice[j]=0;
	    }
	  pos_choices=1;
	  j=1;
	  bool out=0;
	  while (!out)   //for all choices
	    {
	      /*generation of candidates + test*/
	      //choose nodes among neighbors dist 1 according to choices_j : generate candidates  
	      //nb nodes we have to choose : topology_matrix[t][j-1]  (m)
	      if (topology_matrix[t][j] > dist_pos[j+1] - dist_pos[j])  //we don't have enough candidates
		{
		  test=0;
		  last_choice[j]=1;
		}
	      else  //we have enough candidates
		{
		  for (int k=pos_choices;k<topology_matrix[t][j]+pos_choices;k++)
		    {
		      solution[k]=reactions_picked[dist_pos[j]+choices[k]];
		    }
		  		  
		  /*to test a set of candidate nodes, you have to test all combinations of colors these nodes can give (multicolor nodes)*/
		  /*therefore, we build a list of tabs (candidate_colors) summarizing all color_choices possible for a set of nodes*/
		  
		  fill_color_candidates(solution+pos_choices, topology_matrix[t][j],pos_choices, color_candidates, nb_color_candidates+j-1, colors, motif_size,color_choice, color_sizes);
		  /*test colors*/
		  if (j==end-1)  //if one of the color_choice is ok, the test is ok and you don't have to test the others
		    test=final_test(color_candidates,pos_choices+topology_matrix[t][j], nb_color_candidates[j], motif_colors, motif_size, matched_colors);
		  else //all color choices must be tested, if one of them is ok, the test is ok but you still have to test them all to get rid of the 'not ok' ones
		    //test=1;  //just to test the execution time without this pruning
		    test=partial_test(color_candidates, pos_choices+topology_matrix[t][j],nb_color_candidates+j,motif_colors, motif_size, matched_colors);

		}
	      
	      
	      last_choice[j]=is_last_choice(choices+pos_choices,topology_matrix[t][j],dist_pos[j+1]-dist_pos[j]);
	     

	      /*treatment after test*/
	      				
	      if (test  && j<end-1)  //so far, the colors are ok, let's extend the solution to the next level (distance)
		{
		  //add candidates to partial_solution
		  
		  //extend based on choices_j;
		  for (int i=0;i<topology_matrix[t][j];i++)
		      pick_neighbors(*(solution+pos_choices+i),adjacency_list, picked_eliminated, neighbors, reactions_picked, &pos);
		  
		  pos_choices += topology_matrix[t][j];
		  j++;
		  dist_pos[j+1]=pos;
		}
	      else
		{
		  if (test && j==end-1)  //we found a solution
		    {
		      /*version when you only want to count the solutions*/
		      nb_solutions++;

                      /*new version for Graph*/
		      // cout<<"j'ajoute une solution"<<endl;
// 		      int *temp_sol=new int[motif_size];
// 		      for (int i=0;i<motif_size;i++)
// 			temp_sol[i]=solution[i];
// 		      solutions->push_back(temp_sol);

		      /*old version for Reaction_Graph*/
		      //solutions->add(new Solution(solution, motif_size, tab_reactions));
		      
		    }
		  
		  /*either the test was good but we reached the end or the test was bad, anyway, we need to move to next choice*/
		  
		  /*we backtrack as long as all choices have been explored for level j*/
		  while (last_choice[j])
		    {
		      /*initialize choices : you have to reinitialize choices while computing because when you come back to level j, you will need to start from the first choice*/
		      first_choice(choices+pos_choices, topology_matrix[t][j]);
		      last_choice[j]=0;

		      j--;
		      pos_choices-=topology_matrix[t][j];
		      if (j>0)
			{
			  unpick_neighbors(dist_pos[j+1],picked_eliminated, neighbors, reactions_picked, &pos);
			}
		    }
		  /*after backtracking there is still a choice not yet explored*/
		  if (j>=1)
		    {
		      next_choice(choices+pos_choices,topology_matrix[t][j],dist_pos[j+1]-dist_pos[j]) ;//returns 0 if the last choice has been tested
		    }
		  /*after backtracking, all choices have been explored, we move to the next topology*/
		  else 
		    out=1;
		  
		}
	    }
	  /*move to next topology*/
	}
      //eliminate reaction i
      picked_eliminated[i][1]=1;
    }
 

  /*free memory*/
  delete[] (reactions_picked);
  for (int i=0;i<2;i++)
    delete[] (neighbors[i]);
  delete[] (neighbors);
  for (int i=0;i<nb_nodes;i++)
    free(picked_eliminated[i]);
  free(picked_eliminated);
  delete[] (choices);
  delete[] (last_choice);
  
  //delete(topologies);
  delete[] (dist_pos);
  
  for (int i=0;i<nb_topologies;i++)
    delete[] (topology_matrix[i]);
  delete[] topology_matrix;

  delete[] (matched_colors);
  delete[] (color_choice);
  delete[] (color_sizes);
  delete[] (solution);
  
  delete[] (nb_color_candidates);
  for (list<int *>::iterator it=color_candidates->begin();it!=color_candidates->end();it++)
    delete[] (*it);
  delete (color_candidates);

  return nb_solutions;
}




/*ced*/
bool** Graph::get_adjacency_matrix(){
  return adjacency_matrix;
}

// void Graph::printMotif(){
//   print_tab_list(colors, nb_nodes);
//   cout << "fin list colors" << endl;

// }

