#include "Metabolic_Network.h"
#include <iostream>
#include <fstream>

Metabolic_Network::Metabolic_Network(Data *myData)
{
  connected_components=new list<Reaction_Graph *>;
}

Metabolic_Network::Metabolic_Network(Data *myData, Metabolic_Motif *myMotif)
{
}

Metabolic_Network::~Metabolic_Network()   
{
  list<Reaction_Graph *>::iterator graph_iter = connected_components->begin();
  while (graph_iter != connected_components->end())
    {
      delete(*graph_iter);
      graph_iter++;
    }
  delete(connected_components);
}

int erase(Reaction_Graph *graph_to_erase, list<Reaction_Graph *> *components)
{
  list<Reaction_Graph *>::iterator graph_iter = components->begin();
  while (graph_iter != components->end())
    {
      if ((*graph_iter)==graph_to_erase)   
	{
	  components->erase(graph_iter); //removes the reaction_graph * from the list
	  return 1;
	}
      graph_iter++;
    }
  cout<<"we are trying to erase a graph that doesn't exist"<<endl;
  return 0;  //0 means error !
}


void Metabolic_Network::enumerate_solutions(Metabolic_Motif *motif, Solution_Group *solutions)
{
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  //for all connected_components
  while (rg_iter != connected_components->end())
    {
      //(*rg_iter)->enumerate_solutions(solutions, motif);  //old version
      (*rg_iter)->enumerate(solutions, motif); //new version
      rg_iter++;
    }
}


//withdraws the connected_components that don't have all the enzymes from the motif (or that verify the quorum in the general case)
void Metabolic_Network::filter_connected_components(Metabolic_Motif *motif)
{
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      if (! (*rg_iter)->contains_all_enzymes(motif) )
	{
	  //note that the erase method increments the iterator automatically
	  rg_iter=connected_components->erase(rg_iter);
	}
      else
	rg_iter++;
    }
}


//this function could be avoided if we did this first fusion with the other fusions, this would mean withdrawing some reactions afterwards for the no-gap case.
//note that withdrawing needless reactions is not a bad idea (whatever the level : gap 0,1,2...) but are they needless ?
void Metabolic_Network::first_fusion(list<Reaction *> *reaction_set)
{
  list<Compound *> ::iterator comp_iter;
  list<Reaction *> ::iterator reac_iter;
  list<Reaction *> ::iterator reac_iter2;
  
  int nb_edges=0;
  //cout <<"first fusion , i=0"<<endl;
  //for all reactions of the reaction set (at distance 0)
  reac_iter = reaction_set->begin();
  while (reac_iter != reaction_set->end())
    {
      //cout <<"boucle reaction";
      //(*reac_iter)->print();
      //cout <<endl;
      //for all compounds
      comp_iter = ((*reac_iter)->get_compounds())->begin();
      //cout <<"boucle compounds"<<endl;
      while (comp_iter != ((*reac_iter)->get_compounds())->end())
	{
	  //for all reactions
	  reac_iter2 = ((*comp_iter)->get_reactions())->begin();
	  while (reac_iter2 != ((*comp_iter)->get_reactions())->end())
	    {
	      if (*reac_iter != *reac_iter2)
		{
		  //treatment
		  //cout <<"boucle reaction2";
 		  //(*reac_iter2)->print();
 		  //cout <<endl;
		  //if edge has not been visited yet
		  if (! ((*comp_iter)->is_visited(*reac_iter2,*reac_iter)) )
		    {
		      nb_edges++;
		      //cout <<"l'arte n'a pas t visite"<<endl;
		      //(*comp_iter)->set_visited(*reac_iter2,*reac_iter);
		      if ((*reac_iter2)->get_distance()==-1)
			{
			  //cout <<"la raction n'a pas t visite"<<endl;
			}
		      else 
			{
			  //cout <<"la raction a t visite"<<endl;
			  if ((*reac_iter2)->connected_component != (*reac_iter)->connected_component)
			    {
			      if ((*reac_iter)->get_distance() + (*reac_iter2)->get_distance() == 0)
				{
				  (*comp_iter)->set_visited(*reac_iter2,*reac_iter);  //c'est ici qu'on devrait tiqueter l'arte avec le compos
				  //fusion of connected components
				  //cerr <<"avant fusion"<<endl;
				  Reaction_Graph *old_connected_component = (*reac_iter2)->connected_component;
				  ((*reac_iter)->connected_component)->fusion(*reac_iter2,*reac_iter);  //fusion with copy
				  //cerr <<"aprs fusion"<<endl;
				  //deletion of the obsolete connected component from the connected_components list
				  //cout<<"je suis dans first_fusion"<<endl;
				  delete(old_connected_component);
				  erase(old_connected_component, connected_components);
				}
			    }
			  else
			    {
			      //adds edges to the concerned connected components
			      ((*reac_iter)->connected_component)->add_edges(*reac_iter2,*reac_iter);   //in case there is a double link, it is managed in the add_edges method
			    }
			}
		    }
		}
	      //cout <<"je passe  la reaction suivante, boucle 2"<<endl;
	      reac_iter2++;
	    }
	  //cout <<"je passe au compos suivant"<<endl;
	  comp_iter++;
	}
      //cout <<"je passe  la reaction suivante, boucle 1"<<endl;
      //print_connected_components();
      reac_iter++;
    }
  //cout<<"le nombre d'artes de mon graphe est "<<nb_edges<<endl;
}


void treatment(list<Reaction *> ::iterator reac_iter, list<Reaction *> ::iterator reac_iter2, list<Compound *> ::iterator comp_iter, list<Reaction_Graph *> *connected_components, int i, list<Reaction *> *temp_reaction_set)
{
  //cout <<"boucle reaction2";
  //(*reac_iter2)->print();
  //cout <<endl;
  //if edge has not been visited yet
  if (! ((*comp_iter)->is_visited(*reac_iter2,*reac_iter)) )
    {
      (*comp_iter)->set_visited(*reac_iter2,*reac_iter);
      //cout <<"l'arte n'a pas t visite"<<endl;
      if ((*reac_iter2)->get_distance()==-1)
	{
	  //cout <<"la raction n'a pas t visite"<<endl;
	  //adds a reaction to the new reaction set (distance i+1)
	  temp_reaction_set->push_back(*reac_iter2);  
	  (*reac_iter2)->set_distance(i+1);
	  //adds node and edges to the concerned connected component
	  ((*reac_iter)->connected_component)->add(*reac_iter2,*reac_iter);  
	  //updating the connected component the reaction belongs to
	  (*reac_iter2)->connected_component = (*reac_iter)->connected_component;  
	}
      else
	{
	  //cout <<"la raction a t visite"<<endl;
	  if ((*reac_iter)->connected_component != (*reac_iter2)->connected_component)
	    {
	      if ((*reac_iter)->get_distance() + (*reac_iter2)->get_distance() <= i+1)  //we could speed things up by replacing i+1 by delta+1 but we'll keep it this way for clarity's sake
		{
		  //fusion of connected components
		  //cout <<"avant fusion"<<endl;
		  Reaction_Graph *old_connected_component = (*reac_iter2)->connected_component;
		  ((*reac_iter)->connected_component)->fusion(*reac_iter2,*reac_iter);
		  //cout <<"aprs fusion"<<endl;
		  //deletion of the obsolete connected component from the connected_components list
		  delete(old_connected_component);
		  erase(old_connected_component, connected_components);
		}
	      else
		{
		  //we'll do the fusion later so we'll need to revisit this edge
		  (*comp_iter)->set_unvisited(*reac_iter2,*reac_iter);
		}
	    }
	  else 
	    {
	      //adds edges to the concerned connected components
	      ((*reac_iter)->connected_component)->add_edges(*reac_iter2,*reac_iter);  
	      //there is a special case when the reactions are already linked by one compound and a double link is being explored, we can fix that in the add_edges method, but it's only temporary
	    }
	}
    }
  else 
    {
      //cout<<"oui l'arte a t visite"<<endl;
    }
}

//this is not very speedy, it adds a list traversal
bool in_list(Reaction * reac, list<Reaction *> *reac_list)
{
  list<Reaction *>::iterator iter = reac_list->begin();
    while (iter != reac_list->end())
      {
	if (reac == *iter)
	  return 1;
	iter++;
      }
  return 0;
}

int Metabolic_Network::build(Data *data, Metabolic_Motif *motif)
{
  list<Reaction *> *reaction_set = new list<Reaction *>;  // auxiliary structure referencing relevant reactions (at distance i from a relevant reaction) during the computation of the reaction graph 
  list<Reaction *> *temp_reaction_set = new list<Reaction *>; 
  int delta = Min(motif -> get_local_gap(), motif->get_global_gap());  //minimum of global gap and local gap
  list<Compound *> ::iterator comp_iter;
  list<Enzyme *> ::iterator enz_iter;
  list<Reaction *> ::iterator reac_iter;
  list<Reaction *> ::iterator reac_iter2;
  Reaction_Graph *temp_graph = NULL;  //maybe we could declare this one elsewhere

  //creation of the set of reactions catalysed by enzymes from the motif (distance 0)
  //and creation of a reaction graph for each of these reactions

  enz_iter = data->enzymes->begin();
  while (enz_iter!=data->enzymes->end())
    {
      // cout <<"je teste l'enzyme ";
//       (*enz_iter)->print();
//       cout <<endl;
      if (motif->contains_similar_enzyme(*enz_iter))  //the enzyme matches at least one enzyme from the motif
	{
	  //cout <<"je suis dans le cas d'une enzyme similaire"<<endl;
	  reac_iter = ((*enz_iter)->get_catalysed_reactions())->begin();
	  while (reac_iter != ((*enz_iter)->get_catalysed_reactions())->end())
	    {
	      /*even if the reaction is already in the list, we should add the color*/
	      //(*reac_iter)->add_colours(motif->enzyme_ranks(*enz_iter), motif->size());
	      list<int> *reaction_colours = new list<int>;
	      motif->get_colours(*enz_iter,reaction_colours);
	      (*reac_iter)->add_colours(reaction_colours);
	      delete reaction_colours;
	      //(*reac_iter)->add_colour(motif->get_colour(*enz_iter));  //here we suppose that an enzyme matches only one enzyme from the motif

	      if (!in_list(*reac_iter, reaction_set))   //adds a list traversal so that the same reaction is not created twice
		{
		  // (*reac_iter)->print();
// 		  cout<<endl;
		  reaction_set->push_back(*reac_iter);
		  temp_graph=new Reaction_Graph(*reac_iter);
		  connected_components->push_back(temp_graph);  //creation of a reaction graph for each reaction
		  (*reac_iter)->set_distance(0);
		  (*reac_iter)->set_first_colour(motif->enzyme_first_rank(*enz_iter));  //this is redundant with the add_colours function (should be tested)
		  //(*reac_iter)->print();
		  (*reac_iter)->connected_component = temp_graph;  //each reaction knows which connected component it belongs to
		  temp_graph=NULL;
		}
	      reac_iter++;
	    }
	}
      enz_iter++;
    }
  //cout<<"avant first_fusion"<<endl;
  //cout <<"j'ai mon paquet de ractions de distance 0"<<endl;
  //print_connected_components();
  //we need to do a first fusion between those reactions in case we want to work with no gap
  first_fusion(reaction_set);
  //cout<<"aprs first_fusion"<<endl;

  //extension of the set of reactions
  for (int i=0;i<delta;i++)
    {
      //cout <<"boucle for, i="<<i<<endl;
      //for all reactions of the reaction set (at distance i)
      reac_iter = reaction_set->begin();
      while (reac_iter!=reaction_set->end())
	{
	  //cout <<"boucle reaction";
	  //(*reac_iter)->print();
	  //cout <<endl;
	  //for all compounds
	  comp_iter = ((*reac_iter)->get_compounds())->begin();
	  //cout <<"boucle compounds"<<endl;
	  while (comp_iter != ((*reac_iter)->get_compounds())->end())
	    {
	      //(*comp_iter)->print_visited_edges();
	      //for all reactions
	      reac_iter2 = ((*comp_iter)->get_reactions())->begin();
	      while (reac_iter2 != ((*comp_iter)->get_reactions())->end())
		{
		  if (*reac_iter != *reac_iter2)
		    treatment(reac_iter, reac_iter2, comp_iter, connected_components, i, temp_reaction_set);
		  //print_connected_components();
		  reac_iter2++;
		}
	      comp_iter++;
	    }
	  //print_connected_components();
	  //cout <<"je passe  la reaction suivante"<<endl;
	  //print_connected_components();
	  reac_iter++;
	}

      cout <<"on transfre la reaction set"<<endl;
      //transfer temp_reaction_set to reaction_set
      reaction_set->clear();
      reac_iter = temp_reaction_set->begin();
      while (reac_iter != temp_reaction_set->end())
	{
	  reaction_set->push_back(*reac_iter);
	  //(*reac_iter)->print();
	  //cout<<"--distance"<<(*reac_iter)->get_distance();
	  reac_iter++;
	}
      temp_reaction_set->clear();
      cout<<endl;
    }

  /*free memory*/
  delete(reaction_set);
  delete(temp_reaction_set);
  //cout <<"le rsultat final est"<<endl;
  //print_connected_components();
  return 1;
}


void Metabolic_Network::print_identity_card()
{
  cout <<"mon rseau complet a "<<connected_components->size()<<" composantes connectes"<<endl;
  cout <<"taille des composantes connectes :"<<endl;
  int nb_edges=0;
  int nb_nodes=0;
  for (list<Reaction_Graph *>::iterator it=connected_components->begin();it!=connected_components->end();it++)
    {
      cout<<(*it)->size()<<" ";
      //cout<<endl;
      //cout<<"nb_nodes ="<<(*it)->get_nb_nodes()<<endl;
      //cout<<"graph size ="<<(*it)->size()<<endl;
      nb_edges+=(*it)->get_nb_edges();
      nb_nodes+=(*it)->get_nb_nodes();
    }
  cout<<endl;
  cout<<"il a "<<nb_edges<<" aretes"<<endl;
  cout<<"et "<<nb_nodes<<" noeuds"<<endl;
  //cout<<"il a "<<sum<<"reactions"<<endl;
}

void Metabolic_Network::print_identity_card(std::ostream& output)
{
  // cout <<"mon rseau complet a "<<connected_components->size()<<" composantes connectes"<<endl;
//   cout <<"taille des composantes connectes :"<<endl;
  int nb_edges=0;
  int nb_nodes=0;
  list<int> sizes;
  for (list<Reaction_Graph *>::iterator it=connected_components->begin();it!=connected_components->end();it++)
    {
      //cout<<(*it)->size()<<" ";
      sizes.push_back((*it)->size());
      //cout<<endl;
      //cout<<"nb_nodes ="<<(*it)->get_nb_nodes()<<endl;
      //cout<<"graph size ="<<(*it)->size()<<endl;
      nb_edges+=(*it)->get_nb_edges();
      nb_nodes+=(*it)->get_nb_nodes();
    }
  sizes.sort();
  sizes.reverse();

  output<<"number of nodes "<<nb_nodes<<endl;
  output <<"number of edges "<<nb_edges<<endl;
  output <<"number of connected components "<<connected_components->size()<<endl;
  output <<"sizes of connected components ";
  for (list<int>::iterator it=sizes.begin(); it!=sizes.end(); it++)
    output<<*it<<" ";
  output<<endl;

}



void Metabolic_Network::print_identity_card_enzyme_graph()
{
  //cout <<"mon rseau complet a "<<connected_components->size()<<" composantes connectes"<<endl;
  //cout <<"taille des composantes connectes :"<<endl;
  //int sum=0;
  int nb_edges=0;
  int nb_nodes=0;
  for (list<Reaction_Graph *>::iterator it=connected_components->begin();it!=connected_components->end();it++)
    {
      //cout<<(*it)->size()<<" ";
      //cout<<endl;
      //cout<<"nb_nodes ="<<(*it)->get_nb_nodes()<<endl;
      //cout<<"graph size ="<<(*it)->size()<<endl;
      nb_edges+=(*it)->get_nb_edges_enzyme_graph();
      nb_nodes+=(*it)->get_nb_nodes_enzyme_graph();
    }
  //cout<<endl;
  cout<<"il a "<<nb_edges<<" aretes"<<endl;
  cout<<"et "<<nb_nodes<<" noeuds"<<endl;
  //cout<<"il a "<<sum<<"reactions"<<endl;
}


void Metabolic_Network::print_labels_enzyme_graph(int threshold)
{
  std::ofstream output;
  //output.open("labels_enzyme_graph", ios_base::app);
  output.open("labels_enzyme_graph");

  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      (*rg_iter)->print_labels_enzyme_graph(output,threshold);
      //output<<endl;
      rg_iter++;
    }

  output.close();
}


/*builds the whole metabolic_network*/
void Metabolic_Network::build(Data *data)
{
  /*declaration of iterators*/
  list<Compound *> ::iterator comp_iter;
  list<Enzyme *> ::iterator enz_iter;
  list<Reaction *> ::iterator reac_iter;
  list<Reaction *> ::iterator reac_iter2;
  Reaction_Graph *temp_graph = NULL;  //maybe we could declare this one elsewhere

  /*creation of the connected_components*/
  reac_iter = data->reactions->begin();
  while (reac_iter != data->reactions->end())
	{
	  temp_graph=new Reaction_Graph(*reac_iter);
	  connected_components->push_back(temp_graph);  //creation of a reaction graph for each reaction
	  (*reac_iter)->connected_component = temp_graph;  //each reaction knows which connected component it belongs to
	  (*reac_iter)->set_distance(0);
	  temp_graph=NULL;
	  reac_iter++;
	}
  
  /*fusion of the connected components*/
  first_fusion(data->reactions);
  //print_connected_components();
}


void Metabolic_Network::convert_graph()
{
  for (list<Reaction_Graph *>::iterator it=connected_components->begin();it!=connected_components->end();it++)
    (*it)->convert_graph();
}

void Metabolic_Network::convert_graph_to_matrix()
{
  for (list<Reaction_Graph *>::iterator it=connected_components->begin();it!=connected_components->end();it++)
    (*it)->convert_graph_to_matrix();
}


void Metabolic_Network::print_components_size(char * whole_stat)
{
  FILE *output;
  char *output_name=whole_stat;
  output=fopen(output_name, "w");

  /*size of connected components of the whole graph*/
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      fprintf(output, "%d\n", (*rg_iter)->size());
      rg_iter++;
    }
  fclose(output);
}

void Metabolic_Network::print_reaction_arity(char * whole_stat)
{
  FILE *output;
  char *output_name=whole_stat;
  output=fopen(output_name, "w");

  /*arity of reactions in the reaction graph*/
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      (*rg_iter)->print_reaction_arity(output);
      rg_iter++;
    }
  fclose(output);
}

/*graph layout*/
void Metabolic_Network::draw_connected_components()
{
  std::ofstream output;
  output.open("bidon.res", ios_base::app);

  output <<"graph G {"<<endl;
  output << "node[shape=box, style=filled];"<<endl;

  // /*declaration of colours*/
//   for (int i=0; i<4; i++)
//     {
//       output << "{node[color=skyblue]";
      
//       output <<"}"<<endl;
//     }


  /*declaration of links*/
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      //(*rg_iter)->draw(output);
      rg_iter++;
    }
  output <<"}"<<endl;
  output.close();

  /*compile with neato*/
  system("neato bidon.res -Tps -o draw.ps -Goverlap=false");
  system("rm bidon.res");

  /*draw with gv*/
  system("gv draw.ps &");
}



/*basic print function*/

void Metabolic_Network::print_adjacency_lists()
{
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      (*rg_iter)->print_adjacency_list();
      cout<<endl;
      rg_iter++;
    }
}

/*print graph as matrix in a file*/
void Metabolic_Network::print_adjacency_matrices()
{
  /*declaration of output file stream, should be done upstairs ?*/
  std::ofstream output;
  output.open("adjacency_matrices", ios_base::app);

  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      (*rg_iter)->print_adjacency_matrix(output);
      output<<endl;
      rg_iter++;
    }

  output.close();
}

void Metabolic_Network::print_connected_components()
{
  int nb=0;
  int sum=0;
  float mean=0.0;
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      (*rg_iter)->print();
      sum+=(*rg_iter)->size();
      cout<<endl;
      //cout <<"next component\n"<<endl;
      rg_iter++;
      nb++;
    }
  mean=(float)sum/(float)nb;
  cout<<"j'ai "<<nb<<" composantes connectes"<<endl;
  cout<<"la taille moyenne de mes composantes connectes est "<<mean<<endl;
  cout<<endl;
}

/*print with several levels of detail*/
void Metabolic_Network::print_connected_components(int verbose)
{
  int nb=0;
  int sum=0;
  float mean=0.0;
  list<Reaction_Graph *> ::iterator rg_iter = connected_components->begin();
  while (rg_iter != connected_components->end())
    {
      (*rg_iter)->print(verbose);
      sum+=(*rg_iter)->size();
      cout<<endl;
      //cout <<"next component\n"<<endl;
      rg_iter++;
      nb++;
    }
  mean=(float)sum/(float)nb;
  //cout<<"j'ai "<<nb<<" composantes connectedes"<<endl;
  //cout<<"la taille moyenne de mes composantes connexes est "<<mean<<endl;
  cout<<endl;
}
