/* Occurrence_Group.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Occurrence_Group.h"
#include <iostream>


Occurrence_Group::Occurrence_Group()
{
  occurrences = new list<Occurrence *>;
}


Occurrence_Group::Occurrence_Group(Occurrence *occurrence)
{
  occurrences = new list<Occurrence *>;
  occurrences->push_back(occurrence);
}

/**
 *Destructor of Occurrence_Group : Note that when you delete the group,
 *you don't delete the occurrences that belong to this group. This is
 *useful when you deal with groups of occurrences that contain occurrences
 *shared among several groups.
 */
Occurrence_Group::~Occurrence_Group()
{
  // for (list<Occurrence *>::iterator it=occurrences->begin();it!=occurrences->end();it++)
//     delete(*it);
  delete(occurrences);
}

/**
 *Delete the occurrences contained in the group. This method is not
 *included in the destructor or Occurrence_Group since in some cases, we
 *are dealing with occurrences that are shared among several groups
 *without being copied.
 */
void Occurrence_Group::delete_occurrences()
{
  for (list<Occurrence *>::iterator it=occurrences->begin();it!=occurrences->end();it++)
    delete(*it);
  occurrences->erase(occurrences->begin(), occurrences->end());
}


// void Occurrence_Group::clear_occurrences()
// {
//   occurrences->clear();
// }

list<Occurrence *> * Occurrence_Group::get_occurrences()
{
  return occurrences;
}

void Occurrence_Group::add(Occurrence *occurrence)
{
  occurrences->push_back(occurrence);
}

// int Occurrence_Group::size()
// {
//   int out = occurrences->size();
//   return out;
//   // return (int)occurrences->size();
// }

/**
 * Recovers the topology of the occurrences in the bipartite_graph.
 *
 * @param whole_graph bipartite graph corresponding to the whole network
 *
 */
void Occurrence_Group::set_adj_matrix(Graph *whole_graph)
{
  for (list<Occurrence *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
    (*it)->set_adj_matrix(whole_graph);
}


/**
 *  Computes the canonical labelling of the bipartite graph of the
 * occurrences. This operation is done by nauty (No AUTomorphism, Yes
 * ?). The result is stored as two hexadecimal numbers.
 *
 */
void Occurrence_Group::compute_canonical_labelling(bool order)
{
  for (list<Occurrence *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
    (*it)->compute_canonical_labelling(order);
}





// /*useful for stats*/
// /*useful to calculate the mean arity*/
// int Occurrence_Group::nb_max_sol(Metabolic_Motif *motif)
// {
//   int out=0;
//   list<Occurrence *>::iterator sol_iter = occurrences->begin();
//   while (sol_iter != occurrences->end())
//     {
//       //cout<<"occurrence_group cpd arity :"<<out<<endl;
//       //out+=((*sol_iter)->size() == motif->size());
//       out+=((*sol_iter)->get_size() > 1);
//       sol_iter++;
//     }
//   return out;
// }

//returns true if the occurrences concern the same set of reactions (i.e. permutation of each other)
// bool is_permutation(Occurrence *sol1, Occurrence *sol2)
// {
//   /*they have to have the same number of reactions in order to have the same set*/
//   //if (iter1->size() != iter2->size())
//   if (sol1->get_size() != sol2->get_size())
//     return 0;

//   bool out=0;
//   list<Reaction *>::iterator rea_iter1 = sol1->get_reactions()->begin();
//   list<Reaction *>::iterator rea_iter2 = sol2->get_reactions()->begin();
//   while (rea_iter1 != sol1->get_reactions()->end())
//     {
//       out=0;
//       rea_iter2 = sol2->get_reactions()->begin();
//       while (rea_iter2 != sol2->get_reactions()->end() && !out)
// 	{
// 	  if ((*rea_iter2) == (*rea_iter1))
// 	  out=1;
// 	  rea_iter2++;
// 	}
//       if (!out)
// 	{return 0;}
//       rea_iter1++;
//     }
//   return 1;
// }

// bool common_reaction(Occurrence *sol1, Occurrence *sol2)
// {
//   list<Reaction *>::iterator rea_iter1 = sol1->get_reactions()->begin();
//   list<Reaction *>::iterator rea_iter2 = sol2->get_reactions()->begin();
//   while (rea_iter1 != sol1->get_reactions()->end())
//     {
//       rea_iter2 = sol2->get_reactions()->begin();
//       while (rea_iter2 != sol2->get_reactions()->end())
// 	{
// 	  /*as soon as we found a common reaction, we return true*/
// 	  if ((*rea_iter2) == (*rea_iter1))
// 	    return true;
// 	  rea_iter2++;
// 	}
//       rea_iter1++;
//     }
//   return false;
// }

/*withdraws the occurrences that are permutations of each other, this could be done while enumerating occurrences*/
// void Occurrence_Group::withdraw_redundance()
// {
//   list< Occurrence *> *unique_occurrences = new list<Occurrence *>;
//   bool ok=0;
//   list< Occurrence *> ::iterator iter1 = occurrences->begin();
//   list< Occurrence *> ::iterator iter2 = unique_occurrences->begin();
//   while (iter1 != occurrences->end())
//     {
//       //cout <<"je suis en dbut de while1"<<endl;
//       ok=0;
//       iter2 = unique_occurrences->begin();
//       while (iter2 != unique_occurrences->end() && !ok)
// 	{
// 	  //cout <<"je suis en dbut de while2"<<endl;
// 	  if (is_permutation(*iter1,*iter2))
// 	    {
// 	      ok=1;
// 	      //cout<<"je sius dans le cas d'une permutation"<<endl;
// 	    }
// 	  //cout<<"je vais itrer iter2"<<endl;
// 	  iter2++;
// 	}
//       if (!ok)
// 	{
// 	  //cout <<"je rajoute une nouvelle occurrence dans unique"<<endl;
// 	  unique_occurrences->push_back(*iter1);
// 	}
//       else 
// 	{
// 	  //this occurrence should be deleted
// 	  delete(*iter1);  //seems to work
// 	}
//       //cout <<"je vais itrer iter1"<<endl;
//       iter1++;
//       //cout<<"j'ai itr iter1"<<endl;
//       }

//     /*transfer of unique_occurrences to occurrences : this could be simplified easily*/
//     occurrences->clear();   //hope no one wants to keep that, it would be nice to erase the occurrences that are counted several times
//     iter2 = unique_occurrences->begin();
//     while (iter2 != unique_occurrences->end())
//       {      
// 	occurrences->push_back(*iter2);
// 	iter2++;
//       }
//     delete(unique_occurrences);
// }

// /*performs the fusion of two groups by adding one to the other*/
// void Occurrence_Group::add_group(Occurrence_Group *group_to_add)
// {
//   list<Occurrence *>::iterator sol_iter = group_to_add->get_occurrences()->begin();
//   while (sol_iter != group_to_add->get_occurrences()->end())
//     {
//       occurrences->push_back(*sol_iter);
//       sol_iter++;
//     }
//   group_to_add->clear_occurrences();  //note that we don't delete the occurrences
//   // cout<<"avant de deleter mon group_to _add"<<endl;
//   //delete(group_to_add);
// //   cout <<"aprs de deleter mon groupe to add"<<endl;
// }


// bool Occurrence_Group::contains_similar_occurrence(Occurrence *occurrence)
// {
//   list<Occurrence *>::iterator sol_iter = occurrences->begin();
//   while (sol_iter != occurrences->end())
//     {
//       if (common_reaction(occurrence, *sol_iter))
// 	return true;
//       sol_iter++;
//     }
//   return false;
// }

// /*creates a partition of the occurrences, every two occurrences that share a reaction are placed in the same group*/
// void Occurrence_Group::group_by_reaction(list<Occurrence_Group *> *groups)
// {
//   list<Occurrence *>::iterator sol_iter = occurrences->begin();
//   list<Occurrence_Group *>::iterator group_iter;
//   bool first_match=0;
//   Occurrence_Group *first_matched_group=NULL;
//   //for all occurrences
//   while (sol_iter != occurrences->end())
//     {
//       first_match=0;
//       //for all groups
//       group_iter = groups->begin();
//       while (group_iter != groups->end())
// 	{
// 	  //1er match, on stocke et les suivants on fusionne
// 	  //no match on cre un groupe
// 	  if ((*group_iter)->contains_similar_occurrence(*sol_iter))
// 		{
// 		  //cout<<"cas ou on a trouv une occurrence similaire"<<endl;
// 		  if (!first_match)
// 		    {
// 		      first_match=true;
// 		      (*group_iter)->add(*sol_iter);
// 		      first_matched_group = *group_iter;
// 		    }
// 		  else
// 		    {
// 		      first_matched_group->add_group(*group_iter);
// 		      //the erase method doesn't delete the object !
// 		      //we could add this line : 
// 		      //group_iter->delete_occurrences();
// 		      //(*group_iter)->clear_occurrences();
// 		      //cout<<"je delete un groupe"<<endl;
// 		      delete(*group_iter);  //we don't delete the occurrences inside the group, we just delete the group
// 		      group_iter=groups->erase(group_iter);
// 		      //cout<<"peut-tre cela va-t-il foirer !"<<endl;
// 		      group_iter--;
// 		    }
// 		}
// 	  group_iter++;
// 	}
//       //the occurrence shares no reaction with any other
//       if (!first_match)
// 	{
// 	  //cout<<"je cre un groupe"<<endl;
// 	  Occurrence_Group *new_group = new Occurrence_Group(*sol_iter);
// 	  groups->push_back(new_group);
// 	}
//       sol_iter++;
//     }
// }

// /*creates a partition of the occurrences, every two occurrences that share the same order are placed in the same group*/
// /*note that in some particular cases, a occurrence can be placed in several groups : Ex : 0/1 2 3 0/1 has the same order as 0 2 3 1 and 1 2 3 0*/
// void Occurrence_Group::group_by_order(list<Occurrence_Group *> *groups)
// {
//   list<Occurrence *>::iterator sol_iter = occurrences->begin();
//   list<Occurrence_Group *>::iterator group_iter;
//   bool first_match=1;
//   //for all occurrences
//   while (sol_iter != occurrences->end())
//     {
//       //for all groups
//       first_match=1;
//       group_iter = groups->begin();
//       while (group_iter != groups->end())
// 	{
// 	  //1er match, on stocke 
// 	  //no match on cre un groupe
// 	  if ((*group_iter)->get_occurrences()->front()->same_order(*sol_iter))
// 		{
// 		  first_match=0;
// 		  (*group_iter)->add(*sol_iter);
// 		  break;
// 		}
// 	  group_iter++;
// 	}
//       //the order has not been found before
//       if (first_match)
// 	{
// 	  //cout<<"je cre un groupe"<<endl;
// 	  Occurrence_Group *new_group = new Occurrence_Group(*sol_iter);
// 	  groups->push_back(new_group);
// 	}
//       sol_iter++;
//     }
// }


/**
 *Creates a partition of occurrences according to their topology in the
 *bipartite graph. Two occurrences sharing the same topology are put in
 *the same group.
 *
 * @param groups resulting partition
 */
void Occurrence_Group::group_by_topology(list<Occurrence_Group *> *groups)
{
  list<Occurrence *>::iterator sol_iter = occurrences->begin();
  list<Occurrence_Group *>::iterator group_iter;
  bool first_match=1;
  //for all occurrences
  while (sol_iter != occurrences->end())
    {
      //for all groups
      first_match=1;
      group_iter = groups->begin();
      while (group_iter != groups->end())
	{
	  //1er match, on stocke 
	  //no match on cre un groupe
	  if ((*group_iter)->get_occurrences()->front()->same_topology(*sol_iter))
		{
		  first_match=0;
		  (*group_iter)->add(*sol_iter);
		  break;
		}
	  group_iter++;
	}
      //the order has not been found before
      if (first_match)
	{
	  //cout<<"je cre un groupe"<<endl;
	  Occurrence_Group *new_group = new Occurrence_Group(*sol_iter);
	  groups->push_back(new_group);
	}
      sol_iter++;
    }
}


/**
 * Counts the number of times a topology occurs in the list of
 * occurrences. The list of topologies and their identifiers is given as
 * input to the function. Note that if, in this set of occurrences, a
 * new topology is identified, it will not be reported. Only the
 * topologies that have been observed in the real graph are considered
 * (this means that p-values are not calculated for motifs that are
 * not observed).
 *
 * @param observed_topologies1 first hex number characterising each of these topologies
 * @param observed_topologies2 second hex number characterising each of these topologies
 * @param nb_observed_topologies number of distinct topologies observed among the occurrences of the motif in the real graph
 * @param result counts of the number of same topologies in this set of occurrences.
 */
void Occurrence_Group::count_observed_topologies(long int *observed_topologies1, long int *observed_topologies2, int nb_observed_topologies, int *result)
{
  for (list<Occurrence *>::iterator iter=occurrences->begin(); iter!=occurrences->end(); iter++)
    {
      int j=0;
      bool topology_found=0;
      /*one can arrive at the end of the loop both in the case where
	the topology has been found or not found*/
      while (j<nb_observed_topologies && !topology_found)
	{
	     //for (int j=0;j<nb_observed_topologies;j++)
	  if (observed_topologies1[j] == (*iter)->get_topo1() && observed_topologies2[j] == (*iter)->get_topo2())
	    {
	      result[j]++;
	      topology_found=true;
	    }    
	  j++;
	}
    }
}




// void Occurrence_Group::print(int verbose)
// {
//   list<Occurrence *>::iterator iter=occurrences->begin();
//   while (iter!=occurrences->end())
//     {
//       (*iter)->print(verbose);
//       iter++;
//       cout<<endl;    
//     }
//   cout <<"this group contains "<<occurrences->size()<<" occurrences"<<endl;
// }


/*print the occurrence_group in a file*/
void Occurrence_Group::print(std::ostream& output)
{
  output<<"Adjacency matrix of the group: "<<endl;
  occurrences->front()->print_adj_matrix(output);
  output<<"Number of occurrences: "<<occurrences->size()<<endl;
  output<<"List of occurrences:"<<endl;
  list<Occurrence *>::iterator iter=occurrences->begin();
  while (iter!=occurrences->end())
    {
      //(*iter)->print_enzymes_pathways();
      (*iter)->print(output);
      iter++;
      output<<endl;    
    }
  output<<endl;
}

// /*print the occurrence_group in a file keeping only the occurrences that are intra-pathway*/
// void Occurrence_Group::print_intra_pathway(std::ostream& output)
// {
//   for (list<Occurrence *>::iterator iter=occurrences->begin();iter!=occurrences->end();iter++)
//     {
//       if ((*iter)->is_intra_pathway())
// 	{
// 	  (*iter)->print(output);
// 	  output<<endl;
// 	}
//     }
// }



/**
 *Print the occurrence_group to a file. Depending on the verbose parameter, several levels of detail can be printed.
 *
 * @param output : any output
 * @param verbose : level of detail for each reaction (0: only reaction IDs 1: reaction IDs + EC numbers)
 * 
 */
// void Occurrence_Group::print(std::ostream& output, int verbose)
// {
//   for (list<Occurrence *>::iterator iter=occurrences->begin();iter!=occurrences->end();iter++)
//     {
//       (*iter)->print(output,verbose);
//       output<<endl;
//     }
// }


// void Occurrence_Group::print_bipartite_graphs()
// {
//   for (list<Occurrence *>::iterator iter=occurrences->begin();iter!=occurrences->end();iter++)
//     {
//       (*iter)->print_bipartite_graph();
//       cout<<endl;
//     }
// }

// void Occurrence_Group::print_bipartite_graph()
// {
//   list<Occurrence *>::iterator iter=occurrences->begin();
//   (*iter)->print_bipartite_graph();
//  //  cout<<endl;
// }

void Occurrence_Group::print()
{
  cout<<"Adjacency matrix of the group: "<<endl;
  occurrences->front()->print_adj_matrix();
  cout<<"Number of occurrences: "<<occurrences->size()<<endl;
  cout<<"List of occurrences:"<<endl;
  list<Occurrence *>::iterator iter=occurrences->begin();
  while (iter!=occurrences->end())
    {
      //(*iter)->print_enzymes_pathways();
      (*iter)->print();
      iter++;
      cout<<endl;    
    }
  cout<<endl;
  // cout <<"this group contains "<<occurrences->size()<<" occurrences"<<endl;
}

void Occurrence_Group::print_occurrences()
{
  // cout<<"Adjacency matrix of the group: "<<endl;
//   occurrences->front()->print_adj_matrix();
//   cout<<"Number of occurrences: "<<occurrences->size()<<endl;
//   cout<<"List of occurrences:"<<endl;
  list<Occurrence *>::iterator iter=occurrences->begin();
  while (iter!=occurrences->end())
    {
      //(*iter)->print_enzymes_pathways();
      (*iter)->print();
      iter++;
      cout<<endl;    
    }
  cout<<endl;
  // cout <<"this group contains "<<occurrences->size()<<" occurrences"<<endl;
}
