/* Reaction.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Reaction.h"
#include <iostream>
#include <map>
#include <cstdlib>

//extern map<void*,bool> reactionPtrs;

Reaction::Reaction(char *rea_id)
{
  //reactionPtrs[this]=false;
  id = (char*)malloc((strlen(rea_id)+1)*sizeof(char));
  strcpy(id,rea_id);
  /*we don't keep the name of the reaction cos there's not enough space*/
  //name = (char*)malloc(strlen(rea_name)*sizeof(char));
  //name = strcpy(name,rea_name);

  /*by default the enzyme list is empty which is a good thing for reactions for which there are no known enzymes*/
  enzymes = new  list<Enzyme *>;
  left_compounds = new list<Compound *>;
  right_compounds = new list<Compound *>;
  left_right_compounds = new list<Compound *>;
  no_status_compounds = new list<Compound *>;
  compounds = new list<Compound *>;
  pathways = new list<Pathway *>;
  colours=new list<int>;

  /*by default, the reaction is considered irrelevant*/
  distance = -1;
  first_colour = -1;
}

Reaction::~Reaction()
{

  
  delete(enzymes);
  delete(left_compounds);
  delete(right_compounds);
  delete(left_right_compounds);
  delete(no_status_compounds);
  delete(compounds);
  delete(pathways);
  delete(colours);
  free(id);
}

// This function is required for built-in STL list functions like sort
int Reaction::operator<(const Reaction *rhs) const
{
  cerr<<"je suis dans le tri, je compare "<<id<<" et "<<rhs->id<<endl;
  return !strcmp(this->id,rhs->id);
}


void Reaction::reset()
{
  distance = -1;
  first_colour = -1;
  colours->clear();
}


void Reaction::clear_colours()
{
  first_colour = -1;
  colours->clear();
}

char *Reaction::get_id()
{
  return id;
}

int Reaction::get_distance()
{
  return distance;
}

void Reaction::set_distance(int dist)
{
  distance = dist;
}

int Reaction::get_first_colour()
{
  //return first_colour;
  return colours->front();
}

void Reaction::set_first_colour(int col)
{
  first_colour=col;
}

/*adds a new colour, withdraws duplicates and keeps the list sorted*/
void Reaction::add_colour(int col)
{
  colours->push_back(col);
  colours->sort();
  colours->unique();
}

/*adds new colours, withdraws duplicates and keeps the list sorted*/
void Reaction::add_colours(list<int> *col)
{
  for (list<int>::iterator it=col->begin();it!=col->end();it++)
    colours->push_back(*it);
  colours->sort();
  colours->unique();
}

list<Enzyme *> * Reaction::get_enzymes()
{
  return enzymes;
}

list<Compound *> * Reaction::get_right_compounds()
{
  return right_compounds;
}

list<Compound *> * Reaction::get_left_compounds()
{
  return left_compounds;
}

list<Compound *> * Reaction::get_no_status_compounds()
{
  return no_status_compounds;
}


list<Compound *> * Reaction::get_compounds()
{
  return compounds;
}

list<Pathway *> * Reaction::get_pathways()
{
  return pathways;
}

list<int > * Reaction::get_colours()
{
  return colours;
}


/*test if the compound is already in the compound_list*/
bool in_list(Compound *compound_to_add, list<Compound *> *comp_list)
{
  list<Compound *>::iterator it = comp_list->begin();
  while (it != comp_list->end())
    {
      if (compound_to_add == *it)
	return true;
      it++;
    }
  return false;
}

int Reaction::add_left_compound(Compound *compound_to_add)
{
  list<Compound *>::iterator comp_iter = left_compounds->begin();
  while (comp_iter != left_compounds->end())
    {
      if (compound_to_add == *comp_iter)
	return 0;
      comp_iter++;
    }
  left_compounds->push_back(compound_to_add);
  return 1;
}

int Reaction::add_right_compound(Compound *compound_to_add)
{
  list<Compound *>::iterator comp_iter = right_compounds->begin();
  while (comp_iter != right_compounds->end())
    {
        if (compound_to_add == *comp_iter)
	return 0;
      comp_iter++;
    }
  right_compounds->push_back(compound_to_add);
  return 1;
}

int Reaction::add_left_right_compound(Compound *compound_to_add)
{
  //cout<<"inside add left_right_compound"<<endl;
  list<Compound *>::iterator comp_iter = left_right_compounds->begin();
  while (comp_iter != left_right_compounds->end())
    {
        if (compound_to_add == *comp_iter)
	return 0;
      comp_iter++;
    }
  //cout<<"avant d'ajouter"<<endl;
  left_right_compounds->push_back(compound_to_add);
  return 1;
}

int Reaction::add_no_status_compound(Compound *compound_to_add)
{
  list<Compound *>::iterator comp_iter = no_status_compounds->begin();
  while (comp_iter != no_status_compounds->end())
    {
      if (compound_to_add == *comp_iter)
	return 0;
      comp_iter++;
    }
  no_status_compounds->push_back(compound_to_add);
  return 1;
}


/**
 *Removes a compound from a reaction (because it is considered as a frequent compound). The compound is removed from the lists of compounds of the reaction.
 *
 * @param compound_to_remove Compound to remove
 */
int Reaction::remove_compound(Compound *compound_to_remove)
{
  /*first remove it from the list of compounds*/
  list<Compound *>::iterator comp_iter;
  comp_iter = compounds->begin();
  while (comp_iter != compounds->end())
    {
      if (compound_to_remove == *comp_iter)
	{
	  compounds->erase(comp_iter);
	  break;
	}
      comp_iter++;
    }


  /*then remove it from all the other lists of compounds (left, right, left_right and no_status) - these are not in use for the moment but it might be useful in the future*/
  comp_iter = left_compounds->begin();
  while (comp_iter != left_compounds->end())
    {
      if (compound_to_remove == *comp_iter)
	{
	  left_compounds->erase(comp_iter);
	  break;
	}
      comp_iter++;
    }

  comp_iter = right_compounds->begin();
  while (comp_iter != right_compounds->end())
    {
      if (compound_to_remove == *comp_iter)
	{
	  right_compounds->erase(comp_iter);
	  break;
	}
      comp_iter++;
    }

  comp_iter = left_right_compounds->begin();
  while (comp_iter != left_right_compounds->end())
    {
      if (compound_to_remove == *comp_iter)
	{
	  left_right_compounds->erase(comp_iter);
	  break;
	}
      comp_iter++;
    }


  comp_iter = no_status_compounds->begin();
  while (comp_iter != no_status_compounds->end())
    {
      if (compound_to_remove == *comp_iter)
	{
	  no_status_compounds->erase(comp_iter);
	  break;
	}
      comp_iter++;
    }
  return 1;
}


/*returns the side of the compound: 1 LEFT / 2 RIGHT / 3 UNDEFINED / 0 NOWHERE*/
int Reaction::which_side(Compound *cpd)
{
  list<Compound *>::iterator comp_iter = left_compounds->begin();
  while (comp_iter != left_compounds->end())
    {
      if (cpd == *comp_iter)
	return 1; //the compound is on the left side
      comp_iter++;
    }

  comp_iter = right_compounds->begin();
  while (comp_iter != right_compounds->end())
    {
      if (cpd == *comp_iter)
	return 2; //the compound is on the right side
      comp_iter++;
    }

  comp_iter = no_status_compounds->begin();
  while (comp_iter != no_status_compounds->end())
    {
      if (cpd == *comp_iter)
	return 3; //the compound has no side
      comp_iter++;
    }

  return 0; //the compound is in none of the lists
}

void Reaction::concatenate_compounds()
{
  list<Compound *>::iterator comp_iter;
  copy(left_compounds->begin(),left_compounds->end(),back_inserter(*compounds));  //compounds = left_compounds;
  comp_iter = right_compounds->begin();
  while (comp_iter != right_compounds->end())
    {
      if (!in_list(*comp_iter, compounds))
	compounds->push_back(*comp_iter);
      else
	{
	  //already in list ?
	}
      comp_iter++;
    }

  /*we don't need to add the no_status_compounds ?*/
  comp_iter = no_status_compounds->begin();
  while (comp_iter != no_status_compounds->end())
    {
      if (!in_list(*comp_iter, compounds))
	compounds->push_back(*comp_iter);
      comp_iter++;
    }

  // compounds.unique();  //this line is useless + it doesn't work properly since you need to sort the list first
//   print();
  //cout <<"tailles de left :"<<left_compounds->size()<<" right "<<right_compounds->size()<<" et all "<<compounds->size()<<endl;
}

/**Adds an enzyme to the list of enzymes of the reaction. Also checks
 * that the enzyme is not already in the list
 *
*/
int Reaction::add_enzyme(Enzyme *enzyme_to_add)
{
  /*if the EC number is NA, then it means than it is not known,
    therefore no enzyme is added in the list*/
  // if (!strcmp("NA",enzyme_to_add->get_ec()))
//     {
//       cout<<id<<" NA found"<<endl;
//       return 0;
//     }
  list<Enzyme *>::iterator enz_iter;
  enz_iter = enzymes->begin();
  while (enz_iter != enzymes->end())
    {
      if (enzyme_to_add == *enz_iter)
	return 0;
      enz_iter++;
    }
  enzymes->push_back(enzyme_to_add);
  return 1;
}

/**Adds a pathway to the list of pathways of the reaction. Also checks that the pathway is not already in the list
 *
*/
int Reaction::add_pathway(Pathway *pathway_to_add)  //do we need to traverse this list to verify the pathway is not already in the list ?
{
  list<Pathway *>::iterator pat_iter;
  pat_iter = pathways->begin();
  while (pat_iter != pathways->end())
    {
      if (pathway_to_add == *pat_iter)
	return 0;
      pat_iter++;
    }
  pathways->push_back(pathway_to_add);
  return 1;
}


void Reaction::contains_similar_compounds(Compound *cpd1, Compound *cpd2, bool *matched_compounds, int threshold)
{
  /**/
  //we'll have to do something about multiple matches after
  //on autorise qu'un compos matche avec entre et sortie
  list<Compound *>::iterator cpd_iter=compounds->begin();
  while (cpd_iter != compounds->end())
    {
      if ((*cpd_iter)->is_similar_to(cpd1, threshold))
	matched_compounds[0]=true;
      if ((*cpd_iter)->is_similar_to(cpd2, threshold))
	matched_compounds[1]=true;
      cpd_iter++;
    }
}


int Reaction::arity()
{
  //qu'est-ce que l'arit d'une raction ?
  //1- le nombre de ses composs (arit dans le graphe bipartite)
  //2- le nombre de ractions  laquelle elle est lie (arit dans le graphe de ractions)
  return 0;
}

void Reaction::print_compounds()
{
  for (list<Compound *>::iterator it=compounds->begin(); it!=compounds->end(); it++)
    {
      (*it)->print();
      cout<<" ";
    }
}

void Reaction::print_nb_compounds()
{
  cout<<compounds->size()<<" ";
}

/*print id in file used to draw the graph*/
void Reaction::draw(std::ostream &output)
{
  output <<"\""<<id<<"\"";
}

void Reaction::print_colours()
{
  for(list<int>::iterator it=colours->begin();it!=colours->end();it++)
    cout<<" "<<*it;
  cout<<endl;
}

/*basic print function*/
void Reaction::print()
{
  cout<<id;
  //cout <<"Reaction name :"<<name<<"id "<<id<<endl;
}

void Reaction::print(ostream& output)
{
  output <<id;
  //cout <<"Reaction name :"<<name<<"id "<<id<<endl;
}

/*prints to file the enzymes that catalyse the reaction*/
void Reaction::print_enzymes(ostream& output)
{
  for (list<Enzyme *>::iterator iter=enzymes->begin(); iter!=enzymes->end(); iter++)
    {
      (*iter)->print(output);
      output<<" ";
    }
}

/*prints to file the pathways in which this reaction is included*/
void Reaction::print_pathways(ostream& output)
{
  for (list<Pathway *>::iterator iter=pathways->begin(); iter!=pathways->end(); iter++)
    {
      (*iter)->print(output);
      output<<" ";
    }
}

/*prints to file the reaction and pathways in which this reaction is included*/
void Reaction::print_reaction_pathways(ostream& output)
{
  for (list<Pathway *>::iterator iter=pathways->begin(); iter!=pathways->end(); iter++)
    {
      output<<id;
      output<<"\t";
      (*iter)->print(output);
      output<<endl;
    }
}

/*prints to file the reaction and enzymes that catalyse this reaction*/
void Reaction::print_reaction_enzymes(ostream& output)
{
  for (list<Enzyme *>::iterator iter=enzymes->begin(); iter!=enzymes->end(); iter++)
    {
      output<<id;
      output<<"\t";
      (*iter)->print(output);
      output<<endl;
    }
}

/**
 *Print the reaction to a file. Depending on the verbose parameter, several levels of detail can be printed.
 *
 * @param output : any output
 * @param verbose : level of detail for each reaction (0: only reaction IDs 1: reaction IDs + EC numbers)
 * 
 */
void Reaction::print(std::ostream& output,int verbose)
{
  output<<id;
  if (verbose >= 1)  // we print not only the reaction IDs but also the EC numbers of the enzymes that catalyze these reactions
    {
      for (list<Enzyme *>::iterator iter=enzymes->begin(); iter!=enzymes->end(); iter++)
	{
	  output<<"[";
	  (*iter)->print(output);
	  output<<"]";
	}
    }
}


/**
 *Print the reaction to standard output. Depending on the verbose parameter, several levels of detail can be printed.
 *
 * @param verbose : level of detail for each reaction (0: only reaction IDs 1: reaction IDs + EC numbers)
 * 
 */
void Reaction::print(int verbose)
{
  cout<<id;
  if (verbose >= 1)  // we print not only the reaction IDs but also the EC numbers of the enzymes that catalyze these reactions
    {
      for (list<Enzyme *>::iterator iter=enzymes->begin(); iter!=enzymes->end(); iter++)
	{
	  cout<<"[";
	  (*iter)->print();
	  cout<<"]";
	}
    }
}


