/* Reaction_Graph.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "../include/Reaction_Graph.h"
#include <math.h>
#include <iostream>
#include <fstream>
#include "../include/my_library.h"
#include <cstdlib>
//#include <map>
//using namespace std;

#define __DEBUG__


/*you give a Reaction *, it returns the appropriate integer (index of the Reaction * in the array*/
/*it would be nice to find a way not to traverse this tab*/
/*This should be doable using maps*/
/*maybe this should be put in my_library.cpp*/
int adress_book(Reaction *rea, Reaction **tab_reactions,int size)
{
  for (int i=0;i<size;i++)
    if (rea==tab_reactions[i])
      {
	//cout<<"adress_book, je vais retourner :"<<i<<endl;
	return i;
      }
      
  //cout<<"Reaction_Graph::adress_book, the reaction is not in the tab_reactions"<<endl;
  return -1;
}


/**
 *Constructor of Reaction_Graph. You just need to provide a set of
 *reactions with which you want to build your graph. This set of
 *reactions might have been obtained through a pre-filtering
 *step. This constructor creates several representations of the graph:
 *as an adjacency_matrix and as an adjacency_list.
 *
 * @param reactions array of reactions from which you want to build the graph
 * @param nb_reactions number of reactions in the array
  */
Reaction_Graph::Reaction_Graph(Reaction **reactions, int nb_reactions):Graph(nb_reactions)
{
  /* Done by Graph(int) 
  nb_nodes = nb_reactions;
  */
  
  /*matrix creation + initialisation*/
  /*adjacency_matrix = new bool *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    {
      adjacency_matrix[i] = new bool[nb_nodes];
      for (int j=0;j<nb_nodes;j++)
	adjacency_matrix[i][j]=0;
    }
  */

  /*node labeling*/
  tab_reactions = new Reaction *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    tab_reactions[i]=reactions[i];
  
  /*fill matrix*/
  
  for (int i=0;i<nb_nodes;i++)
    {
      for (list<Compound *> ::iterator comp_iter = tab_reactions[i]->get_compounds()->begin(); comp_iter !=tab_reactions[i]->get_compounds()->end(); comp_iter++)
	{
	  for (list<Reaction *> ::iterator rea_iter = (*comp_iter)->get_reactions()->begin();rea_iter != (*comp_iter)->get_reactions()->end() ;rea_iter++)
	    {
	      int j = adress_book(*rea_iter,tab_reactions,nb_nodes);
	      if (j!=-1)
		adjacency_matrix[i][j] = (j!=i);  //no self loops
	    }
	}
    }
  
  
  /*count nb of edges*/
  nb_edges=0;
  for (int i=0;i<nb_nodes;i++)
    for (int j=i;j<nb_nodes;j++)
      nb_edges+=adjacency_matrix[i][j];
  

    /*create an adjacency_list representation of the graph, useful for further calculation*/
  convert_matrix_to_list();

  /*fill colours (memory has been allocated before in Graph.cpp*/
  for (int i=0;i<nb_nodes;i++)
    {
      copy(tab_reactions[i]->get_colours()->begin(),tab_reactions[i]->get_colours()->end(),back_inserter(*colors[i]));
    }
  
}


/*copy constructor, uses the copy constructor of its base parent*/
Reaction_Graph::Reaction_Graph(Reaction_Graph *ReactionGraphToCopy):Graph(ReactionGraphToCopy)
{
  /*fill information of Reaction_Graph that is not in Graph*/
  nb_edges=ReactionGraphToCopy->get_nb_edges();
  tab_reactions = new Reaction *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    tab_reactions[i]=ReactionGraphToCopy->get_tab_reactions()[i];
}


/**
 *Copy and Filter constructor. The graph is first copied and then filtered using only colours from the motif.
 * @param graph_to_copy graph to copy and filter
 * @param colors_to_filter which colours we should keep in the filtered graph
 * @param motif_size number of colours to keep
 */
Reaction_Graph::Reaction_Graph(Reaction_Graph *graph_to_copy, int *colors_to_filter, int motif_size):Graph(graph_to_copy,colors_to_filter,motif_size)
{

  tab_reactions = new Reaction *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    tab_reactions[i]=graph_to_copy->get_tab_reactions()[parent_nodes[i]];

  /*the number of edges could be calculated but I'm lazy for the moment*/
  nb_edges=-1;
}

/*copy constructor + shuffle, uses the copy constructor of its base parent*/
Reaction_Graph::Reaction_Graph(Reaction_Graph *ReactionGraphToCopy, int *shuffle):Graph(ReactionGraphToCopy,shuffle)
{
  //cout<<"dans shuffle de reaction_graph"<<endl;
  /*fill information of Reaction_Graph that is not in Graph*/
  nb_edges=ReactionGraphToCopy->get_nb_edges();
  //cout<<"avant de copier le tableau de rezation"<<endl;
  /*shuffle labels*/
  /*In fact, it is better not to shuffle this array but only to shuffle the colors*/
  /*The advantage is that when Solution objects are created, you can
    use all the old methods, for instance, the topology in the
    reaction_graph will be unchanged.*/
  tab_reactions = new Reaction *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    tab_reactions[i]=ReactionGraphToCopy->get_tab_reactions()[i];
}


Reaction_Graph::~Reaction_Graph()
{

   if (tab_reactions!=NULL)
    delete[](tab_reactions);
  
   /*Already done by ~Graph
     
   if(adjacency_list!=NULL)
    {
      for (int i=0;i<nb_nodes;i++)
	delete(adjacency_list[i]);
      delete[](adjacency_list);
    }
  
  if (adjacency_matrix!=NULL)
    {
      for (int i=0;i<nb_nodes;i++)
	delete[](adjacency_matrix[i]);
      delete[](adjacency_matrix);
      }
   */
}

// void Reaction_Graph::shuffle()
// {

//   /*apply it to your graph*/
//   /*apply it to colours*/
//   for (int i=0;i<nb_nodes;i++)
//     for (list<int>::iterator it=colors[i]->begin();it!=colors[i]->end();it++){
//       *it=correspondance[*it];
//     }


//   /*apply it to tab_Reaction*/


// }


int Reaction_Graph::get_nb_nodes()
{
  return nb_nodes;
}

int Reaction_Graph::get_nb_edges()
{
  if (nb_edges==-1)
    cerr<<"watch out you are dealing with a graph with a number of edges not calculated"<<endl;

  return nb_edges;
}

Reaction **Reaction_Graph::get_tab_reactions()
{
  return tab_reactions;
}

int Reaction_Graph::get_nb_nodes_enzyme_graph()
{
  int nb_nodes_enzyme_graph=0;
  for (int i=0;i<nb_nodes;i++)
    {
      nb_nodes_enzyme_graph+=tab_reactions[i]->get_enzymes()->size();
    }
  return nb_nodes_enzyme_graph;
}

/*calculate the number of edges in the equivalent enzyme graph*/
/*for this, we traverse every edge in the graph (using the matrix representation) and see in how many edges it has to split*/
int Reaction_Graph::get_nb_edges_enzyme_graph()
{
  int nb_edges_enzyme_graph=0;
  for (int i=0;i<nb_nodes;i++)
    {
      for (int j=i;j<nb_nodes;j++)
	if (adjacency_matrix[i][j])
	  {
	    nb_edges_enzyme_graph+= tab_reactions[i]->get_enzymes()->size() * tab_reactions[j]->get_enzymes()->size();
	  }
    }
  return nb_edges_enzyme_graph;
}

/*draws the graph using neato*/
void Reaction_Graph::draw()
{
  std::ofstream output;
  output.open("bidon.res");
  
  output <<"graph G {"<<endl;
  output << "node[shape=box, style=filled];"<<endl;

  /*declaration of colors*/
  for (int i=0;i<nb_nodes;i++)
    {
      int colour = tab_reactions[i]->get_first_colour();
      switch(colour)
	{
	case 0 : output <<"{node[color="<<COLOUR0<<"]";
	  break;
	case 1 : output <<"{node[color="<<COLOUR1<<"]";
	  break;
	case 2 : output <<"{node[color="<<COLOUR2<<"]";
	  break;
	case 3 : output <<"{node[color="<<COLOUR3<<"]";
	  break;
	case 4 : output <<"{node[color="<<COLOUR4<<"]";
	  break;
	// case 5 : output <<"{node[color="<<COLOUR5<<"]";
// 	  break;
	default : output <<"{node[color="<<COLOUR5<<"]";
	  break;
	}
      tab_reactions[i]->draw(output);
      output <<"}"<<endl;
    }

  /*declaration of edges*/
  for (int i=0;i<nb_nodes;i++)
    {
      for (list<int>::iterator iter=adjacency_list[i]->begin(); iter!=adjacency_list[i]->end(); iter++)
	{
	  /*always rewrite first reaction*/
	  tab_reactions[i]->draw(output);
	  output <<"--";
	  tab_reactions[*iter]->draw(output);
	  output<<endl;
	}
    }

  output <<"}"<<endl;
  output.close();
  
  /*compile with neato*/
  system("neato bidon.res -Tps -o draw.ps -Goverlap=false");
  system("rm bidon.res");
  
  /*draw with gv*/
  system("gv draw.ps &");
}

/*intersection of two lists*/
/*this operation would be useless if the edges of the graph were labelled with the compounds shared by the reactions*/
void intersect(list<Compound *> *l1, list<Compound *> *l2, list<Compound *> *l_inter)  
{
  l_inter->clear();
  //  cerr<<"les tailles des listes sont :"<<l1->size()<<l2->size()<<l_inter->size()<<endl;
  for ( list<Compound *>::iterator it1 = l1->begin(); it1!=l1->end(); it1++)
    {
      for ( list<Compound *>::iterator it2 = l2->begin(); it2!=l2->end(); it2++)
	{
	  if (*it1 == *it2)
	    l_inter->push_back((*it1));
	}
    }
  //return l_inter;
}


/*print function with several levels of detail*/
// void Reaction_Graph::print(int verbose)
// {
//   int nb=0;
//   list< list<Reaction *> *> ::iterator iter = graph->begin();
//   list<Reaction *>::iterator rea_iter;
//   list<Compound *> l_inter;
//   while (iter != graph->end())  // we print the graph as an adjacency list, line by line
//     {
//       rea_iter = (*iter)->begin();
//       while (rea_iter != (*iter)->end())
// 	{
// 	  (*rea_iter)->print(verbose);  //prints the reaction ID (and the ec numbers if verbose > 0)
// 	  if (verbose == 2)
// 	    {
// 	      //cout <<"true"<<endl;
// 	      if (rea_iter != (*iter)->begin())
// 		{
// 		  intersect((*rea_iter)->get_compounds(),(*(*iter)->begin())->get_compounds(),&l_inter);
// 		  for (list<Compound *>::iterator it = l_inter.begin(); it!=l_inter.end(); it++)
// 		    {
// 		      cout<<"(";
// 		      (*it)->print();
// 		      cout<<")";
// 		    }
// 		}
// 	    }
// 	  rea_iter++;
// 	  cout <<"--";
// 	}
//       cout<<endl;
//       iter++;
//       nb++;
//     }
//   cout <<"this connected component contains "<<graph->size()<<" reactions"<<endl;
// }


// void Reaction_Graph::print()
// {
//   int nb=0;
//   list< list<Reaction *> *> ::iterator iter = graph->begin();
//   list<Reaction *>::iterator rea_iter;
//   while (iter != graph->end())
//     {
//       rea_iter = (*iter)->begin();
//       while (rea_iter != (*iter)->end())
// 	{
// 	  (*rea_iter)->print();
// 	  cout <<"--";
// 	  rea_iter++;
// 	}
//       cout<<endl;
//       iter++;
//       nb++;
//     }
//   cout <<"ma composante contient "<<nb<<" ractions"<<endl;
// }


/*prints the sif version of the graph*/
/*sif is a list of edges where an edge is given by Reaction IDs separated by LinkedWith*/
void Reaction_Graph::print_sif(std::ostream& output)
{
  for (int i=0;i<nb_nodes;i++)
    for (int j=i;j<nb_nodes;j++)
      if (adjacency_matrix[i][j])
	{
	  tab_reactions[i]->print(output);
	  output <<"\tlinkedWith\t";
	  tab_reactions[j]->print(output);
	  output<<endl;
	}
 
}

/*prints the matrix version of the graph*/
void Reaction_Graph::print_adjacency_matrix(std::ostream& output)
{
  for (int i=0;i<nb_nodes;i++)
    {
      tab_reactions[i]->print(output);
      output<<" ";
      for (int j=0;j<nb_nodes;j++)
	output<<adjacency_matrix[i][j]<<" ";
      output<<endl;
    }
}

/*prints the matrix version of the graph*/
/*first line is node ids separated by tabulations, then each line is a line of the matrix*/
void Reaction_Graph::print_nodes_matrix(std::ostream& output)
{
  /*print node ids*/
  for (int i=0;i<nb_nodes;i++)
    {
      tab_reactions[i]->print(output);
      output<<"\t";
    }
  output<<endl;
  /*print matrix*/
  for (int i=0;i<nb_nodes;i++)
    {
      for (int j=0;j<nb_nodes;j++)
	output<<adjacency_matrix[i][j];
      output<<endl;
    }
}

void Reaction_Graph::print_labels_enzyme_graph(std::ostream & output,int threshold)
{
  for (int i=0;i<nb_nodes;i++)
    {
      for (list<Enzyme *>::iterator it=tab_reactions[i]->get_enzymes()->begin(); it != tab_reactions[i]->get_enzymes()->end(); it++)
	{
	  (*it)->print(threshold,output);
	  output<<endl;
	}
    }
}

void Reaction_Graph::print_labels_reaction_graph(std::ostream & output,int threshold)
{
  for (int i=0;i<nb_nodes;i++)
    {
      for (list<Enzyme *>::iterator it=tab_reactions[i]->get_enzymes()->begin(); it != tab_reactions[i]->get_enzymes()->end(); it++)
	{
	  (*it)->print(threshold,output);
	  output<<" ";
	}
      output<<endl;
    }
}

// void Reaction_Graph::convert_graph()
// {
//   /*initialization of graph parameters, can't be done before cos the graph ain't built completely*/
//   nb_nodes=graph->size();
//   nb_edges=0;
//   for (list<list<Reaction *> *>::iterator it=graph->begin();it!=graph->end();it++)
//     nb_edges+=(*it)->size()-1;
//   nb_edges=(int)(nb_edges/2); //cos every edge is counted twice in this representation
//   /*memory allocation*/
//   tab_reactions = new Reaction *[nb_nodes];
//   adjacency_list = new list<int> *[nb_nodes];
//   //map<void *,int> *adress_book = new map<void *,int>;

//   /*fill tab_reactions*/
//   int i=0;
//   for (list<list<Reaction *> *>::iterator it=graph->begin();it!=graph->end();it++)
//     {
//       tab_reactions[i]=(*it)->front();
//       i++;
//     }

//   /*fill adjacency_list*/
//   i=0;
//   for (list<list<Reaction *> *>::iterator it=graph->begin();it!=graph->end();it++)
//     {
//       adjacency_list[i]=new list<int>;
//       for(list<Reaction *>::iterator it2=++(*it)->begin(); it2!=(*it)->end(); it2++)  //copy the list without taking the first reaction (note that we use ++it instead of it++)
// 	adjacency_list[i]->push_back(adress_book(*it2,tab_reactions,nb_nodes));
//       i++;
//     }
// }


void Reaction_Graph::convert_list_to_matrix()
{
  /*set all cells to 0*/
  adjacency_matrix = initialize_matrix(adjacency_matrix, nb_nodes); //funnily, this function HAS TO return something in order to be able to manipulate the matrix later

  /*for every line of the adjacency list, report edges to the matrix*/
  for (int i=0;i<nb_nodes;i++)
    for (list<int>::iterator it=adjacency_list[i]->begin();it!=adjacency_list[i]->end();it++)
      adjacency_matrix[i][*it]=1;
}

/**
 *Convert graph from adjacency_matrix representation to adjacency_list representation
 */
void Reaction_Graph::convert_matrix_to_list()
{
  // /*lists creation - memory allocation*/
//   adjacency_list = new list<int> *[nb_nodes];
//   for (int i=0;i<nb_nodes;i++)
//     adjacency_list[i]= new list<int>;

  /*from matrix to lists*/
  for (int i=0;i<nb_nodes;i++)
    for (int j=i;j<nb_nodes;j++)
      {
	if (adjacency_matrix[i][j])
	  {

	    adjacency_list[i]->push_back(j);
	    adjacency_list[j]->push_back(i);
	  }
      }
    
}


/**
 *Prints to file some characteristics of the graph. Properties retained are, for the moment, the number of nodes and the number of edges.
 *
 * @param output where you want ot write
 */
void Reaction_Graph::print_identity_card(std::ostream & output)
{
  output<<"number of nodes "<<nb_nodes<<endl;
  output <<"number of edges "<<nb_edges<<endl;

/*when I have some time, I could implement this: count the number of connected components of my graph*/
//   output <<"number of connected components "<<connected_components->size()<<endl;
  // output <<"sizes of connected components ";
//   for (list<int>::iterator it=sizes.begin(); it!=sizes.end(); it++)
//     output<<*it<<" ";
//   output<<endl;
}









// /*FROM NOW ON, THESE FUNCTIONS PERFORM A FAST SEARCH OF ALL OCCURRENCES OF THE MOTIF IN THE GRAPH
// (what is different from the first search is that it gives also non-linear motifs and it performs the search on a more compact representation of the graph*/


// void print_mat(int **mat, int nl, int nc)
// {
//   for (int i=0; i<nl; i++)
//     print_tab(mat[i], nc);
//   cout<<endl;
// }


// /*print the new representation of the graph*/
// void Reaction_Graph::print_adjacency_list()
// {
//   print_tab_list(adjacency_list,nb_nodes);
// }


/*new version of the enumeration algorithm : 02/05*/
/*faster, able to account for non-linear occurrences, and using a more compact representation of graph (adjacency list as a tab of lists)*/
int Reaction_Graph::enumerate(Solution_Group *solutions, Metabolic_Motif *motif)
{
  int motif_size=motif->size();
  
  /*local copy of the colors of the nodes*/

//   list<int> **colors;
//   colors=(list<int> **)malloc(nb_nodes*sizeof(list<int> *));
//   for (int i=0;i<nb_nodes;i++)
//     {
//       colors[i]=new list<int>;
//       copy(tab_reactions[i]->get_colours()->begin(),tab_reactions[i]->get_colours()->end(),back_inserter(*colors[i]));
//     }

   // cout<<"voici mes couleurs :"<<endl;
//    print_tab_list(colors, nb_nodes);

//    cout<<"voici mon graphe"<<endl;
//    print_adjacency_list();

  int *motif_colors = motif->get_colours();//colors of the motif
  
  // cout<<"voici mon motif"<<endl;
//   print_tab(motif_colors, motif_size);

  /*structures needed for computation*/
  /*first tab that references the reactions I picked*/
  int *reactions_picked;
  reactions_picked=new int[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    reactions_picked[i]=-1;

  /*second tab giving for each reaction of tab 1, the direct neighbors (first row gives the index in tab 1 of the first neighbor and second row gives the index of last neighbor)*/
  int **neighbors;
  neighbors=new int *[2];
  for (int i=0;i<2;i++)
    neighbors[i]=new int[nb_nodes];
  for (int i=0;i<2;i++)
    for(int j=0;j<nb_nodes;j++)
      neighbors[i][j]=-1;

  /*third tab giving for each reaction, if it has been picked (first column) or eliminated (second column)*/
  bool **picked_eliminated;
  picked_eliminated=(bool**)malloc(nb_nodes*sizeof(bool*));
  for (int i=0;i<nb_nodes;i++)
    picked_eliminated[i]=(bool*)malloc(2*sizeof(bool));

  for (int i=0;i<nb_nodes;i++)
    for(int j=0;j<2;j++)
      picked_eliminated[i][j]=0;

  /*tab of size |motif| to store choices for a given topology*/
  int *choices;
  choices=new int[motif_size];   //all digits are used (except for the first one) : then the first digits are for distance 1, then distance 2... (according to the topology being tested)
  bool *last_choice;  // last_choice[i]=1 if all choices have been explored for distance i
  last_choice=new bool[motif_size];
  last_choice[0]=0; //useful cos tested when backtracking
  
  int end=0; //initialisation ? position of first digit equal to 0 in topology
  
/**************ADDED 20/02/06***************/
  
  /*matrix giving all topologies to test when starting from a node. Every line of the matrix is a different topology*/
  /*Example: for a motif of size 4, the topology 0 1 2 1 means you pick 1 reaction at distance 1, 2 reactions at distance 2 and 1 reaction at distance 3 (the first cell is never used)*/
  /*The number of toplogies for a motif of size k is equal to the number of ways to decompose k as a sum of integers: 2^k -1*/
  int **topology_matrix;
  //cout<<"motif_size="<<motif_size<<endl;
  int nb_topologies= (int)(pow((double)2,(double)motif_size-2));
  //cout<<"nb_topologies= "<<nb_topologies<<endl;
  topology_matrix = new int*[nb_topologies];
  for (int i=0;i<nb_topologies;i++)
    topology_matrix[i]=new int[motif_size+1];//the number of columns should be motif_size - 1 but I think that at some point we are testing one step too far.
  initialize_topology_matrix(topology_matrix,nb_topologies,motif_size+1);
  fill_topology_matrix(topology_matrix,motif_size);


/**************END OF ADDED 20/02/06***************/


  int pos=0; //postion of pointer in reactions_picked
  int *dist_pos;  //dist_pos[i] gives the position of the first reaction at distance i in reactions_picked (offset)
  dist_pos=new int[motif_size+1];
  
  /*for optimizing test colors : these tabs are never used at a global scope, nevertheless, they are declared here so that the memory allocation is done only once (even if the whole space won't be needed at each call)*/
  bool *matched_colors;
  matched_colors=new bool[motif_size];
  int *color_choice; //useful to fill color_candidates
  color_choice=new int[motif_size];
  int *color_sizes;  //useful to fill color_candidates
  color_sizes=new int[motif_size];

  bool test=0; //test color result, if test not ok, then move to the next choice

  int *solution;
  solution = new int[motif_size];

  /*useful to handle multiple matches*/
  list<int *> *color_candidates; //if all nodes are unicolor, there will be only one tab, otherwise there si a tab for all possible sets of colors related to a group of candidate nodes
  color_candidates = new list<int *>;
  int *nb_color_candidates;  // needed to optimize the space : nb_color_candidates[j] gives the nb of tabs to consider in color_candidates at level j
  nb_color_candidates = new int[motif_size];
  
    
  /*special case : there's only one enzyme in the motif*/
  if (motif_size==1)
    {
      for (int i=0;i<nb_nodes;i++)
	{
	  solution[0]=i;
	  solutions->add(new Solution(solution, motif_size, tab_reactions, parent_nodes));
	}
      delete[](reactions_picked);
      for (int i=0;i<2;i++)
	delete[](neighbors[i]);
      delete[](neighbors);
      for (int i=0;i<nb_nodes;i++)
	free(picked_eliminated[i]);
      free(picked_eliminated);
      delete[](choices);
      delete[](last_choice);
      delete[](dist_pos);
      for (int i=0;i<nb_topologies;i++)
	delete[] (topology_matrix[i]);
      delete[] topology_matrix;
      delete[](matched_colors);
      delete[](color_choice);
      delete[](color_sizes);
      delete[](solution);
      delete[](nb_color_candidates);
      for (list<int *>::iterator it=color_candidates->begin();it!=color_candidates->end();it++)
	delete[](*it);
      delete(color_candidates);
      return 1;
    }



  for (int i=0;i<nb_nodes;i++)
    {
      // cout <<"maintenant , je traite la reaction :"<<i<<endl;
      pos=0;
      //reset reactions_picked and picked_eliminated
      for (int k=0;k<nb_nodes;k++)
	{
	  reactions_picked[k]=-1;
	  picked_eliminated[k][0]=0;
	  neighbors[0][k]=-1;
	  neighbors[1][k]=-1;
	}
      
      //pick reaction i
      reactions_picked[pos]=i;
      pos++;
      dist_pos[0]=0;
      picked_eliminated[i][0]=1;
      solution[0]=i;
      
      /*initialize color_candidates*/
      initialize_color_candidates(color_candidates, colors[i],motif_size);
      nb_color_candidates[0]=colors[i]->size();
      	

      /*get neighbors at distance 1*/
      pick_neighbors(i,adjacency_list, picked_eliminated, neighbors, reactions_picked, &pos);
      dist_pos[1]=1;
      dist_pos[2]=pos;  //reactions at distance 2 will be added from position pos
            
      for (int t=0;t<nb_topologies;t++)  //for every topology
	{
	  //cout <<"je rentre dans la boucle des topologies"<<endl;
	  /*initialize choices*/
	  int j=0; //distance being considered 
	  int pos_choices=1;  //current position of cursor in tab choices (it is also the position of cursor in tab solution)
	  end=1;
	  while (topology_matrix[t][end]!=0)  //initialisation of end
	    end++;
	  // cout<<"j'ai initialis end :"<<end<<endl;
	  for (j=1; j<end;j++)
	    {
	      first_choice(choices+pos_choices, topology_matrix[t][j]);
	      pos_choices+=topology_matrix[t][j];
	      last_choice[j]=0;
	    }
	  pos_choices=1;

	  j=1;
	  bool out=0;
	  while (!out)   //for all choices
	    {
	      /*generation of candidates + test*/
	      //choose nodes among neighbors dist 1 according to choices_j : generate candidates  
	      //nb neighbors dist j = dist_pos[j+1] - dist_pos[j]  (n)
	      //nb nodes we have to choose : topologies[j-1]  (m)
	      if (topology_matrix[t][j] > dist_pos[j+1] - dist_pos[j])  //we don't have enough candidates
		{
		  //cout <<"je suis dans le cas ou je n'ai pas assez de candidats"<<endl;
		  test=0;
		  last_choice[j]=1;
		}
	      else  //we have enough candidates
		{
		  //cout <<"je vais choisir parmi mes candidats"<<endl;
		  for (int k=pos_choices;k<topology_matrix[t][j]+pos_choices;k++)
		    {
		      // cout <<"j'ajoute des candidats"<<endl;
		      solution[k]=reactions_picked[dist_pos[j]+choices[k]];
		      // cout <<"aprs rajout des candidats  la solution"<<endl;
		    }
		  // cout<<"j'ai "<<topologies[j]<<" candidats"<<endl; 
		  
		  /*to test a set of candidate nodes, you have to test all combinations of colors these nodes can give (multicolor nodes)*/
		  /*therefore, we build a list of tabs (candidate_colors) summarizing all color_choices possible for a set of nodes*/
		  //cout<<"on est au niveau j ="<<j<<endl;
		  //cout<<"nombre de lignes  considrer :"<<endl;
		  //print_tab(nb_color_candidates, motif_size);
		  //cout<<"avant fill color_candidates ="<<endl;
 		  //print_list_tab(color_candidates->begin(), color_candidates->end(),motif_size);
		  fill_color_candidates(solution+pos_choices, topology_matrix[t][j],pos_choices, color_candidates, nb_color_candidates+j-1, colors, motif_size,color_choice, color_sizes);
		  //cout<<"nombre de lignes  considrer aprs fill:"<<endl;
 		  //print_tab(nb_color_candidates, motif_size);
		  //cout<<"aprs fill color_candidates ="<<endl;
		  //print_list_tab(color_candidates->begin(), color_candidates->end(),motif_size);
		  //cout <<"je vais tester mes candidats"<<endl;
		  //print_tab(solution+pos_choices, topologies[j]);
		  //cout<<"nombre de lignes  considrer pour le test"<<nb_color_candidates[j]<<endl;
		  /*test colors*/
		  if (j==end-1)  //if one of the color_choice is ok, the test is ok and you don't have to test the others
		    test=final_test(color_candidates,pos_choices+topology_matrix[t][j], nb_color_candidates[j], motif_colors, motif_size, matched_colors);
		  else //all color choices must be tested, if one of them is ok, the test is ok but you still have to test them all to get rid of the 'not ok' ones
		    //  test=1;  //just to test the execution time without this pruning
		    test=partial_test(color_candidates, pos_choices+topology_matrix[t][j],nb_color_candidates+j,motif_colors, motif_size, matched_colors);

		  //cout <<"rsultats du test :"<<test<<endl;
		}
	      
	      
	      last_choice[j]=is_last_choice(choices+pos_choices,topology_matrix[t][j],dist_pos[j+1]-dist_pos[j]);
	     

	      /*treatment after test*/
	      				
	      if (test  && j<end-1)  //so far, the colors are ok, let's extend the solution to the next level (distance)
		{
		  //cout <<"le test est ok, j'tends"<<endl;
		  //add candidates to partial_solution
		  
		  //extend based on choices_j;
		  // cout<<"avant pick_neighbors, pos = "<<pos<<endl;
		  for (int i=0;i<topology_matrix[t][j];i++)
		      pick_neighbors(*(solution+pos_choices+i),adjacency_list, picked_eliminated, neighbors, reactions_picked, &pos);
		  // cout <<"aprs pick_neighbors, pos ="<<pos<<endl;
// 		  cout<<"reactions picked :"<<endl;
// 		  print_tab(reactions_picked, nb_nodes);
		  
		  // cout<<"neighbors :"<<endl;
// 		  print_mat(neighbors, 2,nb_nodes);
		  
// 		  cout<<"picked_eliminated:"<<endl;
// 		  print_mat(picked_eliminated,nb_nodes, 2);
// 		  cout<<"pos_choices avant incrmentation "<<pos_choices<<endl;
		  pos_choices += topology_matrix[t][j];
		  j++;
		  // cout<<"dist_pos avant extension"<<endl;
// 		  print_tab(dist_pos, motif_size);
		  dist_pos[j+1]=pos;
		  // cout<<"dist_pos aprs extension"<<endl;
// 		  print_tab(dist_pos, motif_size);
// 		  cout<<"aprs extension ,pos_choices = "<<pos_choices<<endl; 
// 		  cout<<"topologies :"<<endl;
// 		  print_tab(topologies,motif_size);
// 		  cout<<"choices "<<endl;
// 		  print_tab(choices,motif_size);
		}
	      else
		{
		  if (test && j==end-1)
		    {
		      //cout <<"J AI TROUVE UNE SOLUTION"<<endl;
		      
		      solutions->add(new Solution(solution, motif_size, tab_reactions, parent_nodes));
		      // print_tab(solution, motif_size);
// 		      cout<<endl;
		    }
		  
		  /*either the test was good but we reached the end or the test was bad, anyway, we need to move to next choice*/
		  //cout<<"je retire mes candidats de ma solution car le test tait mauvais ou bon et solution"<<endl;
		  /*we backtrack as long as all choices have been explored for level j*/
		  // cout<<"avant de dcrmenter ventuellement plusieurs fois, last_choice ="<<endl;
// 		  print_tab(last_choice, motif_size);
		  while (last_choice[j])
		    {
		      //cout <<"je suis dans last choice, je vais dcrmenter j= "<<j<<endl;
		      /*initialize choices : you have to reinitialize choices while computing because when you come back to level j, you will need to start from the first choice*/
		      first_choice(choices+pos_choices, topology_matrix[t][j]);
		      last_choice[j]=0;

		      j--;
		      pos_choices-=topology_matrix[t][j];
		      if (j>0)
			{
			  // cout<<"avant unpick neighbors"<<endl;
			  
// 			  cout<<"reactions picked :"<<endl;
// 			  print_tab(reactions_picked, nb_nodes);
			  
// 			  cout<<"neighbors :"<<endl;
// 			  print_mat(neighbors, 2,nb_nodes);
			  
// 			  cout <<"picked_eliminated"<<endl;
// 			  print_mat(picked_eliminated, nb_nodes, 2);
			  
			  unpick_neighbors(dist_pos[j+1],picked_eliminated, neighbors, reactions_picked, &pos);
			  // cout<<"aprs unpick neighbors"<<endl;
// 			  cout<<"reactions picked :"<<endl;
// 			  print_tab(reactions_picked, nb_nodes);
			  
// 			  cout<<"neighbors :"<<endl;
// 			  print_mat(neighbors, 2,nb_nodes);
			  
// 			  cout <<"picked_eliminated"<<endl;
// 			  print_mat(picked_eliminated, nb_nodes, 2);
			}
		    }
		  /*after backtracking there is still a choice not yet explored*/
		  if (j>=1)
		    {
		      //cout <<"je passe au choix suivant"<<endl;
// 		      cout<<"dist_pos ="<<endl;
// 		      print_tab(dist_pos, motif_size);
// 		      cout <<"je choisis "<<topologies[j]<<" parmi "<<dist_pos[j+1]-dist_pos[j]<<endl;
		      next_choice(choices+pos_choices,topology_matrix[t][j],dist_pos[j+1]-dist_pos[j]) ;//returns 0 if the last choice has been tested
		    }
		  /*after backtracking, all choices have been explored, we move to the next topology*/
		  else 
		    out=1;
		  
		}
	    }
	  //cout <<"let's try a new topology"<<endl;
	  //last_topology = !next_topology(topologies+1, motif_size-1);  //returns 0 if the last topology has been tested
	  // cout <<"est-ce la dernire topologie ?"<<last_topology<<endl;
// 	  cout<<"topology  ="<<endl;
// 	  print_tab(topologies, motif_size);
	}

      /*eliminate reaction i*/
      picked_eliminated[i][1]=1;
    }
  
  //cerr<<"je vais deleter mes trucs"<<endl;
  /*free memory*/
  delete[](reactions_picked);
  for (int i=0;i<2;i++)
    delete[](neighbors[i]);
  delete[](neighbors);
  for (int i=0;i<nb_nodes;i++)
    free(picked_eliminated[i]);
  free(picked_eliminated);
  delete[](choices);
  delete[](last_choice);
  delete[](dist_pos);
  for (int i=0;i<nb_topologies;i++)
    delete[] (topology_matrix[i]);
  delete[] topology_matrix;
  delete[](matched_colors);
  delete[](color_choice);
  delete[](color_sizes);
  delete[](solution);
  delete[](nb_color_candidates);
  for (list<int *>::iterator it=color_candidates->begin();it!=color_candidates->end();it++)
    delete[](*it);
  delete(color_candidates);
  // for (int i=0;i<nb_nodes;i++)
//     delete(colors[i]);
//   free(colors);
  return 1;
}

