#include <iostream>
#include "../FilteredGraph.h"
#include "../Data.h"
#include "../my_library.h"
#include <unistd.h>

/**
 *
 */
void printUsage(void){
  cout << "Usage : test -G graphFile -L labelFile -m motif -l value1 -g value2 -s value3"<<endl;
  cout << "\t-G : the adjacency matrix of the graph"<< endl;
  cout << "\t-L : the file containing the color of node; first line correspond to first node, etc"<< endl;
  cout << "\t-m : motif to find in the graph seperate by a point"<< endl;
  cout << "\t-l : the local gap" << endl;
  cout << "\t-g : the global gap"<< endl;
  cout << endl << "exemple pour la version actuel : " << endl <<"./test -G exemple/ligneRougeGraph -L exemple/ligneRougeLabel -m 1.1.1.1.1 -l 1 -g 2" << endl;
  cout << endl;
}

/**
 *
 */
int main (int argc, char **argv){
  
 
  
  /////////// Variables
  char *data_file = new char[255];
  Data *myData;
  int local_gap, global_gap, nb_color, motif_size;
  int sigma_E = 3;
  char *graph_file ;
  char *labels_file ;

  char **EC_numbers= new char *[MAX_NB_EC];
  for (int i=0; i<MAX_NB_EC; i++)
    EC_numbers[i]=new char[MAX];

  int nb_EC;
  FilteredGraph *graph; 
  int * colorsToFilter = new int[255];
  int * motif;
  list<int*> solutions;
  
  char *endptr = new char[255];
  char *tmp = new char[255];
  char *ptr_endptr=endptr;
  char *ptr_tmp=tmp;
  int j = 0;
  int count=0;

  int option_char;
  while ((option_char = getopt (argc, argv,"G:m:L:g:l:h")) != EOF){
    switch (option_char){  

    case 'G' : 
      graph_file = new char[strlen(optarg)+1];
      strcpy(graph_file,optarg);
      break;

    case 'L' :
      labels_file = new char[strlen(optarg)+1];
      strcpy(labels_file,optarg); 
      break;
      
    case 'm':
      tmp = optarg;    
      while(tmp[j] != '\0'){
	if(tmp[j] == '.')
	  count++;
	j++;
      }
      count++;
      motif_size = count;
      tmp = optarg;
      motif = new int[count];
      count = 0;
      j=0;

      do{
       motif[j]= strtol(tmp,&endptr,10);
	if(*endptr!='\0')
	  tmp=endptr+1;
	j++;
      }
      while(*endptr!='\0');
      break;
    
    case 'g': 
      global_gap = atoi(optarg);
      cout << endl << "global gap = " << global_gap << endl;
      break;

    case 'l':
      local_gap = atoi(optarg);
      cout << endl << "local gap = " << local_gap << endl;
      break;
      
    case 'h': printUsage();
      exit(1);
	
    case '?': cout << "Error in command line " << endl;
      printUsage();
      exit(1);
    }
  }
  delete []ptr_endptr;
  delete []ptr_tmp;
  ///////////////////////////////////////////
  //////////// creation du graph ////////////
  strcpy(data_file,"primCpdsSmmReactionsCompounds.col");
  myData = new Data(data_file);
  nb_EC=myData->fill_tab_enzymes(sigma_E,EC_numbers);
  for (int i=0;i< nb_EC; i++)
    cout<<EC_numbers[i]<<" ";
  cout<<endl;
  //fin creation du graph, la conversion vers la liste est effectue
  ///////////////////////////////////////////
  
  ///////////////////////////////////////////
  ///// construction of filtered graph //////

  //test graph_test
  int k =0 ;
  j=0;
  count = 0;
  bool id = false;
  cout << "motif_size = " << motif_size << endl;
  for(int i = 0 ; i< motif_size; i++){
    k=0;
    while(k<j){
      if(colorsToFilter[k] == motif[i]){
	id = true;
	break;
      }
      k++;
    }
    if(id == true){
      id = false;
      continue;
    }
    colorsToFilter[j] = motif[i];
    count++;
    cout << "colorsToFilter[j] = " << colorsToFilter[j] << endl;
    j++;
  }
  nb_color = count;

  graph = new FilteredGraph(graph_file, labels_file, EC_numbers, nb_EC,colorsToFilter,nb_color,local_gap,global_gap);

  cout << endl << "Filtered Graph : " << endl;
  graph->print_distance_list();
  
  // end of construction of filtered graph //
  ///////////////////////////////////////////

 cout << endl;

 int nb_solution;
 nb_solution = graph->enumerate(&solutions,motif_size,motif);
 
 cout << "number of matching : " <<  nb_solution << endl <<endl;
 cout << "result in original graph : "<< endl;
 for(list<int*>::iterator it = solutions.begin();
     it != solutions.end();
     it++){
   for(int i = 0 ; i < motif_size ; i++){
     cout << (*it)[i] << " ";
   }
   cout << endl;
 }
 cout << endl;
   
 ///////////////////////////////////////////
 ////////////    free memory    ////////////
 
 delete []motif;
 delete []data_file;
 delete []graph_file;
 delete []labels_file;
 deleteMatrix(EC_numbers,MAX_NB_EC);
 
 delete []colorsToFilter;
 delete myData;
 delete graph;
 cout << "Fin du programme"<< endl;
 return 0;
}

