/* my_library.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <iostream>
#include <cstdlib>
#include <cstring>
#include "../include/my_library.h"


using namespace std;

/*those are string functions*/
/*I first thought strlen, strcmp, strcpy didn't work properly but I was wrong*/
/*those functionsare now useless and should be replaced by the standard str functions (which are faster)*/
int string_length(char *str)
{
  int i=0;
  while (str[i]!='\0')
    {
      i++;
    }
  return i;
}

void string_copy(char *str1, char *str2)
{
  int i=0;
  for (i=0;i<string_length(str2);i++)
    {
      str1[i]=str2[i];
    }
  str2[i+1]='\0';
}

bool string_compare(char *str1, char *str2)
{
  if (string_length(str1)!=string_length(str2))
    return false;
  for (int i=0;i<string_length(str1);i++)
    {
      if (str1[i]!=str2[i]) 
	return false;
    }
  return true;
}


/*************************stat functions*********************/

/**
 * Calculates the arithmetic mean of a collection of integers
 * @param tab collection of integers
 * @param size size of the collection
 * @return mean arithmetic mean
 */
double mean(int *tab,int size)
{
  double sum=0.0;
  double mean=0.0;
  for (int i=0;i<size;i++)
    sum+=tab[i];
  mean = sum/size;
  return mean;
}

/**
 * Calculates the variance (sum of squares of deviation to the mean) of a collection of integers
 * @param tab collection of integers
 * @param size size of the collection
 * @param mean arithmetic mean
 * @return variance
 */
double variance(int *tab,int size,double mean)
{
  double sum_sq=0.0;
  double variance=0.0;
  for (int i=0;i<size;i++)
    sum_sq+=(tab[i] - mean)*(tab[i] - mean);
  variance = sum_sq/(size-1);
  return variance;
}

/*************************parsing funtions ********************/

/**
 * Extract a word from a line withdrawing the blanks.
 * @param str string that contains word to extract
 * @param word string where the word to extract will be stored
 * @param end_of_word, character (other than '\0' or '\n') indicating the end of the word
 * @param i reference to the position of the word in the string, the return value is the end of the word
 *
 */
void extract_word(char *str, char *word, char end_of_word, int *i)
{
  //extracts a word from a line, withdrawing the blanks
  int j=0;
  while (str[*i] != '\0' && str[*i] != '\n' && str[*i] != end_of_word )
    {
      if (str[*i]!=' ')
	{
	  word[j]=str[*i];
	  (*i)++;
	  j++;
	}
      else  (*i)++;
    }
  word[j]='\0';
  //cout<<word<<endl;
  //  cout<<*i<<endl;
  if (str[*i] != '\0' && str[*i] != '\n')
    (*i)++;
}

/*counts the number of times a character appears within a string*/
int count_char(char *str, char c)
{
  int count=0;
  int i=0;
  while(str[i]!='\0')
    {    
      count+=(str[i]==c);
      i++;
    }
  return count;
}

/*splits a string of integers separated by white space into array of integers*/
void split(char *str, int *tab, int n)
{
  char number[MAX];
  int j=0;
  for (int i=0;i<n;i++)
    {
      int k=0;
      while(str[j]!=' ' && str[j]!='\0')
	{
	  number[k]=str[j];
	  j++;
	  k++;
	}  
      number[k]='\0';    
      tab[i]=atoi(number);
      j++;
    }
}

/*we only sort the non zero elements of the array*/
/*we use selection sort*/
/*note that if we find a max we immediately change it*/
void sort(int *tab, int nb_groups)
{
  int max=0;
  int tmp=0;
  for (int i=0;i<nb_groups-1;i++)
    {
      max=i;
      for (int j=i+1;j<nb_groups; j++)
	{
	  if (tab[j]>tab[i])
	    {
	      max=j;
	      tmp=tab[max];
	      tab[max]=tab[i];
	      tab[i]=tmp;
	    }
	}
    }
}

/*sort function used to sort p-values, output is a shuffled permutation*/
/*we only sort the non zero elements of the array*/
/*we use selection sort*/
/*note that if we find a max we immediately change it*/
/*this function actually sorts in ascending order*/
void sort(double *tab, int nb_groups, int *permutation)
{
  int max=0;
  double tmp=0;
  int tmp2=0;
  for (int i=0;i<nb_groups-1;i++)
    {
      max=i;
      for (int j=i+1;j<nb_groups; j++)
	{
	  if (tab[j]<tab[i])
	    {
	      max=j;
	      /*exchange positions with new max*/
	      tmp=tab[max];
	      tab[max]=tab[i];
	      tab[i]=tmp;

	      /*do the same for the result file permutation*/
	      tmp2=permutation[max];
	      permutation[max]=permutation[i];
	      permutation[i]=tmp2;

// 	      permutation[i]=j;
// 	      permutation[j]=i;
	    }
	}
    }
}




/*return 1 if tab1 is greater than tab2*/
bool tabcmp(int *tab1, int *tab2)
{
  for (int i=0;i<4;i++)
    {
      if (tab1[i]==-1)
	return 0;
      if (tab2[i]==-1)
	return 1;
      if (tab1[i]>tab2[i])
	return 1;
      if (tab2[i]>tab1[i])
	return 0;
    }
  return 1;
}



/*we only sort the non zero elements of the array*/
/*we use selection sort*/
/*note that if we find a max we immediately change it*/
void sort(int **tab, int nb_groups)
{
  int max=0;
  int* tmp=new int[4];
  for (int i=0;i<nb_groups-1;i++)
    {
      max=i;
      for (int j=i+1;j<nb_groups; j++)
	{
	  if (!tabcmp(tab[j],tab[i]))  //there is a negation here because we want to sort in increasing order
	    {
	      max=j;
	      for (int k=0;k<4;k++)
		{
		  tmp[k]=tab[max][k];
		  tab[max][k]=tab[i][k];
		  tab[i][k]=tmp[k];
		}
	    }
	}
    }
  delete[] tmp;
}

// /*tells you if ec1 is greater than ec2, the comparison is done level by level*/
// bool ec_compar(char *ec1,char *ec2)
// {
//   /*count number of dots in ec number*/
//   int nbdots1=0;
//   int nbdots2=0;
//   for (int i=0;i<strlen(ec1);i++)
//     nbdots1 += (ec1[i]=='.');
//   for (int i=0;i<strlen(ec2);i++)
//     nbdots2 += (ec2[i]=='.');

//   if (

// }


// /*sort a collection of strings*/
// /*we only sort the non zero elements of the array*/
// /*we use selection sort*/
// /*note that if we find a max we immediately change it*/
// void sort(char **tab, int nb_groups)
// {
//   int max=0;
//   char *tmp=(char*)malloc(MAX*sizeof(char));
//   for (int i=0;i<nb_groups-1;i++)
//     {
//       max=i;
//       for (int j=i+1;j<nb_groups; j++)
// 	{
// 	  if (tab[j]>tab[i])
// 	    {
// 	      max=j;
// 	      strcpy(tmp,tab[max]);
// 	      //tmp=tab[max];
// 	      strcpy(tab[max],tab[i]);
// 	      //tab[max]=tab[i];
// 	      strcpy(tab[i],tmp);
// 	      //tab[i]=tmp;
// 	    }
// 	}
//     }
//   free(tmp);
// }


void print_prout()
{
  cout<<"prout"<<endl;
}




/*operations on multisets*/

/*return the factorielle of a number*/
int fact(int n)
{
  cerr<<"fact de "<<n<<endl;
  if (n==0)
    return 1;
  else
    return n*fact(n-1);
}

/*returns the number of choices of k elements among n (repetitions are
  not allowed, order does not matter)*/
int choose(int k, int n)
{
  // cerr<<"choose de "<<k<<","<<n<<endl;
//   cerr<<"fact de 39 = "<<fact(39)<<endl;
//   return (int)(int)fact(n)/(int)(fact(k) * fact(n-k));
/*direct implementation is useless since fact(13) means overflow*/

  if (k>n)
    return 0;

  int b[n+1];

  b[0] = 1;
  for (int i=1;i<=n;++i)
    {
      b[i]=1;
      for (int j=i-1;j>0;--j)
	b[j] += b[j-1];
    }

  //cerr<<"choose de "<<k<<","<<n<<endl;
  return b[k];

}


/*returns the number of multichoices of k elements among n (repetitions are allowed, order does not matter)*/
int multichoose(int k, int n)
{
  //cerr<<"multichoose de "<<k<<","<<n<<endl;
  return choose(k,n+k-1);

}

/*returns the number of multiset which have already been enumerated*/
/*This is in fact an approximation based on the first non-null cell of
  the tab of colours*/
int get_nb_motifs_done(int *tab, int k, int nb_colors)
{
  //print_tab(tab,k);

  /*in case no motifs have been enumerated*/
  if (tab[k-1]==0)
    return 0;

  /*otherwise*/
  int pos=0;
  while (tab[pos]==0)
      pos++;

  int val = tab[pos];

  // cout<<"val "<<val<<endl;
//   cout<<"pos "<<pos<<endl;

  int nb_motifs_done = 0;
  
  /*the job done corresponds to sum over all done blocks : pos is the
    position of the first non-zero digit in color_choice, val is the
    value of this non-zero digit*/
  /*the size of the block is determined by the number of colours and
    the size of the motif.*/
  /*maybe we should only take the first digit*/
  for (int i=0;i<val;i++)
    nb_motifs_done+=multichoose(k -pos-1, nb_colors-i);
  
  //cout<<"le nombre de motifs faits est "<<nb_motifs_done<<endl;

  return nb_motifs_done;
}

/*this function looks very much like the function in Reaction Graph but the number of colors per position is constant here and given by nb_colors*/
/*another big difference is that the order doesn't matter here, meaning we don't do a cartesian product but a multichoice*/
bool next_multichoice(int *color_choice, int n, int nb_colors)
{
  /*get the rightmost candidate whose colors have not all been explored and try a new color, then reset colors for candidates located at its right*/
  int i=n-1;
  bool all_colours_explored=false;
  
  while (color_choice[i]==nb_colors-1)
    {
      if (i!=0)
	i--;
      else
	{
	  all_colours_explored=true;
	  break;
	}
    }
  
  if (all_colours_explored) //stop condition : all choices have been explored
    {
      // cout<<"problme je suis dans le dernier choix de couleurs"<<endl;
      return 0;
    }



  // /*get the rightmost candidate whose colors have not all been explored and try a new color, then reset colors for candidates located at its right*/
//   int i=n-1;
//   while (color_choice[i]==nb_colors-1)
//     i--;
//   if (i<0) //stop condition : all choices have been explored
//     {
//       // cout<<"problme je suis dans le dernier choix de couleurs"<<endl;
//       return 0;
//     }

  else  //try the next color for the rightmost candidate whose colors have not all been explored and reset colors for candidates located at its right
    {
      color_choice[i]++;
      for (int j=i+1;j<n;j++)
	color_choice[j]=color_choice[i];
      return 1;
    }
}


/*different version that enables to skip cases if we know they won't give any occurrences*/
bool next_multichoice(int *color_choice, int *color_choice2, int n, int *nb_children, int nb_colors)
{
  int i=n-1;
  while (color_choice[i]==nb_colors-1)
    i--;
  if (i<0) //stop condition : all choices have been explored
    {
      return 0;
    }

  else  //try the next color (skipping some of them since you know they won't give any occurrences) for the rightmost candidate whose colors have not all been explored and reset colors for candidates located at its right
    {
      color_choice[i]+=nb_children[color_choice2[i]];
      for (int j=i+1;j<n;j++)
	color_choice[j]=0;
      return 1;
    }
}

/*moves to next multichoice within a block define by a start and a stop for each position*/
bool next_multichoice_in_block(int *color_choice, int n, int *start, int *stop)
{
  // int i=n-1;
//   while (color_choice[i]==stop[i]-1)
//     i--;
//   if (i<0) //stop condition : all choices have been explored
//     {
//       return 0;
//     }

/*get the rightmost candidate whose colors have not all been explored and try a new color, then reset colors for candidates located at its right*/
  int i=n-1;
  bool all_colours_explored=false;
  
  while (color_choice[i]==stop[i]-1)
    {
      if (i!=0)
	i--;
      else
	{
	  all_colours_explored=true;
	  break;
	}
    }
  
  if (all_colours_explored) //stop condition : all choices have been explored
    {
      // cout<<"problme je suis dans le dernier choix de couleurs"<<endl;
      return 0;
    }



  else  //try the next color for the rightmost candidate whose colors have not all been explored and reset colors for candidates located at its right
    {
      color_choice[i]++;
      for (int j=i+1;j<n;j++)
	color_choice[j]=max(start[j],color_choice[i]);  //when the two positions are dealing with the same block, then you don't want to get all permutations so you don't start from start but from color_choice[i].
      return 1;
    }
}

bool tab_overlap(int *tab1,int *tab2,int size)
{
  for (int i=0;i<size;i++)
    for (int j=0;j<size;j++)
      if (tab1[i]==tab2[j])
	return true;
  return false;
}

/*returns true is the tab is in the list of tabs*/
bool is_tab_in_list(list<int *> *occurrences, int *occurrence,int size)
{
  list<int *>::iterator sol_iter = occurrences->begin();
  while (sol_iter != occurrences->end())
    {
      if (tab_overlap(occurrence, *sol_iter,size))
	return true;
      sol_iter++;
    }
  return false;
}


/*creates a partition of the solutions, every two solutions that share a reaction are placed in the same group*/
void group_by_overlap(list<int *> *occurrences, list<list<int *> *> *occurrences_groups, int motif_size)
{
  list<int *>::iterator sol_iter = occurrences->begin();
  list<list<int *> *>::iterator group_iter;
  bool first_match=0;
  list<int *> *first_matched_group=NULL;
  //for all solutions
  while (sol_iter != occurrences->end())
    {
      first_match=0;
      //for all groups
      group_iter = occurrences_groups->begin();
      while (group_iter != occurrences_groups->end())
	{
	  //1er match, on stocke et les suivants on fusionne
	  //no match on cre un groupe
	  //if ((*group_iter)->contains_similar_solution(*sol_iter))
	  if (is_tab_in_list(*group_iter,*sol_iter,motif_size))
		{
		  //cout<<"cas ou on a trouv une solution similaire"<<endl;
		  if (!first_match)
		    {
		      first_match=true;
		      //(*group_iter)->add(*sol_iter);
		      (*group_iter)->push_back(*sol_iter);
		      first_matched_group = *group_iter;
		    }
		  else
		    {
		      first_matched_group->push_back(*sol_iter);
		      //		      first_matched_group->add_group(*group_iter);
		      //the erase method doesn't delete the object !
		      //we could add this line : 
		      //group_iter->delete_solutions();
		      //(*group_iter)->clear_solutions();
		      //cout<<"je delete un groupe"<<endl;
		      delete(*group_iter);  //we don't delete the solutions inside the group, we just delete the group
		      group_iter=occurrences_groups->erase(group_iter);
		      //cout<<"peut-tre cela va-t-il foirer !"<<endl;
		      group_iter--;
		    }
		}
	  group_iter++;
	}
      //the solution shares no reaction with any other
      if (!first_match)
	{
	  //cout<<"je cre un groupe"<<endl;
	  list<int *> *new_group = new list<int *>;
	  new_group->push_back(*sol_iter);
	  occurrences_groups->push_back(new_group);
	}
      sol_iter++;
    }
}

/*creates a partition of the solutions, every two solutions that share a reaction are placed in the same group, output the number of groups*/
int get_nb_clumps(list<int *> *occurrences, int motif_size)
{
  list<int *>::iterator sol_iter = occurrences->begin();
  list<list<int *> *> *occurrences_groups = new list<list<int *> *>;
  list<list<int *> *>::iterator group_iter;
  bool first_match=0;
  list<int *> *first_matched_group=NULL;
  //for all solutions
  while (sol_iter != occurrences->end())
    {
      first_match=0;
      //for all groups
      group_iter = occurrences_groups->begin();
      while (group_iter != occurrences_groups->end())
	{
	  //1er match, on stocke et les suivants on fusionne
	  //no match on cre un groupe
	  //if ((*group_iter)->contains_similar_solution(*sol_iter))
	  if (is_tab_in_list(*group_iter,*sol_iter,motif_size))
		{
		  //cout<<"cas ou on a trouv une solution similaire"<<endl;
		  if (!first_match)
		    {
		      first_match=true;
		      //(*group_iter)->add(*sol_iter);
		      (*group_iter)->push_back(*sol_iter);
		      first_matched_group = *group_iter;
		    }
		  else
		    {
		      first_matched_group->push_back(*sol_iter);
		      //		      first_matched_group->add_group(*group_iter);
		      //the erase method doesn't delete the object !
		      //we could add this line : 
		      //group_iter->delete_solutions();
		      //(*group_iter)->clear_solutions();
		      //cout<<"je delete un groupe"<<endl;
		      delete(*group_iter);  //we don't delete the solutions inside the group, we just delete the group
		      group_iter=occurrences_groups->erase(group_iter);
		      //cout<<"peut-tre cela va-t-il foirer !"<<endl;
		      group_iter--;
		    }
		}
	  group_iter++;
	}
      //the solution shares no reaction with any other
      if (!first_match)
	{
	  //cout<<"je cre un groupe"<<endl;
	  list<int *> *new_group = new list<int *>;
	  new_group->push_back(*sol_iter);
	  occurrences_groups->push_back(new_group);
	}
      sol_iter++;
    }

  int nb_clumps=(int)occurrences_groups->size();
  
  /*delete intermediate structures*/
  for (list<list<int *> *>::iterator group_iter=occurrences_groups->begin();group_iter!=occurrences_groups->end();group_iter++)
    delete (*group_iter);
  delete occurrences_groups;
  
  return nb_clumps;
}


// /*test if the compound is already in the compound_list*/
// bool in_list(Compound *compound_to_add, list<Compound *> *comp_list)
// {
//   list<Compound *>::iterator it = comp_list->begin();
//   while (it != comp_list->end())
//     {
//       if (compound_to_add == *it)
// 	return true;
//       it++;
//     }
//   return false;
// }

void print_tab(int *tab, int size)
{
  for (int i=0; i<size; i++)
    cout<<tab[i]<<" ";
  cout<<endl;
}

void print_tab(int *tab, int size, std::ostream & output)
{
  for (int i=0; i<size; i++)
    output<<tab[i]<<" ";
  // output<<endl;
}

void print_tab(long int *tab, int size)
{
  for (int i=0; i<size; i++)
    cout<<tab[i]<<" ";
  cout<<endl;
}



void print_tab(double *tab, int size)
{
  for (int i=0; i<size; i++)
    cout<<tab[i]<<" ";
  cout<<endl;
}

void print_tab(float *tab, int size)
{
  for (int i=0; i<size; i++)
    cerr<<tab[i]<<" ";
  cerr<<endl;
}

void print_tab(char **tab, int size)
{
  for (int i=0; i<size; i++)
    cout<<tab[i]<<" ";
  cout<<endl;
}

void print_tab(bool *tab, int size)
{
  for (int i=0; i<size; i++)
    cout<<tab[i]<<" ";
  cout<<endl;
}

void print_mat(int **mat, int nl, int nc)
{
  for (int i=0; i<nl; i++)
    print_tab(mat[i], nc);
  cout<<endl;
}


void print_mat(bool **mat, int nl, int nc)
{
  for (int i=0; i<nl; i++)
    print_tab(mat[i], nc);
  cout<<endl;
}



bool** initialize_matrix(bool **adjacency_matrix, int nb_nodes)
{
  /*memory allocation*/
  adjacency_matrix=new bool *[nb_nodes];
  for (int i=0;i<nb_nodes;i++)
    adjacency_matrix[i]=new bool[nb_nodes];

  /*initialisation (all cells are initialized to 0)*/
  for (int i=0;i<nb_nodes;i++)
    for (int j=0;j<nb_nodes;j++)
      adjacency_matrix[i][j]=0;
  return adjacency_matrix;
}

/*returns the index of str in tab_str*/
int get_index(char *str, char **tab_str, int n)
{
  //cout << "in get index" << endl;
  for (int i=0;i<n;i++)
    {
      //cout<<str<<" "<<tab_str[i]<<endl;
      if (!strcmp(str,tab_str[i]))
	{
	  //cout<<"yes"<<endl;
	return i;
	}
    }
  cerr<<"get_index::the enzyme you want: "<<str<<"catalyses no reaction in this network, sorry !"<<endl;
  return -1;
}

/*returns the index of int in tab_int*/
int get_index(int k, int *tab, int n)
{
  for (int i=0;i<n;i++)
    {
      //cout<<str<<" "<<tab_str[i]<<endl;
      if (tab[i]==k)
	{
	  //cout<<"yes"<<endl;
	  return i;
	}
    }
  cerr<<"get_index::the int you want is not in the tab, sorry !"<<endl;
  return -1;
}


/*returns true if the two tabs are identical*/
bool tab_equal(int *tab1, int *tab2)
{
  for (int i=0;i<4;i++)
    if (tab1[i]!=tab2[i])
      return 0;

  return 1;
}

/*returns the index of int* in tab_int* */
int get_index(int *tab, int **tab_tab, int n)
{
  //cout << "in get index" << endl;
  for (int i=0;i<n;i++)
    {
      //cout<<str<<" "<<tab_str[i]<<endl;
      if (tab_equal(tab,tab_tab[i]))
	{
	  //cout<<"yes"<<endl;
	return i;
	}
    }
  //cerr<<"get_index::the array: "<<tab<<"is not in this tab of arrays !"<<endl;
  return -1;
}



/*FROM NOW ON, THESE FUNCTIONS PERFORM A FAST SEARCH OF ALL OCCURRENCES OF THE MOTIF IN THE GRAPH
(what is different from the first search is that it gives also non-linear motifs and it performs the search on a more compact representation of the graph*/

void first_choice(int *ones, int m)
{
  for (int i=0;i<m;i++)
    ones[i]=i;
}

bool is_last_choice(int *ones, int m, int n)
{
  if (n<m)   //in case we don't have enough candidates
    return 1;
  if (ones[0]==n-m)
    return 1;
  else 
    return 0;
}

int next_choice(int *ones, int m, int n)
{
  /*stop condition : if all m digits are at the right, then stop*/
  if (ones[0]==n-m)
    return 0;

  /*if the last digit is not 1, then move the rightmost digit equal to one to the right*/
  if (ones[m-1]!=n-1)
    {
      ones[m-1]++;
      return 1;
    }

  /*if the last digit is equal to 1, then move to the right the last digit equal to one (and next to a 0) and then get the rightmost digits equal to one next to him*/
  else
   { 
     int i=m-1;
     while (ones[i]-ones[i-1] == 1)  // look for the last digit equal to one (and next to a 0)
       i--;
     ones[i-1]++;
     
     /*get the rightmost digits equal to one next to him*/
     int diff=ones[i]-ones[i-1];
     if (diff!=0)
       for (int j=i;j<m;j++)
	 ones[j]-=diff-1;
     return 1;
   }
}

void first_topology(int *topologies, int size)
{
  topologies[0]=size;
  for (int i=1; i<size+1; i++)
    topologies[i]=0;
}

bool next_topology(int *topologies, int size)
{
  /*stop condition*/
  if (topologies[size-1]==1)
    return 0;

  /*look for the rightmost digit greater than 1 and move it to the right*/
  int i=size-1;
  while (topologies[i]<2)
    i--;
  topologies[i]--;
  topologies[i+1]++;
  return 1;
}



/***********ADDED 21/02/06******************/
/*solves the problem of traversing all topologies of the graph*/
/*mount: generates all subsets of k and stores them in topology_matrix*/
/*for example, topology_matrix contains lines like 0 4 1 0 0 0 and then 0 3 2 0 0 0 and then 0 3 1 1 0 0 ... meaning you should take 0 nodes at distance 0, 3 nodes at distance 1, 1 node at distance 2...*/
void mount (int v[], int n, int i, int **topology_matrix, int t) {
  int j;

  if (n == 0)
    {/*printf("0\n");*/}
  else {
    v[i] = n;
    /* Replace the printing below by the process of v[0..i] */ 
    while (topology_matrix[t][1]!=0)  //find the last line not already filled
      t++;
    
    for (j=1; j<=i; j++){
      topology_matrix[t][j]=v[j];  //copy vector v to the matrix
      //printf("%d ", v[j]);
    }
    //printf("\n");
    
    for (j=n-1; j>0; j--) {
      v[i] = j;
      mount(v,n-j,i+1,topology_matrix,t);
    }
  }
}


void initialize_topology_matrix(int **topology_matrix,int nb_topologies,int motif_size)
{
  for (int i=0;i<nb_topologies;i++)
    for (int j=0;j<motif_size;j++)
      {
	topology_matrix[i][j]=0;
      }
}

void fill_topology_matrix(int **topology_matrix,int motif_size)
{
  int v[motif_size];
  int t=0; //initialize to first topology
  mount(v,motif_size-1,1,topology_matrix,t);
}
/***********END OF ADDED 21/02/06******************/


void first_color_choice(int *color_choice, int n)
{
  for (int i=0;i<n;i++)
    color_choice[i]=0;
}

/*useful to get all combinations of colors for a set of candidates, the number of colors for each candidate is given by color_sizes*/
/*basically, you have to choose one color for each candidate (cartesian product)*/
bool next_color_choice(int *color_choice, int n, int *color_sizes)
{
  // /*get the rightmost candidate whose colors have not all been explored and try a new color, then reset colors for candidates located at its right*/
//   int i=n-1;
//   while (color_choice[i]==color_sizes[i])
//     i--;
//   if (i<0) //stop condition : all choices have been explored
//     {
//       // cout<<"problme je suis dans le dernier choix de couleurs"<<endl;
//       return 0;
//     }

/*get the rightmost candidate whose colors have not all been explored and try a new color, then reset colors for candidates located at its right*/
  int i=n-1;
  bool all_colours_explored=false;
  
  while (color_choice[i]==color_sizes[i])
    {
      if (i!=0)
	i--;
      else
	all_colours_explored=true;break;
    }
  
  if (all_colours_explored) //stop condition : all choices have been explored
    {
      // cout<<"problme je suis dans le dernier choix de couleurs"<<endl;
      return 0;
    }


  else  //try the next color for the rightmost candidate whose colors have not all been explored and reset colors for candidates located at its right
    {
      color_choice[i]++;
      for (int j=i+1;j<n;j++)
	color_choice[j]=0;
      return 1;
    }
}

void print_list(list<int>::iterator begin, list<int>::iterator end)
{
  for (list<int>::iterator it=begin; it!=end; it++)
    cout<<*it<<" ";
  cout<<endl;
}

void print_list_list(list<list<int> >::iterator begin, list<list<int> >::iterator end)
{
  for (list<list<int> >::iterator it=begin; it!=end; it++)
    print_list(it->begin(),it->end());
}

void print_tab_list(list<int> **tab_list, int size)
{
  for (int i=0; i<size; i++)
    {
      print_list(tab_list[i]->begin(), tab_list[i]->end());
    }
}

void print_list(list<int>::iterator begin, list<int>::iterator end, std::ostream & output)
{
  for (list<int>::iterator it=begin; it!=end; it++)
    output<<*it<<" ";
  output<<endl;
}

void print_tab_list(list<int> **tab_list, int size, std::ostream & output)
{
  for (int i=0; i<size; i++)
    {
      print_list(tab_list[i]->begin(), tab_list[i]->end(), output);
    }
}

// void print_tab(bool *tab, int size)
// {
//   for (int i=0; i<size; i++)
//     cout<<tab[i]<<" ";
//   cout<<endl;
// }

// void print_mat(bool **mat, int nl, int nc)
// {
//   for (int i=0; i<nl; i++)
//     print_tab(mat[i], nc);
//   cout<<endl;
// }

// void print_tab(int *tab, int size)
// {
//   for (int i=0; i<size; i++)
//     cout<<tab[i]<<" ";
//   cout<<endl;
// }

// void print_mat(int **mat, int nl, int nc)
// {
//   for (int i=0; i<nl; i++)
//     print_tab(mat[i], nc);
//   cout<<endl;
// }

// void print_list(list<int>::iterator begin, list<int>::iterator end)
// {
//   for (list<int>::iterator it=begin; it!=end;it++)
//     cout<<*it<<" ";
//   cout<<endl;
// }

void print_list_tab(list<int *>::iterator begin, list<int *>::iterator end, int size)
{
  for (list<int *>::iterator it=begin; it!=end; it++)
    print_tab(*it,size);
  cout<<endl;
}

// void print_tab_list(list<int> **tab_list, int size)
// {
//   for (int i=0; i<size; i++)
//     {
//       print_list(tab_list[i]->begin(), tab_list[i]->end());
//     }
// }

// /*print the new representation of the graph*/
// void Reaction_Graph::print_adjacency_list()
// {
//   print_tab_list(adjacency_list,nb_nodes);
// }

void initialize_color_candidates(list<int *> *color_candidates, list<int> *colors, int motif_size)
{
  /*allocate memory if necessary, otherwise we just overwrite what was written before*/
  int diff= colors->size() - color_candidates->size();
  if (diff>0)
    for (int i=0;i<diff;i++)
      color_candidates->push_back(new int[motif_size]);

  /*initialize each tab with a different color*/
  list<int *>::iterator it2=color_candidates->begin();
  for (list<int>::iterator it=colors->begin();it!=colors->end();it++)
    {
      (*it2)[0]=*it;
      it2++;
    }     
}

void fill_color_candidates(int *candidates, int nb_cand, int pos_choices, list<int *> *color_candidates, int* nb_color_candidates, list<int> **colors, int motif_size, int *color_choice, int *color_sizes)
{
  /*calculate the nb of combinations of colors the set of candidates is responsible for (this nb is equal to 1 for a set of unicolor nodes)*/
  /*(note that two combinations might have the same set of colors, could this be avoided ?)*/
  int nb_comb=1;  
  for (int i=0;i<nb_cand;i++)
    nb_comb=nb_comb*colors[candidates[i]]->size();

  
  
  /*memory control : if the new number of color_candidates is larger than the memory already allocated for the structure "candidate_colors", then allocate some more memory*/
  int diff=nb_comb*(*nb_color_candidates) - color_candidates->size();
  if (diff > 0)
    for (int i=0;i<diff;i++)
      color_candidates->push_back(new int[motif_size]);
      
  /*duplication of color sets : if there's a multicolor node in the set, then the nb_comb is greater than one and the list of color candidates ought to be duplicated before being filled with the new candidates*/
  /*precalculation of the line where the new block will start*/
  list<int *>::iterator start_new_block=color_candidates->begin();
  for (int i=0;i<*nb_color_candidates;i++)
    start_new_block++;

  /*duplication*/
  // cout<<"avant duplication, nb_comb ="<<nb_comb<< "et nb_color_candidates = "<<*nb_color_candidates<<endl;
  list<int *>::iterator it,it2;
  it2=start_new_block;
  for (int i=1;i<nb_comb;i++)  //number of times you want to duplicate the first block
    {
      it=color_candidates->begin();
      for (int j=0;j<*nb_color_candidates;j++)  //size of block to duplicate
	{
	  for (int k=0;k<pos_choices;k++)  //nb of positions to duplicate in each line
	    (*it2)[k]=(*it)[k];   //copy block one in new block
	  it++;
	  it2++;
	}
      //the value of it2 here is the start of another block
    }

  // cout<<"aprs duplication, color_candidates = "<<endl;
//   print_list_tab(color_candidates->begin(), color_candidates->end(),motif_size);

  /*initialization of color sizes and color_choice*/
  for (int i=0;i<nb_cand;i++)
    color_sizes[i]=colors[candidates[i]]->size()-1; //the -1 term is for later comparison
  // cout<<"color_sizes ="<<endl;
//   print_tab(color_sizes, nb_cand);
  first_color_choice(color_choice, nb_cand);  //initialize to all 0's


  /*fill with new candidates : first_block gets first combination of colors, second block gets the second combination...*/
  it=color_candidates->begin();
  for (int i=0;i<nb_comb;i++)  //for every combination of colors (i.e. for every block)
    {
      for (int j=0;j<*nb_color_candidates;j++) //for every line in the block
	{
	  // cout<<"pendant fill, color_choice ="<<endl;
// 	  print_tab(color_choice, nb_cand);
	  for (int k=0;k<nb_cand;k++)
	    {
	      list<int>::iterator it2=colors[candidates[k]]->begin();
	      for (int l=0;l<color_choice[k];l++)  //get the right color
		it2++;
	      (*it)[pos_choices+k]=*it2;  //fill every end of line by the appropriate combination of colors from the candidates
	    }
	  it++;
	}
      //it++;
      next_color_choice(color_choice, nb_cand, color_sizes);
    }

  /*update the number of valid lines in color_candidates : nb_color_candidates[j] = nb_color_candidates[j-1] * nb_comb*/
  *(nb_color_candidates+1)= (*(nb_color_candidates)) * nb_comb;
  // cout <<"aprs fill, nb_colors_candidates ="<<*(nb_color_candidates+1)<<endl;
}

/*given a set of colors, this function tells you if these colors match the ones from the motif (the number of repeats is specified in the motif)*/
bool color_test(int *color_choice, int partial_size, int *motif, int motif_size, bool *matched_colors)
{
  // cout<<"je suis dans color test"<<endl;
//   cout<<"je vais tester le choix suivant :"<<endl;
//   print_tab(color_choice, partial_size);
  /*initialize matched_colors*/
  for (int i=0;i<motif_size;i++)
    matched_colors[i]=0;

  /*test color_choice*/
  bool candidate_ok=0;
  for (int i=0;i<partial_size;i++)  //for all candidates
    {
      candidate_ok=0;
      for (int j=0;j<motif_size;j++) //look for a new match in the motif
	{
	  if (color_choice[i]==motif[j] && !matched_colors[j])  //we have a new match
	    {
	      matched_colors[j]=1;
	      candidate_ok=1;
	      break;
	    }
	}
      if (!candidate_ok)
	return 0;
    }
  /*if all candidates are ok, then return 1*/
  return 1;
}


bool partial_test(list<int *> *color_candidates, int partial_size, int *nb_color_candidates, int *motif, int motif_size, bool *matched_colors)
{
  // cout<<"je suis dans partial test"<<endl;
//   cout<<"le nombre de lignes  considrer est :"<<*nb_color_candidates<<endl;
  bool partial_test=0;
  bool line_test=0;
  //int nb_lines_lost=0;
  int nb_good_lines=0;
  list<int *>::iterator it=color_candidates->begin();
  for (int i=0;i<*nb_color_candidates;i++) //for all lines in color_candidates (i.e. all color choices)
    {
      // cout<<"Partial_test : je teste la ligne "<<endl;
//       print_tab(*it,motif_size);
//       cout <<"je vais faire line_test"<<endl;
      line_test=color_test(*it,partial_size,motif, motif_size,matched_colors);
      if (!line_test)  
	{
	  it++;
	}
      else  //when a color choice is good, we push it in front of the list (we don't delete bad color choices in order not to lose time dealing with the memory)
	{
	  color_candidates->push_front(*it);
	  it=color_candidates->erase(it);  //note that the erase method increments the iterator automatically
	  nb_good_lines++;
	  partial_test=true;  //one line_test is enough to give a good partial test
	}
    }
  *nb_color_candidates=nb_good_lines; //update the number of valid color choices in color_candidates
  return partial_test;
}


bool final_test(list<int *> *color_candidates, int partial_size, int nb_color_candidates, int *motif, int motif_size, bool *matched_colors)
{
  list<int *>::iterator it=color_candidates->begin();
  for (int i=0;i<nb_color_candidates;i++) //for all lines in color_candidates (i.e. all color choices)
    {
      if (color_test(*it,partial_size,motif, motif_size,matched_colors))  //when a test is successful, we stop the search and return a positive answer
	return 1;
      it++;
    }
  return 0; //if all tests are successless, we return a negative answer
}

//note that we could check for the color as well
void pick_neighbors(int reaction, list<int> **adjacency_list, bool **picked_eliminated, int **neighbors, int *reactions_picked, int *pos)
{
  int pos0=*pos-1;
  neighbors[0][pos0]=*pos;
  for (list<int>::iterator it =(*adjacency_list[reaction]).begin(); it !=(*adjacency_list[reaction]).end(); it++)
    {
      if (!picked_eliminated[*it][1] && !picked_eliminated[*it][0])  //if the reaction is not eliminated and not picked already, we add it to the neighbors
	{
	  reactions_picked[*pos]=*it;
	  picked_eliminated[*it][0]=1;
	  (*pos)++;
	}
    }
  neighbors[1][pos0]=*pos;  //position of last neighbor of node i
}

void unpick_neighbors(int dist_pos,bool **picked_eliminated, int **neighbors, int *reactions_picked, int *pos)
{
  for (int i=dist_pos;i<*pos;i++)
    {
      picked_eliminated[reactions_picked[i]][0]=0;
      reactions_picked[i]=-1;
      neighbors[0][i]=-1;
      neighbors[1][i]=-1;
    }
  *pos=dist_pos;
}




//////////////


/**TESTEE
 * Print a matrix of integer.
 *
 * @param mat the matrix to print.
 * @param size the size of the matrix.
 *
 */
void printMatrix(int **mat, int size){
  for(int i = 0 ; i<size ; i++){
    cout << endl;
    for(int j = 0 ; j<size ; j++)
      cout << mat[i][j];
  }
  cout << endl;
}

/**TESTEE
 * Print a matrix of boolean.
 *
 * @param mat the matrix to print.
 * @param size the size of the matrix.
 *
 */
void printMatrix(bool **mat, int size){
  for(int i = 0 ; i<size ; i++){
    cout << endl;
    for(int j =  0; j<size ; j++)
      cout << mat[i][j];
  }
  cout << endl;
}


/**
 *
 *
 *
 */
void printList(list<int>**l,int size){
  list<int>::iterator it;
  for(int i = 0 ; i<size; i++){
    for(it = l[i]->begin() ; it != l[i]->end() ; it++){
      cout << *it << " " ;
    }
    cout << endl; 
  }
}

/** TESTEE
 * Deleting Matrix of integer.
 *
 * @param adjacencyListmatrix, the matrix to delete.
 * @param size column and row of the matrix.
 */
void deleteMatrix(int ** matrix, int size){
 for (int i=0; i<size; i++){
    if(matrix[i]==NULL)
      cout<<"matrix delete error"<<endl;
    else
      delete []matrix[i];
  }
  delete []matrix;
}



/** TESTEE
 * Deleting Matrix of character.
 *

 * @param adjacencyListmatrix, the matrix to delete.
 * @param size column and row of the matrix.
 */
void deleteMatrix(char ** matrix, int size){
 for (int i=0; i<size; i++){
    if(matrix[i]==NULL)
      cout<<"matrix delete error"<<endl;
    else
      delete []matrix[i];
  }
  delete []matrix;
}



/** TESTEE
 * Convert the adjacency list to adjacency Matrix
 *
 * @param adjacencyList the adjacency List.
 * @param size number of node.
 * @return the adjacency matrix.
 */
bool** adjacencyListToAdjacencyMatrix(list<int> **adjacencyList, int size)
{
  bool **adjacencyMatrix; 
  list<int>::iterator it,tmp;
  int i;
  
  adjacencyMatrix = new bool*[size];
  for(i=0 ; i<size ;i++){
    adjacencyMatrix[i] = new bool[size];
    for(int j = 0 ;j<size; j++)
      adjacencyMatrix[i][j]=0;
  }
  
  for (i=0; i<size; i++)
    for (it =(*adjacencyList[i]).begin(); it !=(*adjacencyList[i]).end(); it++)
      adjacencyMatrix[i][*it]=1;
  
  return adjacencyMatrix;
}

/**
 * Convert the matrix distance to adjacency list
 *
 * @param m the matrix distance.
 * @param size number of node.
 * @return the adjacency List.
 */
list<int> ** distanceMatrixToAdjacencylist(int ** m, int size){
  list <int>**adjacencyList = new list<int>*[size];
  int i;
  for (i=0;i<size;i++)
      adjacencyList[i] = new list<int>;
  
  for(i = 0; i<size ; i++)
    {
      for(int j = i ; j < size ; j++)
	{
	  if (m[i][j])
	    {
	      adjacencyList[i]->push_back(j);
	      adjacencyList[j]->push_back(i);
	    }
	}
    }
  return adjacencyList;
}

/**
 * Convert the adjacency matrix to distance matrix
 *
 * @param m the adjacency matrix.
 * @param size row and column matrix.
 * @return the distance matrix.
 */
int** boolMtrixToIntMatrix(bool** m, int size){
  int **adjacencyMatrix; 
  int i;
  
  adjacencyMatrix = new int*[size];
  for(i=0 ; i<size ;i++){
    adjacencyMatrix[i] = new int[size];
    for(int j = 0 ;j<size; j++)
	if(m[i][j])
      adjacencyMatrix[i][j]=1;
	else	
	 adjacencyMatrix[i][j]=0;
  }
  return adjacencyMatrix;

}

/**TESTEE
 * we suppose the next value for the given matrix :
 * #0 for no neighbore
 * #1 for a neighbore
 *
 * Floyd algorithm wich limits lenght between vertex by a given value "p" (it will be the local gap -1)
 * so the result with given argument "p" represent the p-transitive closure.
 *
 * complexity : n^3 with n number of vertex. 
 *
 * @param p value for the p-transitive closure.
 * @param data adjacency matrix.
 * @param n the length of column and row.
 * @return new adjacency matrix with p-transitive closing.
 */
#define NoConnection 0
int** floyd(int p, bool** adjacencyMatrix, int n){// adjacencyMatrix should be "const"
  
  int i,j,k;
  int dik,dkj;
  int distance, max=p+1;
  int** closure;

  closure = new int*[n];
  for(i=0 ; i<n ;i++){
    closure[i] = new int[n];
  }
  
  for (i=0; i<n; i++) {
    for (j=0; j<n; j++) {
      closure[i][j] = adjacencyMatrix[i][j];
    }
  }

  for (k=0; k<n; k++) {
    for (i=0; i<n; i++) {
      for (j=0; j<n; j++) {
	dik = closure[i][k];
	dkj = closure[k][j];

	if (dik == NoConnection || dkj == NoConnection) 
	  continue;
	distance = dik + dkj;

	if(distance > max)
	  continue;

	if(closure[i][j] == NoConnection && i!=j)
	  closure[i][j] = distance;
	else if (distance < closure[i][j]) {
	  closure[i][j] = distance;
	}
      }
    }
  }
  return closure;
}


/*returns true if one of the element of the list is in the tab*/
bool is_list_in_tab(list<int> *l,int *tab,int n)
{
  for (list<int>::iterator it=l->begin();it!=l->end();it++){
    //cout << "list color : "<< *it<<endl;
    for (int i=0;i<n;i++){
      if(*it == tab[i])
	return 1;
    }
  }
  return 0;
}


/*returns true if the query is in the tab*/
bool is_in_tab(int query,int *tab,int n)
{
  for (int i=0;i<n;i++)
    {
      if(query == tab[i])
	return 1;
    }
  return 0;
}


